@@ -5,6 +5,8 @@
 extern "C" {
 #endif
 
+struct membuffer;
+
 #define SCALE_SCREEN 1.0
 #define SCALE_PRINT (1.0 / get_screen_dpi())
 
@@ -17,47 +19,48 @@ struct plot_info {
 	int maxtime;
 	int meandepth, maxdepth;
 	int minpressure, maxpressure;
+	int minhr, maxhr;
 	int mintemp, maxtemp;
+	enum {AIR, NITROX, TRIMIX} dive_type;
 	double endtempcoord;
 	double maxpp;
 	bool has_ndl;
 	struct plot_data *entry;
 };
 
-/*
- * handy datastructure to keep all of our scaling data in one place
- */
-struct graphics_context {
-	int printer;
-	double maxx, maxy;
-	double leftx, rightx;
-	double topy, bottomy;
-	unsigned int maxtime;
-	struct plot_info pi;
-};
+typedef enum {
+	SC_SCREEN,
+	SC_PRINT
+} scale_mode_t;
 
-typedef enum { SC_SCREEN, SC_PRINT } scale_mode_t;
-
-extern struct divecomputer *select_dc(struct divecomputer *main);
-extern void get_plot_details(struct graphics_context *gc, int time, char *buf, int bufsize);
+extern struct divecomputer *select_dc(struct dive *);
 
 struct options {
-	enum { PRETTY, TABLE, TWOPERPAGE } type;
+	enum {
+		PRETTY,
+		TABLE,
+		TWOPERPAGE,
+		ONEPERPAGE
+	} type;
 	int print_selected;
 	int color_selected;
 	bool notes_up;
-	int profile_height, notes_height, tanks_height;
 };
 
-extern char zoomed_plot, dc_number;
+extern unsigned int dc_number;
 
 extern unsigned int amount_selected;
 
 extern int is_default_dive_computer_device(const char *);
 extern int is_default_dive_computer(const char *, const char *);
 
-typedef void (*device_callback_t) (const char *name, void *userdata);
-int enumerate_devices (device_callback_t callback, void *userdata);
+typedef void (*device_callback_t)(const char *name, void *userdata);
+
+#define DC_TYPE_SERIAL 1
+#define DC_TYPE_UEMIS 2
+#define DC_TYPE_OTHER 3
+
+int enumerate_devices(device_callback_t callback, void *userdata, int dc_type);
 
 extern const char *default_dive_computer_vendor;
 extern const char *default_dive_computer_product;
@@ -67,4 +70,4 @@ extern const char *default_dive_computer_device;
 }
 #endif
 
-#endif
+#endif // DISPLAY_H
