@@ -42,10 +42,11 @@
 #include "dive.h"
 #include "divelist.h"
 #include "display.h"
+#include "planner.h"
 
-static short dive_list_changed = FALSE;
+static short dive_list_changed = false;
 
-short autogroup = FALSE;
+short autogroup = false;
 
 dive_trip_t *dive_trip_list;
 
@@ -113,49 +114,13 @@ void get_dive_gas(struct dive *dive, int *o2_p, int *he_p, int *o2low_p)
 
 	for (i = 0; i < MAX_CYLINDERS; i++) {
 		cylinder_t *cyl = dive->cylinder + i;
-		struct gasmix *mix = &cyl->gasmix;
-		int o2 = mix->o2.permille;
-		int he = mix->he.permille;
-		struct divecomputer *dc = &dive->dc;
-		int used = 0;
-		int first_gas_explicit = 0;
+		int o2 = get_o2(&cyl->gasmix);
+		int he = get_he(&cyl->gasmix);
 
-		while (dc) {
-			struct event *event = dc->events;
-			while (event) {
-				if (event->value) {
-					if (event->name && !strcmp(event->name, "gaschange")) {
-						unsigned int event_he = event->value >> 16;
-						unsigned int event_o2 = event->value & 0xffff;
-
-						if (event->time.seconds < 30)
-							first_gas_explicit = 1;
-						if (is_air(o2, he)) {
-							if (is_air(event_o2 * 10, event_he * 10))
-								used = 1;
-						}
-						else {
-							if (he == event_he*10 && o2 == event_o2*10)
-								used = 1;
-						}
-					}
-				}
-				event = event->next;
-			}
-			dc = dc->next;
-		}
-
-		/* Unless explicity set, the first gas to use has index 0 */
-		if (i == 0 && !first_gas_explicit)
-			used = 1;
-
-		if (!used)
+		if (!is_cylinder_used(dive, i))
 			continue;
-
 		if (cylinder_none(cyl))
 			continue;
-		if (!o2)
-			o2 = O2_IN_AIR;
 		if (o2 < mino2)
 			mino2 = o2;
 		if (he > maxhe)
@@ -164,7 +129,7 @@ void get_dive_gas(struct dive *dive, int *o2_p, int *he_p, int *o2low_p)
 			continue;
 		if (o2 <= maxo2)
 			continue;
-newmax:
+	newmax:
 		maxhe = he;
 		maxo2 = o2;
 	}
@@ -181,27 +146,16 @@ int total_weight(struct dive *dive)
 	int i, total_grams = 0;
 
 	if (dive)
-		for (i=0; i< MAX_WEIGHTSYSTEMS; i++)
+		for (i = 0; i < MAX_WEIGHTSYSTEMS; i++)
 			total_grams += dive->weightsystem[i].weight.grams;
 	return total_grams;
 }
 
 static int active_o2(struct dive *dive, struct divecomputer *dc, duration_t time)
 {
-	int o2permille = dive->cylinder[0].gasmix.o2.permille;
-	struct event *event = dc->events;
-
-	if (!o2permille)
-		o2permille = O2_IN_AIR;
-
-	for (event = dc->events; event; event = event->next) {
-		if (event->time.seconds > time.seconds)
-			break;
-		if (strcmp(event->name, "gaschange"))
-			continue;
-		o2permille = 10*(event->value & 0xffff);
-	}
-	return o2permille;
+	struct gasmix gas;
+	get_gas_at_time(dive, dc, time, &gas);
+	return get_o2(&gas);
 }
 
 /* calculate OTU for a dive - this only takes the first divecomputer into account */
@@ -217,31 +171,31 @@ static int calculate_otu(struct dive *dive)
 		struct sample *sample = dc->sample + i;
 		struct sample *psample = sample - 1;
 		t = sample->time.seconds - psample->time.seconds;
-		if (sample->po2) {
-			po2 = sample->po2;
+		if (sample->po2.mbar) {
+			po2 = sample->po2.mbar;
 		} else {
 			int o2 = active_o2(dive, dc, sample->time);
-			po2 = o2 / 1000.0 * depth_to_mbar(sample->depth.mm, dive);
+			po2 = o2 * depth_to_atm(sample->depth.mm, dive);
 		}
 		if (po2 >= 500)
 			otu += pow((po2 - 500) / 1000.0, 0.83) * t / 30.0;
 	}
-	return otu + 0.5;
+	return rint(otu);
 }
 /* calculate CNS for a dive - this only takes the first divecomputer into account */
 int const cns_table[][3] = {
-/* po2, Maximum Single Exposure, Maximum 24 hour Exposure */
-	{1600,  45 * 60, 150 * 60},
-	{1500, 120 * 60, 180 * 60},
-	{1400, 150 * 60, 180 * 60},
-	{1300, 180 * 60, 210 * 60},
-	{1200, 210 * 60, 240 * 60},
-	{1100, 240 * 60, 270 * 60},
-	{1000, 300 * 60, 300 * 60},
-	{ 900, 360 * 60, 360 * 60},
-	{ 800, 450 * 60, 450 * 60},
-	{ 700, 570 * 60, 570 * 60},
-	{ 600, 720 * 60, 720 * 60}
+	/* po2, Maximum Single Exposure, Maximum 24 hour Exposure */
+	{ 1600, 45 * 60, 150 * 60 },
+	{ 1500, 120 * 60, 180 * 60 },
+	{ 1400, 150 * 60, 180 * 60 },
+	{ 1300, 180 * 60, 210 * 60 },
+	{ 1200, 210 * 60, 240 * 60 },
+	{ 1100, 240 * 60, 270 * 60 },
+	{ 1000, 300 * 60, 300 * 60 },
+	{ 900, 360 * 60, 360 * 60 },
+	{ 800, 450 * 60, 450 * 60 },
+	{ 700, 570 * 60, 570 * 60 },
+	{ 600, 720 * 60, 720 * 60 }
 };
 
 /* this only gets called if dive->maxcns == 0 which means we know that
@@ -265,12 +219,12 @@ static int calculate_cns(struct dive *dive)
 	 */
 	divenr = get_divenr(dive);
 	if (divenr) {
-		prev_dive = get_dive(divenr -1 );
+		prev_dive = get_dive(divenr - 1);
 		if (prev_dive) {
 			endtime = prev_dive->when + prev_dive->duration.seconds;
 			if (dive->when < (endtime + 3600 * 12)) {
 				cns = calculate_cns(prev_dive);
-				cns = cns * 1/pow(2, (dive->when - endtime) / (90.0 * 60.0));
+				cns = cns * 1 / pow(2, (dive->when - endtime) / (90.0 * 60.0));
 			}
 		}
 	}
@@ -281,18 +235,21 @@ static int calculate_cns(struct dive *dive)
 		struct sample *sample = dc->sample + i;
 		struct sample *psample = sample - 1;
 		t = sample->time.seconds - psample->time.seconds;
-		if (sample->po2) {
-			po2 = sample->po2;
+		if (sample->po2.mbar) {
+			po2 = sample->po2.mbar;
 		} else {
 			int o2 = active_o2(dive, dc, sample->time);
-			po2 = o2 / 1000.0 * depth_to_mbar(sample->depth.mm, dive);
+			po2 = o2 * depth_to_atm(sample->depth.mm, dive);
 		}
+		/* CNS don't increse when below 500 matm */
+		if (po2 < 500)
+			continue;
 		/* Find what table-row we should calculate % for */
-		for (j = 1; j < sizeof(cns_table)/(sizeof(int) * 3); j++)
+		for (j = 1; j < sizeof(cns_table) / (sizeof(int) * 3); j++)
 			if (po2 > cns_table[j][0])
 				break;
 		j--;
-		cns += ((double)t)/((double)cns_table[j][1]) * 100;
+		cns += ((double)t) / ((double)cns_table[j][1]) * 100;
 	}
 	/* save calculated cns in dive struct */
 	dive->cns = cns;
@@ -312,6 +269,8 @@ static double calculate_airuse(struct dive *dive)
 
 		start = cyl->start.mbar ? cyl->start : cyl->sample_start;
 		end = cyl->end.mbar ? cyl->end : cyl->sample_end;
+		if (!end.mbar || start.mbar <= end.mbar)
+			continue;
 
 		airuse += gas_volume(cyl, start) - gas_volume(cyl, end);
 	}
@@ -338,7 +297,7 @@ static int calculate_sac(struct dive *dive)
 		return 0;
 
 	/* Mean pressure in ATM (SAC calculations are in atm*l/min) */
-	pressure = (double) depth_to_mbar(meandepth, dive) / SURFACE_PRESSURE;
+	pressure = depth_to_atm(meandepth, dive);
 	sac = airuse / pressure * 60 / duration;
 
 	/* milliliters per minute.. */
@@ -362,8 +321,8 @@ static void add_dive_to_deco(struct dive *dive)
 
 		for (j = t0; j < t1; j++) {
 			int depth = interpolate(psample->depth.mm, sample->depth.mm, j - t0, t1 - t0);
-			(void) add_segment(depth_to_mbar(depth, dive) / 1000.0,
-					   &dive->cylinder[sample->sensor].gasmix, 1, sample->po2, dive);
+			(void)add_segment(depth_to_mbar(depth, dive) / 1000.0,
+					  &dive->cylinder[sample->sensor].gasmix, 1, sample->po2.mbar, dive);
 		}
 	}
 }
@@ -372,13 +331,16 @@ int get_divenr(struct dive *dive)
 {
 	int i;
 	struct dive *d;
-	for_each_dive(i, d)
-		if (d == dive)
-			return i;
+	// tempting as it may be, don't die when called with dive=NULL
+	if (dive)
+		for_each_dive(i, d) {
+			if (d->id == dive->id) // don't compare pointers, we could be passing in a copy of the dive
+				return i;
+		}
 	return -1;
 }
 
-static struct gasmix air = { .o2.permille = O2_IN_AIR };
+static struct gasmix air = { .o2.permille = O2_IN_AIR, .he.permille = 0 };
 
 /* take into account previous dives until there is a 48h gap between dives */
 double init_decompression(struct dive *dive)
@@ -386,17 +348,24 @@ double init_decompression(struct dive *dive)
 	int i, divenr = -1;
 	unsigned int surface_time;
 	timestamp_t when, lasttime = 0;
-	bool deco_init = FALSE;
+	bool deco_init = false;
 	double tissue_tolerance, surface_pressure;
-	tissue_tolerance = surface_pressure = get_surface_pressure_in_mbar(dive, TRUE) / 1000.0;
 
 	if (!dive)
 		return 0.0;
+
+	tissue_tolerance = surface_pressure = get_surface_pressure_in_mbar(dive, true) / 1000.0;
 	divenr = get_divenr(dive);
 	when = dive->when;
 	i = divenr;
-	while (i && --i) {
-		struct dive* pdive = get_dive(i);
+	if (i < 0) {
+		i = dive_table.nr - 1;
+		while (i >= 0 && get_dive(i)->when > when)
+			--i;
+		i++;
+	}
+	while (i--) {
+		struct dive *pdive = get_dive(i);
 		/* we don't want to mix dives from different trips as we keep looking
 		 * for how far back we need to go */
 		if (dive->divetrip && pdive->divetrip != dive->divetrip)
@@ -406,15 +375,15 @@ double init_decompression(struct dive *dive)
 		when = pdive->when;
 		lasttime = when + pdive->duration.seconds;
 	}
-	while (++i < divenr) {
-		struct dive* pdive = get_dive(i);
+	while (++i < (divenr >= 0 ? divenr : dive_table.nr)) {
+		struct dive *pdive = get_dive(i);
 		/* again skip dives from different trips */
 		if (dive->divetrip && dive->divetrip != pdive->divetrip)
 			continue;
-		surface_pressure = get_surface_pressure_in_mbar(pdive, TRUE) / 1000.0;
+		surface_pressure = get_surface_pressure_in_mbar(pdive, true) / 1000.0;
 		if (!deco_init) {
 			clear_deco(surface_pressure);
-			deco_init = TRUE;
+			deco_init = true;
 #if DECO_CALC_DEBUG & 2
 			dump_tissues();
 #endif
@@ -429,7 +398,7 @@ double init_decompression(struct dive *dive)
 			lasttime = pdive->when + pdive->duration.seconds;
 			tissue_tolerance = add_segment(surface_pressure, &air, surface_time, 0, dive);
 #if DECO_CALC_DEBUG & 2
-			printf("after surface intervall of %d:%02u\n", FRACTION(surface_time,60));
+			printf("after surface intervall of %d:%02u\n", FRACTION(surface_time, 60));
 			dump_tissues();
 #endif
 		}
@@ -437,15 +406,15 @@ double init_decompression(struct dive *dive)
 	/* add the final surface time */
 	if (lasttime && dive->when > lasttime) {
 		surface_time = dive->when - lasttime;
-		surface_pressure = get_surface_pressure_in_mbar(dive, TRUE) / 1000.0;
+		surface_pressure = get_surface_pressure_in_mbar(dive, true) / 1000.0;
 		tissue_tolerance = add_segment(surface_pressure, &air, surface_time, 0, dive);
 #if DECO_CALC_DEBUG & 2
-		printf("after surface intervall of %d:%02u\n", FRACTION(surface_time,60));
+		printf("after surface intervall of %d:%02u\n", FRACTION(surface_time, 60));
 		dump_tissues();
 #endif
 	}
 	if (!deco_init) {
-		surface_pressure = get_surface_pressure_in_mbar(dive, TRUE) / 1000.0;
+		surface_pressure = get_surface_pressure_in_mbar(dive, true) / 1000.0;
 		clear_deco(surface_pressure);
 #if DECO_CALC_DEBUG & 2
 		printf("no previous dive\n");
@@ -488,7 +457,7 @@ char *get_nitrox_string(struct dive *dive)
 			else
 				snprintf(buffer, MAX_NITROX_STRING, "%d" UTF8_ELLIPSIS "%d", o2low, o2);
 		else
-			strcpy(buffer, translate("gettextFromC","air"));
+			strcpy(buffer, translate("gettextFromC", "air"));
 	}
 	return buffer;
 }
@@ -500,7 +469,7 @@ char *get_nitrox_string(struct dive *dive)
 void dump_trip_list(void)
 {
 	dive_trip_t *trip;
-	int i=0;
+	int i = 0;
 	timestamp_t last_time = 0;
 
 	for (trip = dive_trip_list; trip; trip = trip->next) {
@@ -509,10 +478,10 @@ void dump_trip_list(void)
 		if (trip->when < last_time)
 			printf("\n\ndive_trip_list OUT OF ORDER!!!\n\n\n");
 		printf("%s trip %d to \"%s\" on %04u-%02u-%02u %02u:%02u:%02u (%d dives - %p)\n",
-			trip->autogen ? "autogen " : "",
-			++i, trip->location,
-			tm.tm_year + 1900, tm.tm_mon+1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec,
-			trip->nrdives, trip);
+		       trip->autogen ? "autogen " : "",
+		       ++i, trip->location,
+		       tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec,
+		       trip->nrdives, trip);
 		last_time = trip->when;
 	}
 	printf("-----\n");
@@ -537,9 +506,9 @@ dive_trip_t *find_matching_trip(timestamp_t when)
 		struct tm tm;
 		utc_mkdate(trip->when, &tm);
 		printf("found trip %p @ %04d-%02d-%02d %02d:%02d:%02d\n",
-			trip,
-			tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday,
-			tm.tm_hour, tm.tm_min, tm.tm_sec);
+		       trip,
+		       tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,
+		       tm.tm_hour, tm.tm_min, tm.tm_sec);
 	}
 #endif
 	return trip;
@@ -560,9 +529,9 @@ void insert_trip(dive_trip_t **dive_trip_p)
 		p = &trip->next;
 
 	if (trip && trip->when == dive_trip->when) {
-		if (! trip->location)
+		if (!trip->location)
 			trip->location = dive_trip->location;
-		if (! trip->notes)
+		if (!trip->notes)
 			trip->notes = dive_trip->notes;
 		divep = dive_trip->dives;
 		while (divep) {
@@ -596,10 +565,8 @@ static void delete_trip(dive_trip_t *trip)
 	}
 
 	/* .. and free it */
-	if (trip->location)
-		free(trip->location);
-	if (trip->notes)
-		free(trip->notes);
+	free(trip->location);
+	free(trip->notes);
 	free(trip);
 }
 
@@ -615,6 +582,44 @@ void find_new_trip_start_time(dive_trip_t *trip)
 	trip->when = when;
 }
 
+/* check if we have a trip right before / after this dive */
+bool is_trip_before_after(struct dive *dive, bool before)
+{
+	int idx = get_idx_by_uniq_id(dive->id);
+	if (before) {
+		if (idx > 0 && get_dive(idx - 1)->divetrip)
+			return true;
+	} else {
+		if (idx < dive_table.nr - 1 && get_dive(idx + 1)->divetrip)
+			return true;
+	}
+	return false;
+}
+
+struct dive *first_selected_dive()
+{
+	int idx;
+	struct dive *d;
+
+	for_each_dive (idx, d) {
+		if (d->selected)
+			return d;
+	}
+	return NULL;
+}
+
+struct dive *last_selected_dive()
+{
+	int idx;
+	struct dive *d, *ret = NULL;
+
+	for_each_dive (idx, d) {
+		if (d->selected)
+			ret = d;
+	}
+	return ret;
+}
+
 void remove_dive_from_trip(struct dive *dive, short was_autogen)
 {
 	struct dive *next, **pprev;
@@ -647,7 +652,7 @@ void add_dive_to_trip(struct dive *dive, dive_trip_t *trip)
 	if (dive->divetrip == trip)
 		return;
 	assert(trip->when);
-	remove_dive_from_trip(dive, FALSE);
+	remove_dive_from_trip(dive, false);
 	trip->nrdives++;
 	dive->divetrip = trip;
 	dive->tripflag = ASSIGNED_TRIP;
@@ -665,7 +670,7 @@ void add_dive_to_trip(struct dive *dive, dive_trip_t *trip)
 
 dive_trip_t *create_and_hookup_trip_from_dive(struct dive *dive)
 {
-	dive_trip_t *dive_trip = calloc(sizeof(dive_trip_t),1);
+	dive_trip_t *dive_trip = calloc(1, sizeof(dive_trip_t));
 	dive_trip->when = dive->when;
 	if (dive->location)
 		dive_trip->location = strdup(dive->location);
@@ -725,29 +730,20 @@ void delete_single_dive(int idx)
 	struct dive *dive = get_dive(idx);
 	if (!dive)
 		return; /* this should never happen */
-	remove_dive_from_trip(dive, FALSE);
+	remove_dive_from_trip(dive, false);
 	if (dive->selected)
 		deselect_dive(idx);
 	for (i = idx; i < dive_table.nr - 1; i++)
-		dive_table.dives[i] = dive_table.dives[i+1];
+		dive_table.dives[i] = dive_table.dives[i + 1];
 	dive_table.dives[--dive_table.nr] = NULL;
 	/* free all allocations */
 	free(dive->dc.sample);
-	if (dive->location)
-		free((void *)dive->location);
-	if (dive->notes)
-		free((void *)dive->notes);
-	if (dive->divemaster)
-		free((void *)dive->divemaster);
-	if (dive->buddy)
-		free((void *)dive->buddy);
-	if (dive->suit)
-		free((void *)dive->suit);
-	if (dive->tag_list) {
-		taglist_clear(dive->tag_list);
-		/* Remove head of list */
-		free((void *)dive->tag_list);
-	}
+	free((void *)dive->location);
+	free((void *)dive->notes);
+	free((void *)dive->divemaster);
+	free((void *)dive->buddy);
+	free((void *)dive->suit);
+	taglist_free(dive->tag_list);
 	free(dive);
 }
 
@@ -757,7 +753,7 @@ void add_single_dive(int idx, struct dive *dive)
 	dive_table.nr++;
 	if (dive->selected)
 		amount_selected++;
-	for (i = idx; i < dive_table.nr ; i++) {
+	for (i = idx; i < dive_table.nr; i++) {
 		struct dive *tmp = dive_table.dives[i];
 		dive_table.dives[i] = dive;
 		dive = tmp;
@@ -768,21 +764,21 @@ bool consecutive_selected()
 {
 	struct dive *d;
 	int i;
-	bool consecutive = TRUE;
-	bool firstfound = FALSE;
-	bool lastfound = FALSE;
+	bool consecutive = true;
+	bool firstfound = false;
+	bool lastfound = false;
 
 	if (amount_selected == 0 || amount_selected == 1)
-		return TRUE;
+		return true;
 
 	for_each_dive(i, d) {
 		if (d->selected) {
 			if (!firstfound)
-				firstfound = TRUE;
+				firstfound = true;
 			else if (lastfound)
-				consecutive = FALSE;
+				consecutive = false;
 		} else if (firstfound) {
-			lastfound = TRUE;
+			lastfound = true;
 		}
 	}
 	return consecutive;
@@ -791,20 +787,27 @@ bool consecutive_selected()
 struct dive *merge_two_dives(struct dive *a, struct dive *b)
 {
 	struct dive *res;
-	int i,j;
+	int i, j;
+	int id;
 
 	if (!a || !b)
 		return NULL;
+
+	id = a->id;
 	i = get_divenr(a);
 	j = get_divenr(b);
-	res = merge_dives(a, b, b->when - a->when, FALSE);
+	res = merge_dives(a, b, b->when - a->when, false);
 	if (!res)
 		return NULL;
 
 	add_single_dive(i, res);
-	delete_single_dive(i+1);
+	delete_single_dive(i + 1);
 	delete_single_dive(j);
-	mark_divelist_changed(TRUE);
+	// now make sure that we keep the id of the first dive.
+	// why?
+	// because this way one of the previously selected ids is still around
+	res->id = id;
+	mark_divelist_changed(true);
 	return res;
 }
 
@@ -847,6 +850,43 @@ void deselect_dive(int idx)
 	}
 }
 
+void deselect_dives_in_trip(struct dive_trip *trip)
+{
+	struct dive *dive;
+	if (!trip)
+		return;
+	for (dive = trip->dives; dive; dive = dive->next)
+		deselect_dive(get_divenr(dive));
+}
+
+void select_dives_in_trip(struct dive_trip *trip)
+{
+	struct dive *dive;
+	if (!trip)
+		return;
+	for (dive = trip->dives; dive; dive = dive->next)
+		select_dive(get_divenr(dive));
+}
+
+/* This only gets called with non-NULL trips.
+ * It does not combine notes or location, just picks the first one
+ * (or the second one if the first one is empty */
+void combine_trips(struct dive_trip *trip_a, struct dive_trip *trip_b)
+{
+	if (same_string(trip_a->location, "") && trip_b->location) {
+		free(trip_a->location);
+		trip_a->location = strdup(trip_b->location);
+	}
+	if (same_string(trip_a->notes, "") && trip_b->notes) {
+		free(trip_a->notes);
+		trip_a->notes = strdup(trip_b->notes);
+	}
+	/* this also removes the dives from trip_b and eventually
+	 * calls delete_trip(trip_b) when the last dive has been moved */
+	while (trip_b->dives)
+		add_dive_to_trip(trip_b->dives, trip_a);
+}
+
 void mark_divelist_changed(int changed)
 {
 	dive_list_changed = changed;
@@ -866,7 +906,7 @@ void remove_autogen_trips()
 		dive_trip_t *trip = dive->divetrip;
 
 		if (trip && trip->autogen)
-			remove_dive_from_trip(dive, TRUE);
+			remove_dive_from_trip(dive, true);
 	}
 }
 
@@ -904,7 +944,7 @@ static void try_to_renumber(struct dive *last, int preexisting)
 	 * we're going to expect the user to do a manual
 	 * renumbering.
 	 */
-	if (preexisting && get_dive(preexisting-1) != last)
+	if (preexisting && get_dive(preexisting - 1) != last)
 		return;
 
 	/*
@@ -946,16 +986,20 @@ void process_dives(bool is_imported, bool prefer_imported)
 		for (i = preexisting; i < dive_table.nr; i++)
 			set_dc_nickname(dive_table.dives[i]);
 
+	for (i = preexisting; i < dive_table.nr; i++)
+		dive_table.dives[i]->downloaded = true;
+
 	/* This does the right thing for -1: NULL */
-	last = get_dive(preexisting-1);
+	last = get_dive(preexisting - 1);
 
 	sort_table(&dive_table);
 
 	for (i = 1; i < dive_table.nr; i++) {
-		struct dive **pp = &dive_table.dives[i-1];
+		struct dive **pp = &dive_table.dives[i - 1];
 		struct dive *prev = pp[0];
 		struct dive *dive = pp[1];
 		struct dive *merged;
+		int id;
 
 		/* only try to merge overlapping dives - or if one of the dives has
 		 * zero duration (that might be a gps marker from the webservice) */
@@ -967,6 +1011,9 @@ void process_dives(bool is_imported, bool prefer_imported)
 		if (!merged)
 			continue;
 
+		// remember the earlier dive's id
+		id = prev->id;
+
 		/* careful - we might free the dive that last points to. Oops... */
 		if (last == prev || last == dive)
 			last = merged;
@@ -974,12 +1021,14 @@ void process_dives(bool is_imported, bool prefer_imported)
 		/* Redo the new 'i'th dive */
 		i--;
 		add_single_dive(i, merged);
-		delete_single_dive(i+1);
-		delete_single_dive(i+1);
+		delete_single_dive(i + 1);
+		delete_single_dive(i + 1);
+		// keep the id or the first dive for the merged dive
+		merged->id = id;
 	}
 	/* make sure no dives are still marked as downloaded */
 	for (i = 1; i < dive_table.nr; i++)
-		dive_table.dives[i]->downloaded = FALSE;
+		dive_table.dives[i]->downloaded = false;
 
 	if (is_imported) {
 		/* If there are dives in the table, are they numbered */
@@ -988,6 +1037,6 @@ void process_dives(bool is_imported, bool prefer_imported)
 
 		/* did we add dives to the dive table? */
 		if (preexisting != dive_table.nr)
-			mark_divelist_changed(TRUE);
+			mark_divelist_changed(true);
 	}
 }
