@@ -11,7 +11,7 @@ extern void update_cylinder_related_info(struct dive *);
 extern void mark_divelist_changed(int);
 extern int unsaved_changes(void);
 extern void remove_autogen_trips(void);
-extern double init_decompression(struct dive * dive);
+extern double init_decompression(struct dive *dive);
 
 /* divelist core logic functions */
 extern void process_dives(bool imported, bool prefer_imported);
@@ -30,7 +30,13 @@ extern struct dive *merge_two_dives(struct dive *a, struct dive *b);
 extern bool consecutive_selected();
 extern void select_dive(int idx);
 extern void deselect_dive(int idx);
-void find_new_trip_start_time(dive_trip_t *trip);
+extern void select_dives_in_trip(struct dive_trip *trip);
+extern void deselect_dives_in_trip(struct dive_trip *trip);
+extern void combine_trips(struct dive_trip *trip_a, struct dive_trip *trip_b);
+extern void find_new_trip_start_time(dive_trip_t *trip);
+extern struct dive *first_selected_dive();
+extern struct dive *last_selected_dive();
+extern bool is_trip_before_after(struct dive *dive, bool before);
 
 #ifdef DEBUG_TRIP
 extern void dump_selection(void);
@@ -41,4 +47,4 @@ extern void dump_trip_list(void);
 }
 #endif
 
-#endif
+#endif // DIVELIST_H
