@@ -12,10 +12,10 @@
 
 /* Christ. Libdivecomputer has the worst configuration system ever. */
 #ifdef HW_FROG_H
-  #define NOT_FROG , 0
-  #define LIBDIVECOMPUTER_SUPPORTS_FROG
+#define NOT_FROG , 0
+#define LIBDIVECOMPUTER_SUPPORTS_FROG
 #else
-  #define NOT_FROG
+#define NOT_FROG
 #endif
 
 char *dumpfile_name;
@@ -40,16 +40,16 @@ static dc_status_t create_parser(device_data_t *devdata, dc_parser_t **parser)
 struct atomics_gas_info {
 	uint8_t gas_nr;
 	uint8_t po2imit;
-	uint8_t tankspecmethod;	/* 1: CF@psi 2: CF@bar 3: wet vol in deciliter */
+	uint8_t tankspecmethod; /* 1: CF@psi 2: CF@bar 3: wet vol in deciliter */
 	uint8_t gasmixtype;
 	uint8_t fo2;
 	uint8_t fhe;
-	uint16_t startpressure;	/* in psi */
-	uint16_t tanksize;	/* CF or dl */
+	uint16_t startpressure; /* in psi */
+	uint16_t tanksize;      /* CF or dl */
 	uint16_t workingpressure;
 	uint16_t sensorid;
-	uint16_t endpressure;	/* in psi */
-	uint16_t totalconsumption;	/* in liters */
+	uint16_t endpressure;      /* in psi */
+	uint16_t totalconsumption; /* in liters */
 };
 #define COBALT_CFATPSI 1
 #define COBALT_CFATBAR 2
@@ -70,39 +70,40 @@ static bool get_tanksize(device_data_t *devdata, const unsigned char *data, cyli
 		 * right data */
 		if (*(uint32_t *)data != 0xFFFEFFFE) {
 			printf("incorrect header for Atomics dive\n");
-			return FALSE;
+			return false;
 		}
-		atomics_gas_info = (void*)(data + COBALT_HEADER);
+		atomics_gas_info = (void *)(data + COBALT_HEADER);
 		switch (atomics_gas_info[idx].tankspecmethod) {
 		case COBALT_CFATPSI:
 			airvolume = cuft_to_l(atomics_gas_info[idx].tanksize) * 1000.0;
 			mbar = psi_to_mbar(atomics_gas_info[idx].workingpressure);
-			cyl[idx].type.size.mliter = airvolume / bar_to_atm(mbar / 1000.0) + 0.5;
+			cyl[idx].type.size.mliter = rint(airvolume / bar_to_atm(mbar / 1000.0));
 			cyl[idx].type.workingpressure.mbar = mbar;
 			break;
 		case COBALT_CFATBAR:
 			airvolume = cuft_to_l(atomics_gas_info[idx].tanksize) * 1000.0;
 			mbar = atomics_gas_info[idx].workingpressure * 1000;
-			cyl[idx].type.size.mliter = airvolume / bar_to_atm(mbar / 1000.0) + 0.5;
+			cyl[idx].type.size.mliter = rint(airvolume / bar_to_atm(mbar / 1000.0));
 			cyl[idx].type.workingpressure.mbar = mbar;
 			break;
 		case COBALT_WETINDL:
 			cyl[idx].type.size.mliter = atomics_gas_info[idx].tanksize * 100;
 			break;
 		}
-		return TRUE;
+		return true;
 	}
-	return FALSE;
+	return false;
 }
 
 static int parse_gasmixes(device_data_t *devdata, struct dive *dive, dc_parser_t *parser, int ngases,
-	const unsigned char *data)
+			  const unsigned char *data)
 {
+	static bool shown_warning = false;
 	int i;
 
 	for (i = 0; i < ngases; i++) {
 		int rc;
-		dc_gasmix_t gasmix = {0};
+		dc_gasmix_t gasmix = { 0 };
 		int o2, he;
 
 		rc = dc_parser_get_field(parser, DC_FIELD_GASMIX, i, &gasmix);
@@ -112,19 +113,30 @@ static int parse_gasmixes(device_data_t *devdata, struct dive *dive, dc_parser_t
 		if (i >= MAX_CYLINDERS)
 			continue;
 
-		o2 = gasmix.oxygen * 1000 + 0.5;
-		he = gasmix.helium * 1000 + 0.5;
+		o2 = rint(gasmix.oxygen * 1000);
+		he = rint(gasmix.helium * 1000);
 
 		/* Ignore bogus data - libdivecomputer does some crazy stuff */
-		if (o2 + he <= O2_IN_AIR || o2 >= 1000)
+		if (o2 + he <= O2_IN_AIR || o2 > 1000) {
+			if (!shown_warning) {
+				shown_warning = true;
+				report_error("unlikely dive gas data from libdivecomputer: o2 = %d he = %d", o2, he);
+			}
 			o2 = 0;
-		if (he < 0 || he >= 800 || o2+he >= 1000)
+		}
+		if (he < 0 || o2 + he > 1000) {
+			if (!shown_warning) {
+				shown_warning = true;
+				report_error("unlikely dive gas data from libdivecomputer: o2 = %d he = %d", o2, he);
+			}
 			he = 0;
-
+		}
 		dive->cylinder[i].gasmix.o2.permille = o2;
 		dive->cylinder[i].gasmix.he.permille = he;
 
-		if (!get_tanksize(devdata, data, dive->cylinder, i))
+		/* for the first tank, if there is no tanksize available from the
+		 * dive computer, fill in the default tank information (if set) */
+		if (!get_tanksize(devdata, data, dive->cylinder, i) && i == 0)
 			fill_default_cylinder(&dive->cylinder[i]);
 	}
 	return DC_STATUS_SUCCESS;
@@ -136,12 +148,12 @@ static void handle_event(struct divecomputer *dc, struct sample *sample, dc_samp
 	/* we mark these for translation here, but we store the untranslated strings
 	 * and only translate them when they are displayed on screen */
 	static const char *events[] = {
-		QT_TRANSLATE_NOOP("gettextFromC","none"), QT_TRANSLATE_NOOP("gettextFromC","deco stop"), QT_TRANSLATE_NOOP("gettextFromC","rbt"), QT_TRANSLATE_NOOP("gettextFromC","ascent"), QT_TRANSLATE_NOOP("gettextFromC","ceiling"), QT_TRANSLATE_NOOP("gettextFromC","workload"),
-		QT_TRANSLATE_NOOP("gettextFromC","transmitter"), QT_TRANSLATE_NOOP("gettextFromC","violation"), QT_TRANSLATE_NOOP("gettextFromC","bookmark"), QT_TRANSLATE_NOOP("gettextFromC","surface"), QT_TRANSLATE_NOOP("gettextFromC","safety stop"),
-		QT_TRANSLATE_NOOP("gettextFromC","gaschange"), QT_TRANSLATE_NOOP("gettextFromC","safety stop (voluntary)"), QT_TRANSLATE_NOOP("gettextFromC","safety stop (mandatory)"),
-		QT_TRANSLATE_NOOP("gettextFromC","deepstop"), QT_TRANSLATE_NOOP("gettextFromC","ceiling (safety stop)"), QT_TRANSLATE_NOOP3("gettextFromC","below floor","event showing dive is below deco floor and adding deco time"), QT_TRANSLATE_NOOP("gettextFromC","divetime"),
-		QT_TRANSLATE_NOOP("gettextFromC","maxdepth"), QT_TRANSLATE_NOOP("gettextFromC","OLF"), QT_TRANSLATE_NOOP("gettextFromC","PO2"), QT_TRANSLATE_NOOP("gettextFromC","airtime"), QT_TRANSLATE_NOOP("gettextFromC","rgbm"), QT_TRANSLATE_NOOP("gettextFromC","heading"),
-		QT_TRANSLATE_NOOP("gettextFromC","tissue level warning"), QT_TRANSLATE_NOOP("gettextFromC","gaschange"), QT_TRANSLATE_NOOP("gettextFromC","non stop time")
+		QT_TRANSLATE_NOOP("gettextFromC", "none"), QT_TRANSLATE_NOOP("gettextFromC", "deco stop"), QT_TRANSLATE_NOOP("gettextFromC", "rbt"), QT_TRANSLATE_NOOP("gettextFromC", "ascent"), QT_TRANSLATE_NOOP("gettextFromC", "ceiling"), QT_TRANSLATE_NOOP("gettextFromC", "workload"),
+		QT_TRANSLATE_NOOP("gettextFromC", "transmitter"), QT_TRANSLATE_NOOP("gettextFromC", "violation"), QT_TRANSLATE_NOOP("gettextFromC", "bookmark"), QT_TRANSLATE_NOOP("gettextFromC", "surface"), QT_TRANSLATE_NOOP("gettextFromC", "safety stop"),
+		QT_TRANSLATE_NOOP("gettextFromC", "gaschange"), QT_TRANSLATE_NOOP("gettextFromC", "safety stop (voluntary)"), QT_TRANSLATE_NOOP("gettextFromC", "safety stop (mandatory)"),
+		QT_TRANSLATE_NOOP("gettextFromC", "deepstop"), QT_TRANSLATE_NOOP("gettextFromC", "ceiling (safety stop)"), QT_TRANSLATE_NOOP3("gettextFromC", "below floor", "event showing dive is below deco floor and adding deco time"), QT_TRANSLATE_NOOP("gettextFromC", "divetime"),
+		QT_TRANSLATE_NOOP("gettextFromC", "maxdepth"), QT_TRANSLATE_NOOP("gettextFromC", "OLF"), QT_TRANSLATE_NOOP("gettextFromC", "PO2"), QT_TRANSLATE_NOOP("gettextFromC", "airtime"), QT_TRANSLATE_NOOP("gettextFromC", "rgbm"), QT_TRANSLATE_NOOP("gettextFromC", "heading"),
+		QT_TRANSLATE_NOOP("gettextFromC", "tissue level warning"), QT_TRANSLATE_NOOP("gettextFromC", "gaschange"), QT_TRANSLATE_NOOP("gettextFromC", "non stop time")
 	};
 	const int nr_events = sizeof(events) / sizeof(const char *);
 	const char *name;
@@ -158,7 +170,7 @@ static void handle_event(struct divecomputer *dc, struct sample *sample, dc_samp
 	 * Other evens might be more interesting, but for now we just print them out.
 	 */
 	type = value.event.type;
-	name = QT_TRANSLATE_NOOP("gettextFromC","invalid event number");
+	name = QT_TRANSLATE_NOOP("gettextFromC", "invalid event number");
 	if (type < nr_events)
 		name = events[type];
 
@@ -180,12 +192,12 @@ sample_cb(dc_sample_type_t type, dc_sample_value_t value, void *userdata)
 	 * We fill in the "previous" sample - except for DC_SAMPLE_TIME,
 	 * which creates a new one.
 	 */
-	sample = dc->samples ? dc->sample+dc->samples-1 : NULL;
+	sample = dc->samples ? dc->sample + dc->samples - 1 : NULL;
 
 	/*
-	* Ok, sanity check.
-	* If first sample is not a DC_SAMPLE_TIME, Allocate a sample for us
-	*/
+	 * Ok, sanity check.
+	 * If first sample is not a DC_SAMPLE_TIME, Allocate a sample for us
+	 */
 	if (sample == NULL && type != DC_SAMPLE_TIME)
 		sample = prepare_sample(dc);
 
@@ -196,7 +208,7 @@ sample_cb(dc_sample_type_t type, dc_sample_value_t value, void *userdata)
 			sample->ndl.seconds = ndl;
 			sample->stoptime.seconds = stoptime;
 			sample->stopdepth.mm = stopdepth;
-			sample->po2 = po2;
+			sample->po2.mbar = po2;
 			sample->cns = cns;
 		}
 		sample = prepare_sample(dc);
@@ -204,11 +216,11 @@ sample_cb(dc_sample_type_t type, dc_sample_value_t value, void *userdata)
 		finish_sample(dc);
 		break;
 	case DC_SAMPLE_DEPTH:
-		sample->depth.mm = value.depth * 1000 + 0.5;
+		sample->depth.mm = rint(value.depth * 1000);
 		break;
 	case DC_SAMPLE_PRESSURE:
 		sample->sensor = value.pressure.tank;
-		sample->cylinderpressure.mbar = value.pressure.value * 1000 + 0.5;
+		sample->cylinderpressure.mbar = rint(value.pressure.value * 1000);
 		break;
 	case DC_SAMPLE_TEMPERATURE:
 		sample->temperature.mkelvin = C_to_mkelvin(value.temperature);
@@ -220,43 +232,43 @@ sample_cb(dc_sample_type_t type, dc_sample_value_t value, void *userdata)
 		printf("   <rbt>%u</rbt>\n", value.rbt);
 		break;
 	case DC_SAMPLE_HEARTBEAT:
-		printf("   <heartbeat>%u</heartbeat>\n", value.heartbeat);
+		sample->heartbeat = value.heartbeat;
 		break;
 	case DC_SAMPLE_BEARING:
-		printf("   <bearing>%u</bearing>\n", value.bearing);
+		sample->bearing.degrees = value.bearing;
 		break;
 	case DC_SAMPLE_VENDOR:
 		printf("   <vendor time='%u:%02u' type=\"%u\" size=\"%u\">", FRACTION(sample->time.seconds, 60),
-			value.vendor.type, value.vendor.size);
+		       value.vendor.type, value.vendor.size);
 		for (i = 0; i < value.vendor.size; ++i)
-			printf("%02X", ((unsigned char *) value.vendor.data)[i]);
+			printf("%02X", ((unsigned char *)value.vendor.data)[i]);
 		printf("</vendor>\n");
 		break;
 #if DC_VERSION_CHECK(0, 3, 0)
 	case DC_SAMPLE_SETPOINT:
 		/* for us a setpoint means constant pO2 from here */
-		sample->po2 = po2 = value.setpoint * 1000 + 0.5;
+		sample->po2.mbar = po2 = rint(value.setpoint * 1000);
 		break;
 	case DC_SAMPLE_PPO2:
-		sample->po2 = po2 = value.ppo2 * 1000 + 0.5;
+		sample->po2.mbar = po2 = rint(value.ppo2 * 1000);
 		break;
 	case DC_SAMPLE_CNS:
-		sample->cns = cns = value.cns * 100 + 0.5;
+		sample->cns = cns = rint(value.cns * 100);
 		break;
 	case DC_SAMPLE_DECO:
 		if (value.deco.type == DC_DECO_NDL) {
 			sample->ndl.seconds = ndl = value.deco.time;
-			sample->stopdepth.mm = stopdepth = value.deco.depth * 1000.0 + 0.5;
-			sample->in_deco = in_deco = FALSE;
+			sample->stopdepth.mm = stopdepth = rint(value.deco.depth * 1000.0);
+			sample->in_deco = in_deco = false;
 		} else if (value.deco.type == DC_DECO_DECOSTOP ||
 			   value.deco.type == DC_DECO_DEEPSTOP) {
-			sample->in_deco = in_deco = TRUE;
-			sample->stopdepth.mm = stopdepth = value.deco.depth * 1000.0 + 0.5;
+			sample->in_deco = in_deco = true;
+			sample->stopdepth.mm = stopdepth = rint(value.deco.depth * 1000.0);
 			sample->stoptime.seconds = stoptime = value.deco.time;
 			ndl = 0;
 		} else if (value.deco.type == DC_DECO_SAFETYSTOP) {
-			sample->in_deco = in_deco = FALSE;
-			sample->stopdepth.mm = stopdepth = value.deco.depth * 1000.0 + 0.5;
+			sample->in_deco = in_deco = false;
+			sample->stopdepth.mm = stopdepth = rint(value.deco.depth * 1000.0);
 			sample->stoptime.seconds = stoptime = value.deco.time;
 		}
 #endif
@@ -358,9 +370,9 @@ static char *str_printf(const char *fmt, ...)
 	char buf[1024];
 
 	va_start(args, fmt);
-	vsnprintf(buf, sizeof(buf)-1, fmt, args);
+	vsnprintf(buf, sizeof(buf) - 1, fmt, args);
 	va_end(args);
-	buf[sizeof(buf)-1] = 0;
+	buf[sizeof(buf) - 1] = 0;
 	return strdup(buf);
 }
 
@@ -381,48 +393,48 @@ static uint32_t calculate_diveid(const unsigned char *fingerprint, unsigned int
 	return csum[0];
 }
 
+/* returns true if we want libdivecomputer's dc_device_foreach() to continue,
+ *  false otherwise */
 static int dive_cb(const unsigned char *data, unsigned int size,
-	const unsigned char *fingerprint, unsigned int fsize,
-	void *userdata)
+		   const unsigned char *fingerprint, unsigned int fsize,
+		   void *userdata)
 {
 	int rc;
 	dc_parser_t *parser = NULL;
 	device_data_t *devdata = userdata;
-	dc_datetime_t dt = {0};
+	dc_datetime_t dt = { 0 };
 	struct tm tm;
-	struct dive *dive;
+	struct dive *dive = NULL;
 
 	/* reset the deco / ndl data */
 	ndl = stoptime = stopdepth = 0;
-	in_deco = FALSE;
+	in_deco = false;
 
 	rc = create_parser(devdata, &parser);
 	if (rc != DC_STATUS_SUCCESS) {
-		dev_info(devdata, translate("gettextFromC","Unable to create parser for %s %s"), devdata->vendor, devdata->product);
-		return rc;
+		dev_info(devdata, translate("gettextFromC", "Unable to create parser for %s %s"), devdata->vendor, devdata->product);
+		return false;
 	}
 
 	rc = dc_parser_set_data(parser, data, size);
 	if (rc != DC_STATUS_SUCCESS) {
-		dev_info(devdata, translate("gettextFromC","Error registering the data"));
-		dc_parser_destroy(parser);
-		return rc;
+		dev_info(devdata, translate("gettextFromC", "Error registering the data"));
+		goto error_exit;
 	}
 
 	import_dive_number++;
 	dive = alloc_dive();
 	rc = dc_parser_get_datetime(parser, &dt);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-		dev_info(devdata, translate("gettextFromC","Error parsing the datetime"));
-		dc_parser_destroy(parser);
-		return rc;
+		dev_info(devdata, translate("gettextFromC", "Error parsing the datetime"));
+		goto error_exit;
 	}
 	dive->dc.model = strdup(devdata->model);
 	dive->dc.deviceid = devdata->deviceid;
 	dive->dc.diveid = calculate_diveid(fingerprint, fsize);
 
 	tm.tm_year = dt.year;
-	tm.tm_mon = dt.month-1;
+	tm.tm_mon = dt.month - 1;
 	tm.tm_mday = dt.day;
 	tm.tm_hour = dt.hour;
 	tm.tm_min = dt.minute;
@@ -430,14 +442,12 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 	dive->when = dive->dc.when = utc_mktime(&tm);
 
 	// Parse the divetime.
-	dev_info(devdata, translate("gettextFromC","Dive %d: %s %d %04d"), import_dive_number,
-		monthname(tm.tm_mon), tm.tm_mday, year(tm.tm_year));
+	dev_info(devdata, translate("gettextFromC", "Dive %d: %s"), import_dive_number, get_dive_date_c_string(dive->when));
 	unsigned int divetime = 0;
-	rc = dc_parser_get_field (parser, DC_FIELD_DIVETIME, 0, &divetime);
+	rc = dc_parser_get_field(parser, DC_FIELD_DIVETIME, 0, &divetime);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-		dev_info(devdata, translate("gettextFromC","Error parsing the divetime"));
-		dc_parser_destroy(parser);
-		return rc;
+		dev_info(devdata, translate("gettextFromC", "Error parsing the divetime"));
+		goto error_exit;
 	}
 	dive->dc.duration.seconds = divetime;
 
@@ -445,65 +455,59 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 	double maxdepth = 0.0;
 	rc = dc_parser_get_field(parser, DC_FIELD_MAXDEPTH, 0, &maxdepth);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-		dev_info(devdata, translate("gettextFromC","Error parsing the maxdepth"));
-		dc_parser_destroy(parser);
-		return rc;
+		dev_info(devdata, translate("gettextFromC", "Error parsing the maxdepth"));
+		goto error_exit;
 	}
-	dive->dc.maxdepth.mm = maxdepth * 1000 + 0.5;
+	dive->dc.maxdepth.mm = rint(maxdepth * 1000);
 
 	// Parse the gas mixes.
 	unsigned int ngases = 0;
 	rc = dc_parser_get_field(parser, DC_FIELD_GASMIX_COUNT, 0, &ngases);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-		dev_info(devdata, translate("gettextFromC","Error parsing the gas mix count"));
-		dc_parser_destroy(parser);
-		return rc;
+		dev_info(devdata, translate("gettextFromC", "Error parsing the gas mix count"));
+		goto error_exit;
 	}
 
 #if DC_VERSION_CHECK(0, 3, 0)
 	// Check if the libdivecomputer version already supports salinity & atmospheric
 	dc_salinity_t salinity = {
 		.type = DC_WATER_SALT,
-		.density = SEAWATER_SALINITY/10.0
+		.density = SEAWATER_SALINITY / 10.0
 	};
 	rc = dc_parser_get_field(parser, DC_FIELD_SALINITY, 0, &salinity);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-		dev_info(devdata, translate("gettextFromC","Error obtaining water salinity"));
-		dc_parser_destroy(parser);
-		return rc;
+		dev_info(devdata, translate("gettextFromC", "Error obtaining water salinity"));
+		goto error_exit;
 	}
-	dive->dc.salinity = salinity.density * 10.0 + 0.5;
+	dive->dc.salinity = rint(salinity.density * 10.0);
 
-	double surface_pressure = 1.0;
+	double surface_pressure = 0;
 	rc = dc_parser_get_field(parser, DC_FIELD_ATMOSPHERIC, 0, &surface_pressure);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-		dev_info(devdata, translate("gettextFromC","Error obtaining surface pressure"));
-		dc_parser_destroy(parser);
-		return rc;
+		dev_info(devdata, translate("gettextFromC", "Error obtaining surface pressure"));
+		goto error_exit;
 	}
-	dive->dc.surface_pressure.mbar = surface_pressure * 1000.0 + 0.5;
+	dive->dc.surface_pressure.mbar = rint(surface_pressure * 1000.0);
 #endif
 
 	rc = parse_gasmixes(devdata, dive, parser, ngases, data);
 	if (rc != DC_STATUS_SUCCESS) {
-		dev_info(devdata, translate("gettextFromC","Error parsing the gas mix"));
-		dc_parser_destroy(parser);
-		return rc;
+		dev_info(devdata, translate("gettextFromC", "Error parsing the gas mix"));
+		goto error_exit;
 	}
 
 	// Initialize the sample data.
 	rc = parse_samples(devdata, &dive->dc, parser);
 	if (rc != DC_STATUS_SUCCESS) {
-		dev_info(devdata, translate("gettextFromC","Error parsing the samples"));
-		dc_parser_destroy(parser);
-		return rc;
+		dev_info(devdata, translate("gettextFromC", "Error parsing the samples"));
+		goto error_exit;
 	}
 
-	dc_parser_destroy(parser);
-
 	/* If we already saw this dive, abort. */
 	if (!devdata->force_download && find_dive(&dive->dc))
-		return 0;
+		goto error_exit;
+
+	dc_parser_destroy(parser);
 
 	/* Various libdivecomputer interface fixups */
 	if (first_temp_is_air && dive->dc.samples) {
@@ -511,10 +515,16 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 		dive->dc.sample[0].temperature.mkelvin = 0;
 	}
 
-	dive->downloaded = TRUE;
+	dive->downloaded = true;
 	record_dive(dive);
-	mark_divelist_changed(TRUE);
-	return 1;
+	mark_divelist_changed(true);
+	return true;
+
+error_exit:
+	dc_parser_destroy(parser);
+	free(dive);
+	return false;
+
 }
 
 /*
@@ -568,10 +578,13 @@ static unsigned int undo_libdivecomputer_suunto_nr_changes(unsigned int serial)
 		return serial;
 
 	/* Nope, it was converted. */
-	b0 = serial % 100; serial /= 100;
-	b1 = serial % 100; serial /= 100;
-	b2 = serial % 100; serial /= 100;
-	b3 = serial % 100; serial /= 100;
+	b0 = serial % 100;
+	serial /= 100;
+	b1 = serial % 100;
+	serial /= 100;
+	b2 = serial % 100;
+	serial /= 100;
+	b3 = serial % 100;
 
 	serial = b0 + (b1 << 8) + (b2 << 16) + (b3 << 24);
 	return serial;
@@ -589,16 +602,16 @@ static unsigned int fixup_suunto_versions(device_data_t *devdata, const dc_event
 
 	if (serial) {
 		snprintf(serial_nr, sizeof(serial_nr), "%02d%02d%02d%02d",
-			(devinfo->serial >> 24) & 0xff,
-			(devinfo->serial >> 16) & 0xff,
-			(devinfo->serial >> 8)  & 0xff,
-			(devinfo->serial >> 0)  & 0xff);
+			 (devinfo->serial >> 24) & 0xff,
+			 (devinfo->serial >> 16) & 0xff,
+			 (devinfo->serial >> 8) & 0xff,
+			 (devinfo->serial >> 0) & 0xff);
 	}
 	if (devinfo->firmware) {
 		snprintf(firmware, sizeof(firmware), "%d.%d.%d",
-			(devinfo->firmware >> 16) & 0xff,
-			(devinfo->firmware >> 8)  & 0xff,
-			(devinfo->firmware >> 0)  & 0xff);
+			 (devinfo->firmware >> 16) & 0xff,
+			 (devinfo->firmware >> 8) & 0xff,
+			 (devinfo->firmware >> 0) & 0xff);
 	}
 	create_device_node(devdata->model, devdata->deviceid, serial_nr, firmware, "");
 
@@ -616,18 +629,18 @@ static void event_cb(dc_device_t *device, dc_event_type_t event, const void *dat
 
 	switch (event) {
 	case DC_EVENT_WAITING:
-		dev_info(devdata, translate("gettextFromC","Event: waiting for user action"));
+		dev_info(devdata, translate("gettextFromC", "Event: waiting for user action"));
 		break;
 	case DC_EVENT_PROGRESS:
 		if (!progress->maximum)
 			break;
-		progress_bar_fraction = (double) progress->current / (double) progress->maximum;
+		progress_bar_fraction = (double)progress->current / (double)progress->maximum;
 		break;
 	case DC_EVENT_DEVINFO:
-		dev_info(devdata, translate("gettextFromC","model=%u (0x%08x), firmware=%u (0x%08x), serial=%u (0x%08x)"),
-			devinfo->model, devinfo->model,
-			devinfo->firmware, devinfo->firmware,
-			devinfo->serial, devinfo->serial);
+		dev_info(devdata, translate("gettextFromC", "model=%u (0x%08x), firmware=%u (0x%08x), serial=%u (0x%08x)"),
+			 devinfo->model, devinfo->model,
+			 devinfo->firmware, devinfo->firmware,
+			 devinfo->serial, devinfo->serial);
 		if (devdata->libdc_logfile) {
 			fprintf(devdata->libdc_logfile, "Event: model=%u (0x%08x), firmware=%u (0x%08x), serial=%u (0x%08x)\n",
 				devinfo->model, devinfo->model,
@@ -645,10 +658,10 @@ static void event_cb(dc_device_t *device, dc_event_type_t event, const void *dat
 
 		break;
 	case DC_EVENT_CLOCK:
-			dev_info(devdata, translate("gettextFromC","Event: systime=%"PRId64", devtime=%u\n"),
-			(uint64_t)clock->systime, clock->devtime);
+		dev_info(devdata, translate("gettextFromC", "Event: systime=%" PRId64 ", devtime=%u\n"),
+			 (uint64_t)clock->systime, clock->devtime);
 		if (devdata->libdc_logfile) {
-			fprintf(devdata->libdc_logfile, "Event: systime=%"PRId64", devtime=%u\n",
+			fprintf(devdata->libdc_logfile, "Event: systime=%" PRId64 ", devtime=%u\n",
 				(uint64_t)clock->systime, clock->devtime);
 		}
 		break;
@@ -657,7 +670,7 @@ static void event_cb(dc_device_t *device, dc_event_type_t event, const void *dat
 			fprintf(devdata->libdc_logfile, "Event: vendor=");
 			for (unsigned int i = 0; i < vendor->size; ++i)
 				fprintf(devdata->libdc_logfile, "%02X", vendor->data[i]);
-			fprintf(devdata->libdc_logfile,"\n");
+			fprintf(devdata->libdc_logfile, "\n");
 		}
 		break;
 	default:
@@ -665,7 +678,7 @@ static void event_cb(dc_device_t *device, dc_event_type_t event, const void *dat
 	}
 }
 
-int  import_thread_cancelled;
+int import_thread_cancelled;
 
 static int
 cancel_cb(void *userdata)
@@ -684,32 +697,34 @@ static const char *do_device_import(device_data_t *data)
 	int events = DC_EVENT_WAITING | DC_EVENT_PROGRESS | DC_EVENT_DEVINFO | DC_EVENT_CLOCK | DC_EVENT_VENDOR;
 	rc = dc_device_set_events(device, events, event_cb, data);
 	if (rc != DC_STATUS_SUCCESS)
-		return translate("gettextFromC","Error registering the event handler.");
+		return translate("gettextFromC", "Error registering the event handler.");
 
 	// Register the cancellation handler.
 	rc = dc_device_set_cancel(device, cancel_cb, data);
 	if (rc != DC_STATUS_SUCCESS)
-		return translate("gettextFromC","Error registering the cancellation handler.");
+		return translate("gettextFromC", "Error registering the cancellation handler.");
 
 	if (data->libdc_dump) {
-		dc_buffer_t *buffer = dc_buffer_new (0);
+		dc_buffer_t *buffer = dc_buffer_new(0);
 
-		rc = dc_device_dump (device, buffer);
+		rc = dc_device_dump(device, buffer);
 		if (rc == DC_STATUS_SUCCESS && dumpfile_name) {
-			FILE* fp = subsurface_fopen(dumpfile_name, "wb");
+			FILE *fp = subsurface_fopen(dumpfile_name, "wb");
 			if (fp != NULL) {
-				fwrite (dc_buffer_get_data (buffer), 1, dc_buffer_get_size (buffer), fp);
-				fclose (fp);
+				fwrite(dc_buffer_get_data(buffer), 1, dc_buffer_get_size(buffer), fp);
+				fclose(fp);
 			}
 		}
 
-		dc_buffer_free (buffer);
+		dc_buffer_free(buffer);
 	} else {
 		rc = dc_device_foreach(device, dive_cb, data);
 	}
 
-	if (rc != DC_STATUS_SUCCESS)
-		return translate("gettextFromC","Dive data import error");
+	if (rc != DC_STATUS_SUCCESS) {
+		progress_bar_fraction = 0.0;
+		return translate("gettextFromC", "Dive data import error");
+	}
 
 	/* All good */
 	return NULL;
@@ -718,9 +733,9 @@ static const char *do_device_import(device_data_t *data)
 static void
 logfunc(dc_context_t *context, dc_loglevel_t loglevel, const char *file, unsigned int line, const char *function, const char *msg, void *userdata)
 {
-	const char *loglevels[] = {"NONE", "ERROR", "WARNING", "INFO", "DEBUG", "ALL"};
+	const char *loglevels[] = { "NONE", "ERROR", "WARNING", "INFO", "DEBUG", "ALL" };
 
-	FILE *fp = (FILE *) userdata;
+	FILE *fp = (FILE *)userdata;
 
 	if (loglevel == DC_LOGLEVEL_ERROR || loglevel == DC_LOGLEVEL_WARNING) {
 		fprintf(fp, "%s: %s [in %s:%d (%s)]\n", loglevels[loglevel], msg, file, line, function);
@@ -747,14 +762,14 @@ const char *do_libdivecomputer_import(device_data_t *data)
 
 	rc = dc_context_new(&data->context);
 	if (rc != DC_STATUS_SUCCESS)
-		return translate("gettextFromC","Unable to create libdivecomputer context");
+		return translate("gettextFromC", "Unable to create libdivecomputer context");
 
 	if (fp) {
 		dc_context_set_loglevel(data->context, DC_LOGLEVEL_ALL);
 		dc_context_set_logfunc(data->context, logfunc, fp);
 	}
 
-	err = translate("gettextFromC","Unable to open %s %s (%s)");
+	err = translate("gettextFromC", "Unable to open %s %s (%s)");
 	rc = dc_device_open(&data->device, data->context, data->descriptor, data->devname);
 	if (rc == DC_STATUS_SUCCESS) {
 		err = do_device_import(data);
