@@ -15,7 +15,8 @@ extern "C" {
 
 /* don't forget to include the UI toolkit specific display-XXX.h first
    to get the definition of progressbar_t */
-typedef struct device_data_t {
+typedef struct device_data_t
+{
 	dc_descriptor_t *descriptor;
 	const char *vendor, *product, *devname;
 	const char *model;
@@ -42,4 +43,4 @@ extern char *dumpfile_name;
 }
 #endif
 
-#endif
+#endif // LIBDIVECOMPUTER_H
