@@ -6,23 +6,24 @@
 class QModelIndex;
 class DiveComputerModel;
 
-class DiveComputerManagementDialog : public QDialog{
-Q_OBJECT
+class DiveComputerManagementDialog : public QDialog {
+	Q_OBJECT
 
 public:
 	static DiveComputerManagementDialog *instance();
 	void update();
 	void init();
 
-public  slots:
-	void tryRemove(const QModelIndex& index);
+public
+slots:
+	void tryRemove(const QModelIndex &index);
 	void accept();
 	void reject();
 
 private:
-	explicit DiveComputerManagementDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);
+	explicit DiveComputerManagementDialog(QWidget *parent = 0, Qt::WindowFlags f = 0);
 	Ui::DiveComputerManagementDialog ui;
 	DiveComputerModel *model;
 };
 
-#endif
+#endif // DIVECOMPUTERMANAGEMENTDIALOG_H
