@@ -9,7 +9,10 @@
 #include "modeldelegates.h"
 #include "mainwindow.h"
 #include "subsurfacewebservices.h"
+#include "divelogexportdialog.h"
 #include "../display.h"
+#include "exif.h"
+#include "../file.h"
 #include <QApplication>
 #include <QHeaderView>
 #include <QDebug>
@@ -21,42 +24,48 @@
 #include <QKeyEvent>
 #include <QMenu>
 #include <QFileDialog>
+#include <string>
+#include <iostream>
+#include "../qthelper.h"
 
-DiveListView::DiveListView(QWidget *parent) : QTreeView(parent), mouseClickSelection(false),
-	sortColumn(0), currentOrder(Qt::DescendingOrder), searchBox(new QLineEdit(this))
+DiveListView::DiveListView(QWidget *parent) : QTreeView(parent), mouseClickSelection(false), sortColumn(0),
+	currentOrder(Qt::DescendingOrder), searchBox(this), dontEmitDiveChangedSignal(false), selectionSaved(false)
 {
-	setItemDelegate(new DiveListDelegate );
+	setItemDelegate(new DiveListDelegate(this));
 	setUniformRowHeights(true);
-	setItemDelegateForColumn(DiveTripModel::RATING, new StarWidgetsDelegate());
+	setItemDelegateForColumn(DiveTripModel::RATING, new StarWidgetsDelegate(this));
 	QSortFilterProxyModel *model = new QSortFilterProxyModel(this);
 	model->setSortRole(DiveTripModel::SORT_ROLE);
 	model->setFilterKeyColumn(-1); // filter all columns
+	model->setFilterCaseSensitivity(Qt::CaseInsensitive);
 	setModel(model);
 	connect(model, SIGNAL(layoutChanged()), this, SLOT(fixMessyQtModelBehaviour()));
 
 	setSortingEnabled(false);
 	setContextMenuPolicy(Qt::DefaultContextMenu);
 	header()->setContextMenuPolicy(Qt::ActionsContextMenu);
+#ifdef Q_OS_MAC
+	// Fixes for the layout needed for mac
 	const QFontMetrics metrics(defaultModelFont());
 	header()->setMinimumHeight(metrics.height() + 10);
+#endif
 	header()->setStretchLastSection(true);
-	QAction *showSearchBox = new QAction(tr("Show Search Box"), this);
-	showSearchBox->setShortcut( Qt::CTRL + Qt::Key_F);
-	showSearchBox->setShortcutContext(Qt::ApplicationShortcut);
+	QAction *showSearchBox = new QAction(tr("Show search box"), this);
+	showSearchBox->setShortcut(Qt::CTRL + Qt::Key_F);
+	showSearchBox->setShortcutContext(Qt::WindowShortcut);
 	addAction(showSearchBox);
 
-	searchBox->installEventFilter(this);
-	searchBox->hide();
+	searchBox.installEventFilter(this);
+	searchBox.hide();
 	connect(showSearchBox, SIGNAL(triggered(bool)), this, SLOT(showSearchEdit()));
-	connect(searchBox, SIGNAL(textChanged(QString)), model, SLOT(setFilterFixedString(QString)));
-	setupUi();
+	connect(&searchBox, SIGNAL(textChanged(QString)), model, SLOT(setFilterFixedString(QString)));
 }
 
 DiveListView::~DiveListView()
 {
 	QSettings settings;
 	settings.beginGroup("ListWidget");
-	for (int i = DiveTripModel::NR; i < DiveTripModel::COLUMNS; i++){
+	for (int i = DiveTripModel::NR; i < DiveTripModel::COLUMNS; i++) {
 		if (isColumnHidden(i))
 			continue;
 		settings.setValue(QString("colwidth%1").arg(i), columnWidth(i));
@@ -64,17 +73,18 @@ DiveListView::~DiveListView()
 	settings.endGroup();
 }
 
-void DiveListView::setupUi(){
+void DiveListView::setupUi()
+{
 	QSettings settings;
 	static bool firstRun = true;
-	if(firstRun)
+	if (firstRun)
 		backupExpandedRows();
 	settings.beginGroup("ListWidget");
 	/* if no width are set, use the calculated width for each column;
 	 * for that to work we need to temporarily expand all rows */
 	expandAll();
 	for (int i = DiveTripModel::NR; i < DiveTripModel::COLUMNS; i++) {
-		if(isColumnHidden(i))
+		if (isColumnHidden(i))
 			continue;
 		QVariant width = settings.value(QString("colwidth%1").arg(i));
 		if (width.isValid())
@@ -83,7 +93,7 @@ void DiveListView::setupUi(){
 			setColumnWidth(i, 100);
 	}
 	settings.endGroup();
-	if(firstRun)
+	if (firstRun)
 		restoreExpandedRows();
 	else
 		collapseAll();
@@ -95,31 +105,33 @@ int DiveListView::lastVisibleColumn()
 {
 	int lastColumn = -1;
 	for (int i = DiveTripModel::NR; i < DiveTripModel::COLUMNS; i++) {
-		if(isColumnHidden(i))
+		if (isColumnHidden(i))
 			continue;
 		lastColumn = i;
 	}
 	return lastColumn;
 }
 
-void DiveListView::backupExpandedRows(){
+void DiveListView::backupExpandedRows()
+{
 	expandedRows.clear();
-	for(int i = 0; i < model()->rowCount(); i++)
-		if(isExpanded( model()->index(i, 0) ))
+	for (int i = 0; i < model()->rowCount(); i++)
+		if (isExpanded(model()->index(i, 0)))
 			expandedRows.push_back(i);
 }
 
-void DiveListView::restoreExpandedRows(){
+void DiveListView::restoreExpandedRows()
+{
 	setAnimated(false);
-	Q_FOREACH(const int &i, expandedRows)
-		setExpanded( model()->index(i, 0), true );
+	Q_FOREACH (const int &i, expandedRows)
+		setExpanded(model()->index(i, 0), true);
 	setAnimated(true);
 }
 void DiveListView::fixMessyQtModelBehaviour()
 {
 	QAbstractItemModel *m = model();
-	for(int i = 0; i < model()->rowCount(); i++)
-		if (m->rowCount( m->index(i, 0) ) != 0)
+	for (int i = 0; i < model()->rowCount(); i++)
+		if (m->rowCount(m->index(i, 0)) != 0)
 			setFirstColumnSpanned(i, QModelIndex(), true);
 }
 
@@ -128,48 +140,48 @@ void DiveListView::rememberSelection()
 {
 	selectedDives.clear();
 	QItemSelection selection = selectionModel()->selection();
-	Q_FOREACH(const QModelIndex& index , selection.indexes()) {
+	Q_FOREACH (const QModelIndex &index, selection.indexes()) {
 		if (index.column() != 0) // We only care about the dives, so, let's stick to rows and discard columns.
 			continue;
-		struct dive *d = (struct dive *) index.data(DiveTripModel::DIVE_ROLE).value<void*>();
+		struct dive *d = (struct dive *)index.data(DiveTripModel::DIVE_ROLE).value<void *>();
 		if (d)
 			selectedDives.insert(d->divetrip, get_divenr(d));
 	}
+	selectionSaved = true;
 }
 
 void DiveListView::restoreSelection()
 {
+	if (!selectionSaved)
+		return;
+
+	selectionSaved = false;
+	dontEmitDiveChangedSignal = true;
 	unselectDives();
-	Q_FOREACH(dive_trip_t *trip, selectedDives.keys()){
+	dontEmitDiveChangedSignal = false;
+	Q_FOREACH (dive_trip_t *trip, selectedDives.keys()) {
 		QList<int> divesOnTrip = getDivesInTrip(trip);
 		QList<int> selectedDivesOnTrip = selectedDives.values(trip);
 
-		// Trip was not selected, let's select single-dives.
-		if (trip == NULL || divesOnTrip.count() != selectedDivesOnTrip.count()){
-			Q_FOREACH(int i, selectedDivesOnTrip){
-				selectDive(i);
-			}
-		}else{
+		// Only select trip if all of its dives were selected
+		if (trip != NULL && divesOnTrip.count() == selectedDivesOnTrip.count())
 			selectTrip(trip);
-			Q_FOREACH(int i, selectedDivesOnTrip){
-				selectDive(i);
-			}
-		}
+		selectDives(selectedDivesOnTrip);
 	}
 }
 
-void DiveListView::selectTrip ( dive_trip_t* trip )
+void DiveListView::selectTrip(dive_trip_t *trip)
 {
 	if (!trip)
 		return;
 
-	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel*>(model());
-	QModelIndexList match = m->match(m->index(0,0), DiveTripModel::TRIP_ROLE, QVariant::fromValue<void*>(trip), 2, Qt::MatchRecursive);
+	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel *>(model());
+	QModelIndexList match = m->match(m->index(0, 0), DiveTripModel::TRIP_ROLE, QVariant::fromValue<void *>(trip), 2, Qt::MatchRecursive);
 	QItemSelectionModel::SelectionFlags flags;
 	if (!match.count())
 		return;
 	QModelIndex idx = match.first();
-	flags =  QItemSelectionModel::Select;
+	flags = QItemSelectionModel::Select;
 	flags |= QItemSelectionModel::Rows;
 	selectionModel()->select(idx, flags);
 	expand(idx);
@@ -178,15 +190,21 @@ void DiveListView::selectTrip ( dive_trip_t* trip )
 void DiveListView::unselectDives()
 {
 	selectionModel()->clearSelection();
+	// clearSelection should emit selectionChanged() but sometimes that
+	// appears not to happen
+	int i;
+	struct dive *dive;
+	for_each_dive(i, dive) {
+		deselect_dive(i);
+	}
 }
 
-QList< dive_trip_t* > DiveListView::selectedTrips()
+QList<dive_trip_t *> DiveListView::selectedTrips()
 {
-	QModelIndexList indexes = selectionModel()->selectedRows();
-	QList<dive_trip_t*> ret;
-	Q_FOREACH(const QModelIndex& index, indexes){
-		dive_trip_t *trip = static_cast<dive_trip_t*>(index.data(DiveTripModel::TRIP_ROLE).value<void*>());
-		if(!trip)
+	QList<dive_trip_t *> ret;
+	Q_FOREACH (const QModelIndex &index, selectionModel()->selectedRows()) {
+		dive_trip_t *trip = static_cast<dive_trip_t *>(index.data(DiveTripModel::TRIP_ROLE).value<void *>());
+		if (!trip)
 			continue;
 		ret.push_back(trip);
 	}
@@ -195,92 +213,70 @@ QList< dive_trip_t* > DiveListView::selectedTrips()
 
 void DiveListView::selectDive(int i, bool scrollto, bool toggle)
 {
-	if( i == -1)
+	if (i == -1)
 		return;
-	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel*>(model());
-	QModelIndexList match = m->match(m->index(0,0), DiveTripModel::DIVE_IDX, i, 2, Qt::MatchRecursive);
+	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel *>(model());
+	QModelIndexList match = m->match(m->index(0, 0), DiveTripModel::DIVE_IDX, i, 2, Qt::MatchRecursive);
 	QItemSelectionModel::SelectionFlags flags;
+	if (match.isEmpty())
+		return;
 	QModelIndex idx = match.first();
 	flags = toggle ? QItemSelectionModel::Toggle : QItemSelectionModel::Select;
 	flags |= QItemSelectionModel::Rows;
 	selectionModel()->setCurrentIndex(idx, flags);
-	if(idx.parent().isValid()){
+	if (idx.parent().isValid()) {
 		setAnimated(false);
 		expand(idx.parent());
+		if (scrollto)
+			scrollTo(idx.parent());
 		setAnimated(true);
 	}
 	if (scrollto)
 		scrollTo(idx, PositionAtCenter);
 }
 
-void DiveListView::selectDives(const QList< int >& newDiveSelection)
+void DiveListView::selectDives(const QList<int> &newDiveSelection)
 {
-	if(!newDiveSelection.count())
+	if (!newDiveSelection.count())
 		return;
 
-	disconnect(selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
-		   this, SLOT(selectionChanged(QItemSelection,QItemSelection)));
-	disconnect(selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
-		   this, SLOT(currentChanged(QModelIndex,QModelIndex)));
+	dontEmitDiveChangedSignal = true;
+	// select the dives, highest index first - this way the oldest of the dives
+	// becomes the selected_dive that we scroll to
+	QList<int> sortedSelection = newDiveSelection;
+	qSort(sortedSelection.begin(), sortedSelection.end());
+	while (!sortedSelection.isEmpty())
+		selectDive(sortedSelection.takeLast());
 
-	setAnimated(false);
-	collapseAll();
-	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel*>(model());
-	QItemSelectionModel::SelectionFlags flags = QItemSelectionModel::Select | QItemSelectionModel::Rows;
-
-	QItemSelection newDeselected = selectionModel()->selection();
-	QModelIndexList diveList;
-
-	int firstSelectedDive = -1;
-	/* context for temp. variables. */{
-		int i = 0;
-		struct dive *dive;
-		for_each_dive(i, dive){
-			dive->selected = newDiveSelection.contains(i) == true;
-			if(firstSelectedDive == -1 && dive->selected ){
-				firstSelectedDive = i;
-			}
-		}
-	}
-	select_dive(firstSelectedDive);
-	Q_FOREACH(int i, newDiveSelection){
-		diveList.append(m->match(m->index(0,0), DiveTripModel::DIVE_IDX,
-			i, 2, Qt::MatchRecursive).first());
-	}
-	Q_FOREACH(const QModelIndex& idx, diveList){
-		selectionModel()->select(idx, flags);
-		if(idx.parent().isValid() && !isExpanded(idx.parent())){
-			expand(idx.parent());
-		}
-	}
-	setAnimated(true);
-	QTreeView::selectionChanged(selectionModel()->selection(), newDeselected);
-	connect(selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
-		this, SLOT(selectionChanged(QItemSelection,QItemSelection)));
-	connect(selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
-		this, SLOT(currentChanged(QModelIndex,QModelIndex)));
-	Q_EMIT currentDiveChanged(selected_dive);
-	const QModelIndex& idx = m->match(m->index(0,0), DiveTripModel::DIVE_IDX,selected_dive, 2, Qt::MatchRecursive).first();
+	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel *>(model());
+	QModelIndex idx = m->match(m->index(0, 0), DiveTripModel::DIVE_IDX, selected_dive, 2, Qt::MatchRecursive).first();
+	if (idx.parent().isValid())
+		scrollTo(idx.parent());
 	scrollTo(idx);
+
+	// now that everything is up to date, update the widgets
+	Q_EMIT currentDiveChanged(selected_dive);
+	dontEmitDiveChangedSignal = false;
+	return;
 }
 
 void DiveListView::showSearchEdit()
 {
-	searchBox->show();
-	searchBox->setFocus();
+	searchBox.show();
+	searchBox.setFocus();
 }
 
-bool DiveListView::eventFilter(QObject* , QEvent* event)
+bool DiveListView::eventFilter(QObject *, QEvent *event)
 {
-	if(event->type() != QEvent::KeyPress)
+	if (event->type() != QEvent::KeyPress)
 		return false;
-	QKeyEvent *keyEv = static_cast<QKeyEvent*>(event);
+	QKeyEvent *keyEv = static_cast<QKeyEvent *>(event);
 	if (keyEv->key() != Qt::Key_Escape)
 		return false;
 
-	searchBox->clear();
-	searchBox->hide();
-	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel*>(model());
+	searchBox.clear();
+	searchBox.hide();
+	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel *>(model());
 	m->setFilterFixedString(QString());
 	return true;
 }
@@ -291,7 +287,7 @@ bool DiveListView::eventFilter(QObject* , QEvent* event)
 // index. TRIP_ROLE vs DIVE_ROLE?
 void DiveListView::headerClicked(int i)
 {
-	DiveTripModel::Layout newLayout = i == (int) DiveTripModel::NR ? DiveTripModel::TREE : DiveTripModel::LIST;
+	DiveTripModel::Layout newLayout = i == (int)DiveTripModel::NR ? DiveTripModel::TREE : DiveTripModel::LIST;
 	rememberSelection();
 	unselectDives();
 	/* No layout change? Just re-sort, and scroll to first selection, making sure all selections are expanded */
@@ -300,31 +296,44 @@ void DiveListView::headerClicked(int i)
 		sortByColumn(i, currentOrder);
 	} else {
 		// clear the model, repopulate with new indexes.
-		if(currentLayout == DiveTripModel::TREE){
+		if (currentLayout == DiveTripModel::TREE) {
 			backupExpandedRows();
 		}
 		reload(newLayout, false);
 		currentOrder = Qt::DescendingOrder;
 		sortByColumn(i, currentOrder);
-		if (newLayout == DiveTripModel::TREE){
+		if (newLayout == DiveTripModel::TREE) {
 			restoreExpandedRows();
 		}
 	}
 	restoreSelection();
+	// remember the new sort column
+	sortColumn = i;
 }
 
 void DiveListView::reload(DiveTripModel::Layout layout, bool forceSort)
 {
+	// we want to run setupUi() once we actually are displaying something
+	// in the widget
+	static bool first = true;
+	if (first && dive_table.nr > 0) {
+		setupUi();
+		first = false;
+	}
 	if (layout == DiveTripModel::CURRENT)
 		layout = currentLayout;
 	else
 		currentLayout = layout;
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 	header()->setClickable(true);
+#else
+	header()->setSectionsClickable(true);
+#endif
 	connect(header(), SIGNAL(sectionPressed(int)), this, SLOT(headerClicked(int)), Qt::UniqueConnection);
 
-	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel*>(model());
+	QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel *>(model());
 	QAbstractItemModel *oldModel = m->sourceModel();
-	if (oldModel){
+	if (oldModel) {
 		oldModel->deleteLater();
 	}
 	DiveTripModel *tripModel = new DiveTripModel(this);
@@ -332,32 +341,32 @@ void DiveListView::reload(DiveTripModel::Layout layout, bool forceSort)
 
 	m->setSourceModel(tripModel);
 
-	if(!forceSort)
+	if (!forceSort)
 		return;
 
 	sortByColumn(sortColumn, currentOrder);
 	if (amount_selected && current_dive != NULL) {
 		selectDive(selected_dive, true);
 	} else {
-		QModelIndex firstDiveOrTrip = m->index(0,0);
+		QModelIndex firstDiveOrTrip = m->index(0, 0);
 		if (firstDiveOrTrip.isValid()) {
-			if (m->index(0,0, firstDiveOrTrip).isValid())
-				setCurrentIndex(m->index(0,0, firstDiveOrTrip));
+			if (m->index(0, 0, firstDiveOrTrip).isValid())
+				setCurrentIndex(m->index(0, 0, firstDiveOrTrip));
 			else
 				setCurrentIndex(firstDiveOrTrip);
 		}
 	}
-	setupUi();
-	if(selectedIndexes().count()){
+	if (selectedIndexes().count()) {
 		QModelIndex curr = selectedIndexes().first();
 		curr = curr.parent().isValid() ? curr.parent() : curr;
-		if(!isExpanded(curr)){
+		if (!isExpanded(curr)) {
 			setAnimated(false);
 			expand(curr);
+			scrollTo(curr);
 			setAnimated(true);
 		}
 	}
-	if(currentLayout == DiveTripModel::TREE){
+	if (currentLayout == DiveTripModel::TREE) {
 		fixMessyQtModelBehaviour();
 	}
 }
@@ -369,19 +378,12 @@ void DiveListView::reloadHeaderActions()
 	if (!header()->actions().size()) {
 		QSettings s;
 		s.beginGroup("DiveListColumnState");
-		for(int i = 0; i < model()->columnCount(); i++) {
+		for (int i = 0; i < model()->columnCount(); i++) {
 			QString title = QString("%1").arg(model()->headerData(i, Qt::Horizontal).toString());
 			QString settingName = QString("showColumn%1").arg(i);
 			QAction *a = new QAction(title, header());
 			bool showHeaderFirstRun = !(
-				i == DiveTripModel::MAXCNS
-				|| i == DiveTripModel::NITROX
-				|| i == DiveTripModel::OTU
-				|| i == DiveTripModel::TEMPERATURE
-				|| i == DiveTripModel::TOTALWEIGHT
-				|| i == DiveTripModel::SUIT
-				|| i == DiveTripModel::CYLINDER
-				|| i == DiveTripModel::SAC );
+							i == DiveTripModel::MAXCNS || i == DiveTripModel::NITROX || i == DiveTripModel::OTU || i == DiveTripModel::TEMPERATURE || i == DiveTripModel::TOTALWEIGHT || i == DiveTripModel::SUIT || i == DiveTripModel::CYLINDER || i == DiveTripModel::SAC);
 			bool shown = s.value(settingName, showHeaderFirstRun).toBool();
 			a->setCheckable(true);
 			a->setChecked(shown);
@@ -393,7 +395,7 @@ void DiveListView::reloadHeaderActions()
 		}
 		s.endGroup();
 	} else {
-		for(int i = 0; i < model()->columnCount(); i++) {
+		for (int i = 0; i < model()->columnCount(); i++) {
 			QString title = QString("%1").arg(model()->headerData(i, Qt::Horizontal).toString());
 			header()->actions()[i]->setText(title);
 		}
@@ -402,7 +404,7 @@ void DiveListView::reloadHeaderActions()
 
 void DiveListView::toggleColumnVisibilityByIndex()
 {
-	QAction *action = qobject_cast<QAction*>(sender());
+	QAction *action = qobject_cast<QAction *>(sender());
 	if (!action)
 		return;
 
@@ -415,53 +417,42 @@ void DiveListView::toggleColumnVisibilityByIndex()
 	setColumnWidth(lastVisibleColumn(), 10);
 }
 
-void DiveListView::currentChanged(const QModelIndex& current, const QModelIndex& previous)
+void DiveListView::currentChanged(const QModelIndex &current, const QModelIndex &previous)
 {
 	if (!current.isValid())
 		return;
 	scrollTo(current);
 }
 
-void DiveListView::selectionChanged(const QItemSelection& selected, const QItemSelection& deselected)
+void DiveListView::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected)
 {
 	QItemSelection newSelected = selected.size() ? selected : selectionModel()->selection();
 	QItemSelection newDeselected = deselected;
 
-	disconnect(selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(selectionChanged(QItemSelection,QItemSelection)));
-	disconnect(selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),         this, SLOT(currentChanged(QModelIndex,QModelIndex)));
+	disconnect(selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)), this, SLOT(selectionChanged(QItemSelection, QItemSelection)));
+	disconnect(selectionModel(), SIGNAL(currentChanged(QModelIndex, QModelIndex)), this, SLOT(currentChanged(QModelIndex, QModelIndex)));
 
-	Q_FOREACH(const QModelIndex& index, newDeselected.indexes()) {
+	Q_FOREACH (const QModelIndex &index, newDeselected.indexes()) {
 		if (index.column() != 0)
 			continue;
 		const QAbstractItemModel *model = index.model();
-		struct dive *dive = (struct dive*) model->data(index, DiveTripModel::DIVE_ROLE).value<void*>();
-		if (!dive) { // it's a trip!
-			if (model->rowCount(index)) {
-				struct dive *child = (struct dive*) model->data(index.child(0,0), DiveTripModel::DIVE_ROLE).value<void*>();
-				while (child) {
-					deselect_dive(get_divenr(child));
-					child = child->next;
-				}
-			}
-		} else {
+		struct dive *dive = (struct dive *)model->data(index, DiveTripModel::DIVE_ROLE).value<void *>();
+		if (!dive) // it's a trip!
+			deselect_dives_in_trip((dive_trip_t *)model->data(index, DiveTripModel::TRIP_ROLE).value<void *>());
+		else
 			deselect_dive(get_divenr(dive));
-		}
 	}
-	Q_FOREACH(const QModelIndex& index, newSelected.indexes()) {
+	Q_FOREACH (const QModelIndex &index, newSelected.indexes()) {
 		if (index.column() != 0)
 			continue;
 
 		const QAbstractItemModel *model = index.model();
-		struct dive *dive = (struct dive*) model->data(index, DiveTripModel::DIVE_ROLE).value<void*>();
+		struct dive *dive = (struct dive *)model->data(index, DiveTripModel::DIVE_ROLE).value<void *>();
 		if (!dive) { // it's a trip!
 			if (model->rowCount(index)) {
 				QItemSelection selection;
-				struct dive *child = (struct dive*) model->data(index.child(0,0), DiveTripModel::DIVE_ROLE).value<void*>();
-				while (child) {
-					select_dive(get_divenr(child));
-					child = child->next;
-				}
-				selection.select(index.child(0,0), index.child(model->rowCount(index) -1 , 0));
+				select_dives_in_trip((dive_trip_t *)model->data(index, DiveTripModel::TRIP_ROLE).value<void *>());
+				selection.select(index.child(0, 0), index.child(model->rowCount(index) - 1, 0));
 				selectionModel()->select(selection, QItemSelectionModel::Select | QItemSelectionModel::Rows);
 				selectionModel()->setCurrentIndex(index, QItemSelectionModel::Select | QItemSelectionModel::NoUpdate);
 				if (!isExpanded(index))
@@ -472,10 +463,10 @@ void DiveListView::selectionChanged(const QItemSelection& selected, const QItemS
 		}
 	}
 	QTreeView::selectionChanged(selectionModel()->selection(), newDeselected);
-	connect(selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(selectionChanged(QItemSelection,QItemSelection)));
-	connect(selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),         this, SLOT(currentChanged(QModelIndex,QModelIndex)));
-	// now that everything is up to date, update the widgets
-	Q_EMIT currentDiveChanged(selected_dive);
+	connect(selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)), this, SLOT(selectionChanged(QItemSelection, QItemSelection)));
+	connect(selectionModel(), SIGNAL(currentChanged(QModelIndex, QModelIndex)), this, SLOT(currentChanged(QModelIndex, QModelIndex)));
+	if(!dontEmitDiveChangedSignal)
+		Q_EMIT currentDiveChanged(selected_dive);
 }
 
 static bool can_merge(const struct dive *a, const struct dive *b)
@@ -485,7 +476,7 @@ static bool can_merge(const struct dive *a, const struct dive *b)
 	if (a->when > b->when)
 		return false;
 	/* Don't merge dives if there's more than half an hour between them */
-	if (a->when + a->duration.seconds + 30*60 < b->when)
+	if (a->when + a->duration.seconds + 30 * 60 < b->when)
 		return false;
 	return true;
 }
@@ -495,7 +486,7 @@ void DiveListView::mergeDives()
 	int i;
 	struct dive *dive, *maindive = NULL;
 
-	for_each_dive(i, dive) {
+	for_each_dive (i, dive) {
 		if (dive->selected) {
 			if (!can_merge(maindive, dive)) {
 				maindive = dive;
@@ -505,31 +496,31 @@ void DiveListView::mergeDives()
 			}
 		}
 	}
-	mainWindow()->refreshDisplay();
+	MainWindow::instance()->refreshDisplay();
+}
+
+void DiveListView::renumberDives()
+{
+	RenumberDialog::instance()->renumberOnlySelected();
+	RenumberDialog::instance()->show();
 }
 
 void DiveListView::merge_trip(const QModelIndex &a, int offset)
 {
 	int i = a.row() + offset;
-	QModelIndex b = a.sibling(i,0);
-
-	dive_trip_t *trip_a = (dive_trip_t *) a.data(DiveTripModel::TRIP_ROLE).value<void*>();
-	dive_trip_t *trip_b = (dive_trip_t *) b.data(DiveTripModel::TRIP_ROLE).value<void*>();
+	QModelIndex b = a.sibling(i, 0);
 
+	dive_trip_t *trip_a = (dive_trip_t *)a.data(DiveTripModel::TRIP_ROLE).value<void *>();
+	dive_trip_t *trip_b = (dive_trip_t *)b.data(DiveTripModel::TRIP_ROLE).value<void *>();
 	if (trip_a == trip_b || !trip_a || !trip_b)
 		return;
-
-	if (!trip_a->location && trip_b->location)
-		trip_a->location = strdup(trip_b->location);
-	if (!trip_a->notes && trip_b->notes)
-		trip_a->notes = strdup(trip_b->notes);
-	while (trip_b->dives)
-		add_dive_to_trip(trip_b->dives, trip_a);
+	combine_trips(trip_a, trip_b);
 	rememberSelection();
 	reload(currentLayout, false);
 	fixMessyQtModelBehaviour();
 	restoreSelection();
-	mark_divelist_changed(TRUE);
+	mark_divelist_changed(true);
+	//TODO: emit a signal to signalize that the divelist changed?
 }
 
 void DiveListView::mergeTripAbove()
@@ -544,62 +535,70 @@ void DiveListView::mergeTripBelow()
 
 void DiveListView::removeFromTrip()
 {
+	//TODO: move this to C-code.
 	int i;
 	struct dive *d;
-	for_each_dive(i, d) {
+	for_each_dive (i, d) {
 		if (d->selected)
-			remove_dive_from_trip(d, FALSE);
+			remove_dive_from_trip(d, false);
 	}
 	rememberSelection();
 	reload(currentLayout, false);
 	fixMessyQtModelBehaviour();
 	restoreSelection();
-	mark_divelist_changed(TRUE);
+	mark_divelist_changed(true);
 }
 
 void DiveListView::newTripAbove()
 {
-	dive_trip_t *trip;
-	int idx;
-	struct dive *d = (struct dive *) contextMenuIndex.data(DiveTripModel::DIVE_ROLE).value<void*>();
+	struct dive *d = (struct dive *)contextMenuIndex.data(DiveTripModel::DIVE_ROLE).value<void *>();
 	if (!d) // shouldn't happen as we only are setting up this action if this is a dive
 		return;
+	//TODO: port to c-code.
+	dive_trip_t *trip;
+	int idx;
 	rememberSelection();
 	trip = create_and_hookup_trip_from_dive(d);
-	for_each_dive(idx, d) {
+	for_each_dive (idx, d) {
 		if (d->selected)
 			add_dive_to_trip(d, trip);
 	}
 	trip->expanded = 1;
 	reload(currentLayout, false);
 	fixMessyQtModelBehaviour();
-	mark_divelist_changed(TRUE);
+	mark_divelist_changed(true);
 	restoreSelection();
 }
 
+void DiveListView::addToTripBelow()
+{
+	addToTrip(true);
+}
+
 void DiveListView::addToTripAbove()
 {
-	int idx, delta = (currentOrder == Qt::AscendingOrder) ? -1 : +1;
+	addToTrip(false);
+}
+
+void DiveListView::addToTrip(bool below)
+{
+	int delta = (currentOrder == Qt::AscendingOrder) ? -1 : +1;
+	struct dive *d = (struct dive *)contextMenuIndex.data(DiveTripModel::DIVE_ROLE).value<void *>();
+	rememberSelection();
+
+	//TODO: This part should be moved to C-code.
+	int idx;
 	dive_trip_t *trip = NULL;
 	struct dive *pd = NULL;
-	struct dive *d = (struct dive *) contextMenuIndex.data(DiveTripModel::DIVE_ROLE).value<void*>();
-	if (!d) // shouldn't happen as we only are setting up this action if this is a dive
-		return;
-	rememberSelection();
+	if (below) // Should we add to the trip below instead?
+		delta *= -1;
+
 	if (d->selected) { // we are right-clicking on one of possibly many selected dive(s)
 		// find the top selected dive, depending on the list order
-		if (delta == 1) {
-			for_each_dive(idx, d) {
-				if (d->selected)
-					pd = d;
-			}
-			d = pd; // this way we have the chronologically last
-		} else {
-			for_each_dive(idx, d) {
-				if (d->selected)
-					break; // now that's the chronologically first
-			}
-		}
+		if (delta == 1)
+			d = last_selected_dive();
+		else
+			d = first_selected_dive();
 	}
 	// now find the trip "above" in the dive list
 	if ((pd = get_dive(get_divenr(d) + delta)) != NULL) {
@@ -610,13 +609,14 @@ void DiveListView::addToTripAbove()
 		return;
 	add_dive_to_trip(d, trip);
 	if (d->selected) { // there are possibly other selected dives that we should add
-		for_each_dive(idx, d) {
+		for_each_dive (idx, d) {
 			if (d->selected)
 				add_dive_to_trip(d, trip);
 		}
 	}
 	trip->expanded = 1;
-	mark_divelist_changed(TRUE);
+	mark_divelist_changed(true);
+	//This part stays at C++ code.
 	reload(currentLayout, false);
 	restoreSelection();
 	fixMessyQtModelBehaviour();
@@ -625,21 +625,22 @@ void DiveListView::addToTripAbove()
 void DiveListView::markDiveInvalid()
 {
 	int i;
-	struct dive *d = (struct dive *) contextMenuIndex.data(DiveTripModel::DIVE_ROLE).value<void*>();
+	struct dive *d = (struct dive *)contextMenuIndex.data(DiveTripModel::DIVE_ROLE).value<void *>();
 	if (!d)
 		return;
 	for_each_dive (i, d) {
 		if (!d->selected)
 			continue;
+		//TODO: this should be done in the future
 		// now mark the dive invalid... how do we do THAT?
 		// d->invalid = true;
 	}
 	if (amount_selected == 0) {
-		mainWindow()->cleanUpEmpty();
+		MainWindow::instance()->cleanUpEmpty();
 	}
-	mark_divelist_changed(TRUE);
-	mainWindow()->refreshDisplay();
-	if(prefs.display_invalid_dives == FALSE) {
+	mark_divelist_changed(true);
+	MainWindow::instance()->refreshDisplay();
+	if (prefs.display_invalid_dives == false) {
 		clearSelection();
 		// select top dive that isn't marked invalid
 		rememberSelection();
@@ -649,16 +650,17 @@ void DiveListView::markDiveInvalid()
 
 void DiveListView::deleteDive()
 {
-	int i;
-	struct dive *d = (struct dive *) contextMenuIndex.data(DiveTripModel::DIVE_ROLE).value<void*>();
+	struct dive *d = (struct dive *)contextMenuIndex.data(DiveTripModel::DIVE_ROLE).value<void *>();
 	if (!d)
 		return;
+
+	//TODO: port this to C-code.
+	int i;
 	// after a dive is deleted the ones following it move forward in the dive_table
 	// so instead of using the for_each_dive macro I'm using an explicit for loop
 	// to make this easier to understand
 	int lastDiveNr = -1;
-	for (i = 0; i < dive_table.nr; i++) {
-		d = get_dive(i);
+	for_each_dive (i, d) {
 		if (!d->selected)
 			continue;
 		delete_single_dive(i);
@@ -666,11 +668,11 @@ void DiveListView::deleteDive()
 		lastDiveNr = i;
 	}
 	if (amount_selected == 0) {
-		mainWindow()->cleanUpEmpty();
+		MainWindow::instance()->cleanUpEmpty();
 	}
-	mark_divelist_changed(TRUE);
-	mainWindow()->refreshDisplay();
-	if(lastDiveNr != -1){
+	mark_divelist_changed(true);
+	MainWindow::instance()->refreshDisplay();
+	if (lastDiveNr != -1) {
 		clearSelection();
 		selectDive(lastDiveNr);
 		rememberSelection();
@@ -680,12 +682,12 @@ void DiveListView::deleteDive()
 
 void DiveListView::testSlot()
 {
-	struct dive *d = (struct dive *) contextMenuIndex.data(DiveTripModel::DIVE_ROLE).value<void*>();
+	struct dive *d = (struct dive *)contextMenuIndex.data(DiveTripModel::DIVE_ROLE).value<void *>();
 	if (d) {
 		qDebug("testSlot called on dive #%d", d->number);
 	} else {
 		QModelIndex child = contextMenuIndex.child(0, 0);
-		d = (struct dive *) child.data(DiveTripModel::DIVE_ROLE).value<void*>();
+		d = (struct dive *)child.data(DiveTripModel::DIVE_ROLE).value<void *>();
 		if (d)
 			qDebug("testSlot called on trip including dive #%d", d->number);
 		else
@@ -698,41 +700,55 @@ void DiveListView::contextMenuEvent(QContextMenuEvent *event)
 	QAction *collapseAction = NULL;
 	// let's remember where we are
 	contextMenuIndex = indexAt(event->pos());
-	struct dive *d = (struct dive *) contextMenuIndex.data(DiveTripModel::DIVE_ROLE).value<void*>();
-	dive_trip_t *trip = (dive_trip_t *) contextMenuIndex.data(DiveTripModel::TRIP_ROLE).value<void*>();
+	struct dive *d = (struct dive *)contextMenuIndex.data(DiveTripModel::DIVE_ROLE).value<void *>();
+	dive_trip_t *trip = (dive_trip_t *)contextMenuIndex.data(DiveTripModel::TRIP_ROLE).value<void *>();
 	QMenu popup(this);
 	if (currentLayout == DiveTripModel::TREE) {
-		popup.addAction(tr("expand all"), this, SLOT(expandAll()));
-		popup.addAction(tr("collapse all"), this, SLOT(collapseAll()));
-		collapseAction = popup.addAction(tr("collapse others"), this, SLOT(collapseAll()));
+		popup.addAction(tr("Expand all"), this, SLOT(expandAll()));
+		popup.addAction(tr("Collapse all"), this, SLOT(collapseAll()));
+		collapseAction = popup.addAction(tr("Collapse others"), this, SLOT(collapseAll()));
 		if (d) {
-			popup.addAction(tr("remove dive(s) from trip"), this, SLOT(removeFromTrip()));
-			popup.addAction(tr("create new trip above"), this, SLOT(newTripAbove()));
-			popup.addAction(tr("add dive(s) to trip immediately above"), this, SLOT(addToTripAbove()));
+			popup.addAction(tr("Remove dive(s) from trip"), this, SLOT(removeFromTrip()));
+			popup.addAction(tr("Create new trip above"), this, SLOT(newTripAbove()));
+			if (!d->divetrip) {
+				struct dive *top = d;
+				struct dive *bottom = d;
+				if (d->selected) {
+					if (currentOrder == Qt::AscendingOrder) {
+						top = first_selected_dive();
+						bottom = last_selected_dive();
+					} else {
+						top = last_selected_dive();
+						bottom = first_selected_dive();
+					}
+				}
+				if (is_trip_before_after(top, (currentOrder == Qt::AscendingOrder)))
+					popup.addAction(tr("Add dive(s) to trip immediately above"), this, SLOT(addToTripAbove()));
+				if (is_trip_before_after(bottom, (currentOrder == Qt::DescendingOrder)))
+					popup.addAction(tr("Add dive(s) to trip immediately below"), this, SLOT(addToTripBelow()));
+			}
 		}
 		if (trip) {
-			popup.addAction(tr("merge trip with trip above"), this, SLOT(mergeTripAbove()));
-			popup.addAction(tr("merge trip with trip below"), this, SLOT(mergeTripBelow()));
+			popup.addAction(tr("Merge trip with trip above"), this, SLOT(mergeTripAbove()));
+			popup.addAction(tr("Merge trip with trip below"), this, SLOT(mergeTripBelow()));
 		}
 	}
 	if (d) {
-		popup.addAction(tr("delete dive(s)"), this, SLOT(deleteDive()));
+		popup.addAction(tr("Delete dive(s)"), this, SLOT(deleteDive()));
 #if 0
-		popup.addAction(tr("mark dive(s) invalid", this, SLOT(markDiveInvalid())));
+		popup.addAction(tr("Mark dive(s) invalid", this, SLOT(markDiveInvalid())));
 #endif
 	}
 	if (amount_selected > 1 && consecutive_selected())
-		popup.addAction(tr("merge selected dives"), this, SLOT(mergeDives()));
+		popup.addAction(tr("Merge selected dives"), this, SLOT(mergeDives()));
 	if (amount_selected >= 1) {
-		popup.addAction(tr("save As"), this, SLOT(saveSelectedDivesAs()));
-		popup.addAction(tr("export As UDDF"), this, SLOT(exportSelectedDivesAsUDDF()));
-		popup.addAction(tr("shift times"), this, SLOT(shiftTimes()));
+		popup.addAction(tr("Renumber dive(s)"), this, SLOT(renumberDives()));
+		popup.addAction(tr("Shift times"), this, SLOT(shiftTimes()));
+		popup.addAction(tr("Load images"), this, SLOT(loadImages()));
 	}
-	if (d)
-		popup.addAction(tr("upload dive(s) to divelogs.de"), this, SLOT(uploadToDivelogsDE()));
 	// "collapse all" really closes all trips,
 	// "collapse" keeps the trip with the selected dive open
-	QAction * actionTaken = popup.exec(event->globalPos());
+	QAction *actionTaken = popup.exec(event->globalPos());
 	if (actionTaken == collapseAction && collapseAction) {
 		this->setAnimated(false);
 		selectDive(selected_dive, true);
@@ -742,51 +758,77 @@ void DiveListView::contextMenuEvent(QContextMenuEvent *event)
 	event->accept();
 }
 
-void DiveListView::saveSelectedDivesAs()
-{
-	QSettings settings;
-	QString lastDir = QDir::homePath();
-
-	settings.beginGroup("FileDialog");
-	if (settings.contains("LastDir")) {
-		if(QDir::setCurrent(settings.value("LastDir").toString())) {
-			lastDir = settings.value("LastDir").toString();
-		}
-	}
-	settings.endGroup();
-
-	QString fileName = QFileDialog::getSaveFileName(mainWindow(), tr("Save Dives As..."), QDir::homePath());
-	if (fileName.isEmpty())
-		return;
-
-	// Keep last open dir
-	QFileInfo fileInfo(fileName);
-	settings.beginGroup("FileDialog");
-	settings.setValue("LastDir",fileInfo.dir().path());
-	settings.endGroup();
-
-	QByteArray bt = QFile::encodeName(fileName);
-	save_dives_logic(bt.data(), TRUE);
-}
-
-void DiveListView::exportSelectedDivesAsUDDF()
-{
-	QString filename;
-	QFileInfo fi(system_default_filename());
-
-	filename = QFileDialog::getSaveFileName(this, tr("Save File as"), fi.absolutePath(),
-						tr("UDDF files (*.uddf *.UDDF)"));
-	if (!filename.isNull() && !filename.isEmpty())
-		export_dives_uddf(filename.toUtf8(), true);
-}
-
-
 void DiveListView::shiftTimes()
 {
 	ShiftTimesDialog::instance()->show();
 }
 
-void DiveListView::uploadToDivelogsDE()
+void DiveListView::loadImages()
 {
-	DivelogsDeWebServices::instance()->prepareDivesForUpload();
+	QStringList fileNames = QFileDialog::getOpenFileNames(this, tr("Open image files"), lastUsedImageDir(), tr("Image files (*.jpg *.jpeg *.pnm *.tif *.tiff)"));
+	if (fileNames.isEmpty())
+		return;
+
+	updateLastUsedImageDir(QFileInfo(fileNames[0]).dir().path());
+	ShiftImageTimesDialog shiftDialog(this);
+	shiftDialog.setOffset(lastImageTimeOffset());
+	shiftDialog.exec();
+	updateLastImageTimeOffset(shiftDialog.amount());
+
+	Q_FOREACH(const QString& fileName, fileNames) {
+		int j = 0;
+		struct dive *dive;
+		for_each_dive (j, dive) {
+			if (!dive->selected)
+				continue;
+			dive_create_picture(dive, qstrdup(fileName.toUtf8().data()), shiftDialog.amount());
+		}
+	}
+
+	mark_divelist_changed(true);
+	copy_dive(current_dive, &displayed_dive);
+	DivePictureModel::instance()->updateDivePictures();
+}
+
+QString DiveListView::lastUsedImageDir()
+{
+	QSettings settings;
+	QString lastImageDir = QDir::homePath();
+
+	settings.beginGroup("FileDialog");
+	if (settings.contains("LastImageDir"))
+		if (QDir::setCurrent(settings.value("LastImageDir").toString()))
+			lastImageDir = settings.value("LastIamgeDir").toString();
+	return lastImageDir;
+}
+
+void DiveListView::updateLastUsedImageDir(const QString &dir)
+{
+	QSettings s;
+	s.beginGroup("FileDialog");
+	s.setValue("LastImageDir", dir);
+}
+
+int DiveListView::lastImageTimeOffset()
+{
+	QSettings settings;
+	int offset = 0;
+
+	settings.beginGroup("MainWindow");
+	if (settings.contains("LastImageTimeOffset"))
+		offset = settings.value("LastImageTimeOffset").toInt();
+	return offset;
+}
+
+void DiveListView::updateLastImageTimeOffset(const int offset)
+{
+	QSettings s;
+	s.beginGroup("MainWindow");
+	s.setValue("LastImageTimeOffset", offset);
+}
+
+void DiveListView::endSearch()
+{
+	searchBox.clear();
+	searchBox.hide();
 }
