@@ -1,9 +1,10 @@
 #include "downloadfromdivecomputer.h"
-
+#include "../divecomputer.h"
 #include "../libdivecomputer.h"
 #include "../helpers.h"
 #include "../display.h"
 #include "../divelist.h"
+
 #include "mainwindow.h"
 #include <cstdlib>
 #include <QThread>
@@ -12,6 +13,7 @@
 #include <QTimer>
 #include <QFileDialog>
 #include <QMessageBox>
+#include <QShortcut>
 
 struct product {
 	const char *product;
@@ -32,27 +34,27 @@ struct mydescriptor {
 	unsigned int model;
 };
 
-namespace DownloadFromDcGlobal{
+namespace DownloadFromDcGlobal {
 	const char *err_string;
 };
 
-DownloadFromDCWidget *DownloadFromDCWidget::instance()
-{
-	static DownloadFromDCWidget *dialog = new DownloadFromDCWidget(mainWindow());
-	dialog->setAttribute(Qt::WA_QuitOnClose, false);
-	return dialog;
-}
-
-DownloadFromDCWidget::DownloadFromDCWidget(QWidget* parent, Qt::WindowFlags f) :
-	QDialog(parent, f), thread(0), timer(new QTimer(this)),
-	dumpWarningShown(false), currentState(INITIAL)
+DownloadFromDCWidget::DownloadFromDCWidget(QWidget *parent, Qt::WindowFlags f) : QDialog(parent, f),
+	thread(0),
+	downloading(false),
+	previousLast(0),
+	vendorModel(0),
+	productModel(0),
+	timer(new QTimer(this)),
+	dumpWarningShown(false),
+	currentState(INITIAL)
 {
 	ui.setupUi(this);
 	ui.progressBar->hide();
 	ui.progressBar->setMinimum(0);
 	ui.progressBar->setMaximum(100);
 
-	fill_device_list();
+	progress_bar_text = "";
+
 	fill_computer_list();
 
 	ui.chooseDumpFile->setEnabled(ui.dumpToFile->isChecked());
@@ -70,24 +72,27 @@ DownloadFromDCWidget::DownloadFromDCWidget(QWidget* parent, Qt::WindowFlags f) :
 		if (default_dive_computer_product)
 			ui.product->setCurrentIndex(ui.product->findText(default_dive_computer_product));
 	}
-	connect(ui.product, SIGNAL(currentIndexChanged(int)), this, SLOT(on_product_currentIndexChanged()), Qt::UniqueConnection);
 	if (default_dive_computer_device)
 		ui.device->setEditText(default_dive_computer_device);
 
 	timer->setInterval(200);
 	connect(timer, SIGNAL(timeout()), this, SLOT(updateProgressBar()));
 	updateState(INITIAL);
-}
-
-void DownloadFromDCWidget::runDialog()
-{
-	updateState(INITIAL);
-	exec();
+	memset(&data, 0, sizeof(data));
+	QShortcut *close = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_W), this);
+	connect(close, SIGNAL(activated()), this, SLOT(close()));
+	QShortcut *quit = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q), this);
+	connect(quit, SIGNAL(activated()), parent, SLOT(close()));
 }
 
 void DownloadFromDCWidget::updateProgressBar()
 {
-	ui.progressBar->setValue(progress_bar_fraction *100);
+	if (*progress_bar_text != '\0') {
+		ui.progressBar->setFormat(progress_bar_text);
+	} else {
+		ui.progressBar->setFormat("%p%");
+	}
+	ui.progressBar->setValue(progress_bar_fraction * 100);
 }
 
 void DownloadFromDCWidget::updateState(states state)
@@ -96,7 +101,7 @@ void DownloadFromDCWidget::updateState(states state)
 		return;
 
 	if (state == INITIAL) {
-		fill_device_list();
+		fill_device_list(DC_TYPE_OTHER);
 		ui.progressBar->hide();
 		markChildrenAsEnabled();
 		timer->stop();
@@ -110,8 +115,7 @@ void DownloadFromDCWidget::updateState(states state)
 
 	// user pressed cancel but the application isn't doing anything.
 	// means close the window
-	else if ((currentState == INITIAL || currentState == CANCELLED || currentState == DONE || currentState == ERROR)
-		&& state == CANCELLING) {
+	else if ((currentState == INITIAL || currentState == CANCELLED || currentState == DONE || currentState == ERROR) && state == CANCELLING) {
 		timer->stop();
 		reject();
 		ui.ok->setText(tr("OK"));
@@ -126,19 +130,30 @@ void DownloadFromDCWidget::updateState(states state)
 		markChildrenAsEnabled();
 	}
 
-	// DOWNLOAD is finally done, close the dialog and go back to the main window
+	// DOWNLOAD is finally done, but we don't know if there was an error as libdivecomputer doesn't pass
+	// that information on to us
+	// so check the progressBar text and if no error was reported, close the dialog and go back to the main window
+	// otherwise treat this as if the download was cancelled
 	else if (currentState == DOWNLOADING && state == DONE) {
 		timer->stop();
-		ui.progressBar->setValue(100);
-		markChildrenAsEnabled();
-		ui.ok->setText(tr("OK"));
-		accept();
+		if (QString(progress_bar_text).contains("error", Qt::CaseInsensitive)) {
+			updateProgressBar();
+			markChildrenAsEnabled();
+			progress_bar_text = "";
+			ui.ok->setText(tr("Retry"));
+		} else {
+			ui.progressBar->setValue(100);
+			markChildrenAsEnabled();
+			ui.ok->setText(tr("OK"));
+			accept();
+		}
 	}
 
 	// DOWNLOAD is started.
 	else if (state == DOWNLOADING) {
 		timer->start();
 		ui.progressBar->setValue(0);
+		updateProgressBar();
 		ui.progressBar->show();
 		markChildrenAsDisabled();
 	}
@@ -156,8 +171,9 @@ void DownloadFromDCWidget::updateState(states state)
 	currentState = state;
 }
 
-void DownloadFromDCWidget::on_vendor_currentIndexChanged(const QString& vendor)
+void DownloadFromDCWidget::on_vendor_currentIndexChanged(const QString &vendor)
 {
+	int dcType = DC_TYPE_SERIAL;
 	QAbstractItemModel *currentModel = ui.product->model();
 	if (!currentModel)
 		return;
@@ -165,15 +181,19 @@ void DownloadFromDCWidget::on_vendor_currentIndexChanged(const QString& vendor)
 	productModel = new QStringListModel(productList[vendor]);
 	ui.product->setModel(productModel);
 
+	if (vendor == QString("Uemis"))
+		dcType = DC_TYPE_UEMIS;
+	fill_device_list(dcType);
+
 	// Memleak - but deleting gives me a crash.
 	//currentModel->deleteLater();
 }
 
-void DownloadFromDCWidget::on_product_currentIndexChanged()
+void DownloadFromDCWidget::on_product_currentIndexChanged(const QString &product)
 {
 	// Set up the DC descriptor
 	dc_descriptor_t *descriptor = NULL;
-	descriptor = descriptorLookup[ui.vendor->currentText() + ui.product->currentText()];
+	descriptor = descriptorLookup[ui.vendor->currentText() + product];
 
 	// call dc_descriptor_get_transport to see if the dc_transport_t is DC_TRANSPORT_SERIAL
 	if (dc_descriptor_get_transport(descriptor) == DC_TRANSPORT_SERIAL) {
@@ -193,7 +213,7 @@ void DownloadFromDCWidget::fill_computer_list()
 
 	QStringList computer;
 	dc_descriptor_iterator(&iterator);
-	while (dc_iterator_next (iterator, &descriptor) == DC_STATUS_SUCCESS) {
+	while (dc_iterator_next(iterator, &descriptor) == DC_STATUS_SUCCESS) {
 		const char *vendor = dc_descriptor_get_vendor(descriptor);
 		const char *product = dc_descriptor_get_product(descriptor);
 
@@ -213,7 +233,7 @@ void DownloadFromDCWidget::fill_computer_list()
 	   this WILL BREAK if libdivecomputer changes the dc_descriptor struct...
 	   eventually the UEMIS code needs to move into libdivecomputer, I guess */
 
-	mydescriptor = (struct mydescriptor*) malloc(sizeof(struct mydescriptor));
+	mydescriptor = (struct mydescriptor *)malloc(sizeof(struct mydescriptor));
 	mydescriptor->vendor = "Uemis";
 	mydescriptor->product = "Zurich";
 	mydescriptor->type = DC_FAMILY_NULL;
@@ -225,7 +245,22 @@ void DownloadFromDCWidget::fill_computer_list()
 	if (!productList["Uemis"].contains("Zurich"))
 		productList["Uemis"].push_back("Zurich");
 
-	descriptorLookup[QString("UemisZurich")] = (dc_descriptor_t *)mydescriptor;
+	descriptorLookup["UemisZurich"] = (dc_descriptor_t *)mydescriptor;
+
+	qSort(vendorList);
+}
+
+void DownloadFromDCWidget::on_search_clicked()
+{
+	if (ui.vendor->currentText() == "Uemis") {
+		QString dirName = QFileDialog::getExistingDirectory(this,
+								    tr("Find Uemis dive computer"),
+								    QDir::homePath(),
+								    QFileDialog::ShowDirsOnly);
+		if (ui.device->findText(dirName) == -1)
+			ui.device->addItem(dirName);
+		ui.device->setEditText(dirName);
+	}
 }
 
 void DownloadFromDCWidget::on_cancel_clicked()
@@ -256,9 +291,9 @@ void DownloadFromDCWidget::on_ok_clicked()
 	thread = new DownloadThread(this, &data);
 
 	connect(thread, SIGNAL(finished()),
-			this, SLOT(onDownloadThreadFinished()), Qt::QueuedConnection);
+		this, SLOT(onDownloadThreadFinished()), Qt::QueuedConnection);
 
-	MainWindow *w = mainWindow();
+	MainWindow *w = MainWindow::instance();
 	connect(thread, SIGNAL(finished()), w, SLOT(refreshDisplay()));
 
 	// before we start, remember where the dive_table ended
@@ -283,14 +318,13 @@ void DownloadFromDCWidget::checkLogFile(int state)
 
 void DownloadFromDCWidget::pickLogFile()
 {
-	QString filename = existing_filename ? : prefs.default_filename;
+	QString filename = existing_filename ?: prefs.default_filename;
 	QFileInfo fi(filename);
 	filename = fi.absolutePath().append(QDir::separator()).append("subsurface.log");
 	logFile = QFileDialog::getSaveFileName(this, tr("Choose file for divecomputer download logfile"),
 					       filename, tr("Log files (*.log)"));
 	if (!logFile.isEmpty()) {
-		if (logfile_name)
-			free(logfile_name);
+		free(logfile_name);
 		logfile_name = strdup(logFile.toUtf8().data());
 	}
 }
@@ -312,14 +346,13 @@ void DownloadFromDCWidget::checkDumpFile(int state)
 
 void DownloadFromDCWidget::pickDumpFile()
 {
-	QString filename = existing_filename ? : prefs.default_filename;
+	QString filename = existing_filename ?: prefs.default_filename;
 	QFileInfo fi(filename);
 	filename = fi.absolutePath().append(QDir::separator()).append("subsurface.bin");
 	dumpFile = QFileDialog::getSaveFileName(this, tr("Choose file for divecomputer binary dump file"),
 						filename, tr("Dump files (*.bin)"));
 	if (!dumpFile.isEmpty()) {
-		if (dumpfile_name)
-			free(dumpfile_name);
+		free(dumpfile_name);
 		dumpfile_name = strdup(dumpFile.toUtf8().data());
 	}
 }
@@ -347,10 +380,29 @@ void DownloadFromDCWidget::onDownloadThreadFinished()
 			// down in the dive_table
 			for (int i = dive_table.nr - 1; i >= previousLast; i--)
 				delete_single_dive(i);
-		} else {
-			process_dives(TRUE, preferDownloaded());
+		} else if (dive_table.nr) {
+			int uniqId, idx;
+			// remember the last downloaded dive (on most dive computers this will be the chronologically
+			// first new dive) and select it again after processing all the dives
+			MainWindow::instance()->dive_list()->unselectDives();
+			uniqId = get_dive(dive_table.nr - 1)->id;
+			process_dives(true, preferDownloaded());
+			// after process_dives does any merging or resorting needed, we need
+			// to recreate the model for the dive list so we can select the newest dive
+			MainWindow::instance()->recreateDiveList();
+			idx = get_idx_by_uniq_id(uniqId);
+			// this shouldn't be necessary - but there are reports that somehow existing dives stay selected
+			// (but not visible as selected)
+			MainWindow::instance()->dive_list()->unselectDives();
+			MainWindow::instance()->dive_list()->selectDive(idx, true);
+		}
+	} else if (currentState == CANCELLING || currentState == CANCELLED) {
+		if (import_thread_cancelled) {
+			// walk backwards so we don't keep moving the dives
+			// down in the dive_table
+			for (int i = dive_table.nr - 1; i >= previousLast; i--)
+				delete_single_dive(i);
 		}
-	} else {
 		updateState(CANCELLED);
 	}
 }
@@ -364,6 +416,10 @@ void DownloadFromDCWidget::markChildrenAsDisabled()
 	ui.preferDownloaded->setDisabled(true);
 	ui.ok->setDisabled(true);
 	ui.search->setDisabled(true);
+	ui.logToFile->setDisabled(true);
+	ui.dumpToFile->setDisabled(true);
+	ui.chooseLogFile->setDisabled(true);
+	ui.chooseDumpFile->setDisabled(true);
 }
 
 void DownloadFromDCWidget::markChildrenAsEnabled()
@@ -376,6 +432,10 @@ void DownloadFromDCWidget::markChildrenAsEnabled()
 	ui.ok->setDisabled(false);
 	ui.cancel->setDisabled(false);
 	ui.search->setDisabled(false);
+	ui.logToFile->setDisabled(false);
+	ui.dumpToFile->setDisabled(false);
+	ui.chooseLogFile->setDisabled(false);
+	ui.chooseDumpFile->setDisabled(false);
 }
 
 static void fillDeviceList(const char *name, void *data)
@@ -384,16 +444,16 @@ static void fillDeviceList(const char *name, void *data)
 	comboBox->addItem(name);
 }
 
-void DownloadFromDCWidget::fill_device_list()
+void DownloadFromDCWidget::fill_device_list(int dc_type)
 {
 	int deviceIndex;
 	ui.device->clear();
-	deviceIndex = enumerate_devices(fillDeviceList, ui.device);
+	deviceIndex = enumerate_devices(fillDeviceList, ui.device, dc_type);
 	if (deviceIndex >= 0)
 		ui.device->setCurrentIndex(deviceIndex);
 }
 
-DownloadThread::DownloadThread(QObject* parent, device_data_t* data): QThread(parent),
+DownloadThread::DownloadThread(QObject *parent, device_data_t *data) : QThread(parent),
 	data(data)
 {
 }
@@ -402,7 +462,7 @@ static QString str_error(const char *fmt, ...)
 {
 	va_list args;
 	va_start(args, fmt);
-	const QString str = QString().vsprintf( fmt, args );
+	const QString str = QString().vsprintf(fmt, args);
 	va_end(args);
 
 	return str;
@@ -410,12 +470,12 @@ static QString str_error(const char *fmt, ...)
 
 void DownloadThread::run()
 {
-	const char *error;
+	const char *errorText;
 	import_thread_cancelled = false;
 	if (!strcmp(data->vendor, "Uemis"))
-		error = do_uemis_import(data->devname, data->force_download);
+		errorText = do_uemis_import(data->devname, data->force_download);
 	else
-		error = do_libdivecomputer_import(data);
-	if (error)
-		this->error =  str_error(error, data->devname, data->vendor, data->product);
+		errorText = do_libdivecomputer_import(data);
+	if (errorText)
+		error = str_error(errorText, data->devname, data->vendor, data->product);
 }
