@@ -1,6 +1,7 @@
 #include "subsurfacewebservices.h"
-#include "../webservice.h"
+#include "webservice.h"
 #include "mainwindow.h"
+#include "usersurvey.h"
 #include <libxml/parser.h>
 #include <zip.h>
 #include <errno.h>
@@ -14,82 +15,87 @@
 #include <QSettings>
 #include <QXmlStreamReader>
 #include <qdesktopservices.h>
+#include <QShortcut>
 
-#include "../dive.h"
-#include "../divelist.h"
+#include "dive.h"
+#include "divelist.h"
+#include "display.h"
 
 #ifdef Q_OS_UNIX
-#  include <unistd.h> // for dup(2)
+#include <unistd.h> // for dup(2)
+#endif
+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+#include <QUrlQuery>
 #endif
 
 struct dive_table gps_location_table;
 static bool merge_locations_into_dives(void);
 
-static bool is_automatic_fix(struct dive *gpsfix)
-{
-	if (gpsfix && gpsfix->location &&
-			(!strcmp(gpsfix->location, "automatic fix") ||
-			 !strcmp(gpsfix->location, "Auto-created dive")))
-		return TRUE;
-	return FALSE;
-}
-
-#define SAME_GROUP 6 * 3600   // six hours
-
+#define SAME_GROUP 6 * 3600 // six hours
+//TODO: C Code. static functions are not good if we plan to have a test for them.
 static bool merge_locations_into_dives(void)
 {
-	int i, nr = 0, changed = 0;
-	struct dive *gpsfix, *last_named_fix = NULL, *dive;
+	int i, j, tracer=0, changed=0;
+	struct dive *gpsfix, *nextgpsfix, *dive;
 
 	sort_table(&gps_location_table);
 
-	for_each_gps_location(i, gpsfix) {
-		if (is_automatic_fix(gpsfix)) {
-			dive = find_dive_including(gpsfix->when);
-			if (dive && !dive_has_gps_location(dive)) {
-#if DEBUG_WEBSERVICE
-				struct tm tm;
-				utc_mkdate(gpsfix->when, &tm);
-				printf("found dive named %s @ %04d-%02d-%02d %02d:%02d:%02d\n",
-				       gpsfix->location,
-				       tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday,
-				       tm.tm_hour, tm.tm_min, tm.tm_sec);
-#endif
-				changed++;
-				copy_gps_location(gpsfix, dive);
-			}
-		} else {
-			if (last_named_fix && dive_within_time_range(last_named_fix, gpsfix->when, SAME_GROUP)) {
-				nr++;
-			} else {
-				nr = 1;
-				last_named_fix = gpsfix;
-			}
-			dive = find_dive_n_near(gpsfix->when, nr, SAME_GROUP);
-			if (dive) {
-				if (!dive_has_gps_location(dive)) {
-					copy_gps_location(gpsfix, dive);
-					changed++;
+	for_each_dive (i, dive) {
+		if (!dive_has_gps_location(dive)) {
+			for (j = tracer; (gpsfix = get_gps_location(j, &gps_location_table)) !=NULL; j++) {
+				if (dive_within_time_range (dive, gpsfix->when, SAME_GROUP)) {
+					/*
+					 * If position is fixed during dive. This is the good one.
+					 * Asign and mark position, and end gps_location loop
+					 */
+					if ((dive->when <= gpsfix->when && gpsfix->when <= dive->when + dive->duration.seconds)) {
+						copy_gps_location(gpsfix,dive);
+						changed++;
+						tracer = j;
+						break;
+					} else {
+						/*
+						 * If it is not, check if there are more position fixes in SAME_GROUP range
+						 */
+						if ((nextgpsfix = get_gps_location(j+1,&gps_location_table)) &&
+						    dive_within_time_range (dive, nextgpsfix->when, SAME_GROUP)) {
+							/*
+							 * If distance from gpsfix to end of dive is shorter than distance between
+							 * gpsfix and nextgpsfix, gpsfix is the good one. Asign, mark and end loop.
+							 * If not, simply fail and nextgpsfix will be evaluated in next iteration.
+							 */
+							if ((dive->when + dive->duration.seconds - gpsfix->when) < (nextgpsfix->when - gpsfix->when)) {
+								copy_gps_location(gpsfix,dive);
+								tracer = j;
+								break;
+							}
+						/*
+						 * If no more positions in range, the actual is the one. Asign, mark and end loop.
+						 */
+						} else {
+							copy_gps_location(gpsfix,dive);
+							changed++;
+							tracer = j;
+							break;
+						}
+					}
+				} else {
+					/* If position is out of SAME_GROUP range and in the future, mark position for
+					 * next dive iteration and end the gps_location loop
+					 */
+					if (gpsfix->when >= dive->when + dive->duration.seconds + SAME_GROUP) {
+						tracer = j;
+						break;
+					}
 				}
-				if (!dive->location) {
-					dive->location = strdup(gpsfix->location);
-					changed++;
-				}
-			} else {
-				struct tm tm;
-				utc_mkdate(gpsfix->when, &tm);
-#if DEBUG_WEBSERVICE
-				printf("didn't find dive matching gps fix named %s @ %04d-%02d-%02d %02d:%02d:%02d\n",
-				       gpsfix->location,
-				       tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday,
-				       tm.tm_hour, tm.tm_min, tm.tm_sec);
-#endif
 			}
 		}
 	}
 	return changed > 0;
 }
 
+//TODO: C-code.
 static void clear_table(struct dive_table *table)
 {
 	int i;
@@ -98,11 +104,12 @@ static void clear_table(struct dive_table *table)
 	table->nr = 0;
 }
 
-bool DivelogsDeWebServices::prepare_dives_for_divelogs(const QString &tempfile, const bool selected, QString *errorMsg)
+// TODO: This looks like should be ported to C code. or a big part of it.
+bool DivelogsDeWebServices::prepare_dives_for_divelogs(const QString &tempfile, const bool selected)
 {
 	static const char errPrefix[] = "divelog.de-upload:";
 	if (!amount_selected) {
-		*errorMsg = tr("no dives were selected");
+		report_error(tr("no dives were selected").toUtf8());
 		return false;
 	}
 
@@ -117,17 +124,18 @@ bool DivelogsDeWebServices::prepare_dives_for_divelogs(const QString &tempfile,
 
 
 	int error_code;
-	zip = zip_open(QFile::encodeName(tempfile), ZIP_CREATE, &error_code);
+	zip = zip_open(QFile::encodeName(QDir::toNativeSeparators(tempfile)), ZIP_CREATE, &error_code);
 	if (!zip) {
 		char buffer[1024];
 		zip_error_to_str(buffer, sizeof buffer, error_code, errno);
-		*errorMsg = tr("failed to create zip file for upload: %1")
-			    .arg(QString::fromLocal8Bit(buffer));
+		report_error(tr("failed to create zip file for upload: %s").toUtf8(), buffer);
 		return false;
 	}
 
 	/* walk the dive list in chronological order */
-	for (int i = 0; i < dive_table.nr; i++) {
+	int i;
+	struct dive *dive;
+	for_each_dive (i, dive) {
 		FILE *f;
 		char filename[PATH_MAX];
 		int streamsize;
@@ -139,14 +147,14 @@ bool DivelogsDeWebServices::prepare_dives_for_divelogs(const QString &tempfile,
 		 * Get the i'th dive in XML format so we can process it.
 		 * We need to save to a file before we can reload it back into memory...
 		 */
-		struct dive *dive = get_dive(i);
-		if (!dive)
-			continue;
 		if (selected && !dive->selected)
 			continue;
-		f = tmpfile();
+		QString innerTmpFile = tempfile;
+		QString tmpSuffix = QString::number(qrand() % 9999) + ".tmp";
+		innerTmpFile.replace(".dld", tmpSuffix);
+		f = subsurface_fopen(QFile::encodeName(QDir::toNativeSeparators(innerTmpFile)), "w+");
 		if (!f) {
-			*errorMsg = tr("cannot create temporary file: %1").arg(qt_error_string());
+			report_error(tr("cannot create temporary file: %s").toUtf8(), qt_error_string().toUtf8().data());
 			goto error_close_zip;
 		}
 		save_dive(f, dive);
@@ -155,15 +163,15 @@ bool DivelogsDeWebServices::prepare_dives_for_divelogs(const QString &tempfile,
 		rewind(f);
 
 		membuf = (char *)malloc(streamsize + 1);
-		if (!membuf || !fread(membuf, streamsize, 1, f)) {
-			*errorMsg = tr("internal error: %1").arg(qt_error_string());
+		if (!membuf || (streamsize = fread(membuf, 1, streamsize, f)) == 0) {
+			report_error(tr("internal error: %s").toUtf8(), qt_error_string().toUtf8().data());
 			fclose(f);
 			free((void *)membuf);
 			goto error_close_zip;
 		}
 		membuf[streamsize] = 0;
 		fclose(f);
-
+		unlink(QFile::encodeName(QDir::toNativeSeparators(innerTmpFile)));
 		/*
 		 * Parse the memory buffer into XML document and
 		 * transform it to divelogs.de format, finally dumping
@@ -172,14 +180,14 @@ bool DivelogsDeWebServices::prepare_dives_for_divelogs(const QString &tempfile,
 		xmlDoc *doc = xmlReadMemory(membuf, streamsize, "divelog", NULL, 0);
 		if (!doc) {
 			qWarning() << errPrefix << "could not parse back into memory the XML file we've just created!";
-			*errorMsg = tr("internal error");
+			report_error(tr("internal error").toUtf8());
 			free((void *)membuf);
 			goto error_close_zip;
 		}
 		free((void *)membuf);
 
 		transformed = xsltApplyStylesheet(xslt, doc, NULL);
-		xmlDocDumpMemory(transformed, (xmlChar **) &membuf, &streamsize);
+		xmlDocDumpMemory(transformed, (xmlChar **)&membuf, &streamsize);
 		xmlFreeDoc(doc);
 		xmlFreeDoc(transformed);
 
@@ -205,17 +213,17 @@ error_close_zip:
 	return false;
 }
 
-WebServices::WebServices(QWidget* parent, Qt::WindowFlags f): QDialog(parent, f)
-, reply(0)
+WebServices::WebServices(QWidget *parent, Qt::WindowFlags f) : QDialog(parent, f), reply(0)
 {
 	ui.setupUi(this);
-	connect(ui.buttonBox, SIGNAL(clicked(QAbstractButton*)), this, SLOT(buttonClicked(QAbstractButton*)));
+	connect(ui.buttonBox, SIGNAL(clicked(QAbstractButton *)), this, SLOT(buttonClicked(QAbstractButton *)));
 	connect(ui.download, SIGNAL(clicked(bool)), this, SLOT(startDownload()));
 	connect(ui.upload, SIGNAL(clicked(bool)), this, SLOT(startUpload()));
 	connect(&timeout, SIGNAL(timeout()), this, SLOT(downloadTimedOut()));
 	ui.buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
 	timeout.setSingleShot(true);
 	defaultApplyText = ui.buttonBox->button(QDialogButtonBox::Apply)->text();
+	userAgent = UserSurvey::getUserAgent();
 }
 
 void WebServices::hidePassword()
@@ -282,8 +290,8 @@ void WebServices::connectSignalsForDownload(QNetworkReply *reply)
 	connect(reply, SIGNAL(finished()), this, SLOT(downloadFinished()));
 	connect(reply, SIGNAL(error(QNetworkReply::NetworkError)),
 		this, SLOT(downloadError(QNetworkReply::NetworkError)));
-	connect(reply, SIGNAL(downloadProgress(qint64,qint64)), this,
-		SLOT(updateProgress(qint64,qint64)));
+	connect(reply, SIGNAL(downloadProgress(qint64, qint64)), this,
+		SLOT(updateProgress(qint64, qint64)));
 
 	timeout.start(30000); // 30s
 }
@@ -295,7 +303,7 @@ void WebServices::resetState()
 	ui.userID->setEnabled(true);
 	ui.password->setEnabled(true);
 	ui.progressBar->reset();
-	ui.progressBar->setRange(0,1);
+	ui.progressBar->setRange(0, 1);
 	ui.status->setText(QString());
 	ui.buttonBox->button(QDialogButtonBox::Apply)->setText(defaultApplyText);
 }
@@ -306,44 +314,64 @@ void WebServices::resetState()
 // #
 // #
 
-SubsurfaceWebServices* SubsurfaceWebServices::instance()
-{
-	static SubsurfaceWebServices *self = new SubsurfaceWebServices(mainWindow());
-	self->setAttribute(Qt::WA_QuitOnClose, false);
-	return self;
-}
-
-SubsurfaceWebServices::SubsurfaceWebServices(QWidget* parent, Qt::WindowFlags f)
+SubsurfaceWebServices::SubsurfaceWebServices(QWidget *parent, Qt::WindowFlags f) : WebServices(parent, f)
 {
 	QSettings s;
-	ui.userID->setText(s.value("subsurface_webservice_uid").toString().toUpper());
+	if (!prefs.save_userid_local || !*prefs.userid)
+		ui.userID->setText(s.value("subsurface_webservice_uid").toString().toUpper());
+	else
+		ui.userID->setText(prefs.userid);
 	hidePassword();
 	hideUpload();
+	ui.progressBar->setFormat(tr("Enter User ID and click Download"));
+	ui.progressBar->setRange(0, 1);
+	ui.progressBar->setValue(-1);
+	ui.progressBar->setAlignment(Qt::AlignCenter);
+	ui.saveUidLocal->setChecked(prefs.save_userid_local);
+	QShortcut *close = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_W), this);
+	connect(close, SIGNAL(activated()), this, SLOT(close()));
+	QShortcut *quit = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q), this);
+	connect(quit, SIGNAL(activated()), parent, SLOT(close()));
 }
 
-void SubsurfaceWebServices::buttonClicked(QAbstractButton* button)
+void SubsurfaceWebServices::buttonClicked(QAbstractButton *button)
 {
 	ui.buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
-	switch(ui.buttonBox->buttonRole(button)){
-	case QDialogButtonBox::ApplyRole:{
+	switch (ui.buttonBox->buttonRole(button)) {
+	case QDialogButtonBox::ApplyRole: {
 		clear_table(&gps_location_table);
 		QByteArray url = tr("Webservice").toLocal8Bit();
-		parse_xml_buffer(url.data(), downloadedData.data(), downloadedData.length(), &gps_location_table, NULL, NULL);
+		parse_xml_buffer(url.data(), downloadedData.data(), downloadedData.length(), &gps_location_table, NULL);
 
 		/* now merge the data in the gps_location table into the dive_table */
 		if (merge_locations_into_dives()) {
-			mark_divelist_changed(TRUE);
+			mark_divelist_changed(true);
+			MainWindow::instance()->globe()->repopulateLabels();
+			MainWindow::instance()->globe()->centerOnCurrentDive();
+			MainWindow::instance()->information()->updateDiveInfo();
 		}
 
 		/* store last entered uid in config */
 		QSettings s;
-		s.setValue("subsurface_webservice_uid", ui.userID->text().toUpper());
+		QString qDialogUid = ui.userID->text().toUpper();
+		bool qSaveUid = ui.saveUidLocal->checkState();
+		set_save_userid_local(qSaveUid);
+		if (qSaveUid) {
+			QString qSettingUid = s.value("subsurface_webservice_uid").toString();
+			QString qFileUid = QString(prefs.userid);
+			bool s_eq_d = (qSettingUid == qDialogUid);
+			bool d_eq_f = (qDialogUid == qFileUid);
+			if (!d_eq_f || s_eq_d)
+				s.setValue("subsurface_webservice_uid", qDialogUid);
+			set_userid(qDialogUid.toLocal8Bit().data());
+		} else {
+			s.setValue("subsurface_webservice_uid", qDialogUid);
+		}
 		s.sync();
 		hide();
 		close();
 		resetState();
-	}
-		break;
+	} break;
 	case QDialogButtonBox::RejectRole:
 		if (reply != NULL && reply->isOpen()) {
 			reply->abort();
@@ -363,14 +391,22 @@ void SubsurfaceWebServices::buttonClicked(QAbstractButton* button)
 void SubsurfaceWebServices::startDownload()
 {
 	QUrl url("http://api.hohndel.org/api/dive/get/");
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 	url.addQueryItem("login", ui.userID->text().toUpper());
+#else
+	QUrlQuery query;
+	query.addQueryItem("login", ui.userID->text().toUpper());
+	url.setQuery(query);
+#endif
 
 	QNetworkRequest request;
 	request.setUrl(url);
 	request.setRawHeader("Accept", "text/xml");
+	request.setRawHeader("User-Agent", userAgent.toUtf8());
 	reply = manager()->get(request);
 	ui.status->setText(tr("Connecting..."));
-	ui.progressBar->setRange(0,0); // this makes the progressbar do an 'infinite spin'
+	ui.progressBar->setEnabled(true);
+	ui.progressBar->setRange(0, 0); // this makes the progressbar do an 'infinite spin'
 	ui.download->setEnabled(false);
 	ui.buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
 	connectSignalsForDownload(reply);
@@ -381,7 +417,9 @@ void SubsurfaceWebServices::downloadFinished()
 	if (!reply)
 		return;
 
-	ui.progressBar->setRange(0,1);
+	ui.progressBar->setRange(0, 1);
+	ui.progressBar->setValue(1);
+	ui.progressBar->setFormat("%p%");
 	downloadedData = reply->readAll();
 
 	ui.download->setEnabled(true);
@@ -389,7 +427,7 @@ void SubsurfaceWebServices::downloadFinished()
 
 	uint resultCode = download_dialog_parse_response(downloadedData);
 	setStatusText(resultCode);
-	if (resultCode == DD_STATUS_OK){
+	if (resultCode == DD_STATUS_OK) {
 		ui.buttonBox->button(QDialogButtonBox::Apply)->setEnabled(true);
 	}
 	reply->deleteLater();
@@ -407,32 +445,42 @@ void SubsurfaceWebServices::downloadError(QNetworkReply::NetworkError)
 void SubsurfaceWebServices::setStatusText(int status)
 {
 	QString text;
-	switch (status)	{
-	case DD_STATUS_ERROR_CONNECT:	text = tr("Connection Error: ");	break;
-	case DD_STATUS_ERROR_ID:	text = tr("Invalid user identifier!");	break;
-	case DD_STATUS_ERROR_PARSE:	text = tr("Cannot parse response!");	break;
-	case DD_STATUS_OK:		text = tr("Download Success!");		break;
+	switch (status) {
+	case DD_STATUS_ERROR_CONNECT:
+		text = tr("Connection error: ");
+		break;
+	case DD_STATUS_ERROR_ID:
+		text = tr("Invalid user identifier!");
+		break;
+	case DD_STATUS_ERROR_PARSE:
+		text = tr("Cannot parse response!");
+		break;
+	case DD_STATUS_OK:
+		text = tr("Download successful");
+		break;
 	}
 	ui.status->setText(text);
 }
 
+//TODO: C-Code.
 /* requires that there is a <download> or <error> tag under the <root> tag */
 void SubsurfaceWebServices::download_dialog_traverse_xml(xmlNodePtr node, unsigned int *download_status)
 {
 	xmlNodePtr cur_node;
 	for (cur_node = node; cur_node; cur_node = cur_node->next) {
 		if ((!strcmp((const char *)cur_node->name, (const char *)"download")) &&
-				(!strcmp((const char *)xmlNodeGetContent(cur_node), (const char *)"ok"))) {
+		    (!strcmp((const char *)xmlNodeGetContent(cur_node), (const char *)"ok"))) {
 			*download_status = DD_STATUS_OK;
 			return;
-		}	else if (!strcmp((const char *)cur_node->name, (const char *)"error")) {
+		} else if (!strcmp((const char *)cur_node->name, (const char *)"error")) {
 			*download_status = DD_STATUS_ERROR_ID;
 			return;
 		}
 	}
 }
 
-unsigned int SubsurfaceWebServices::download_dialog_parse_response(const QByteArray& xml)
+// TODO: C-Code
+unsigned int SubsurfaceWebServices::download_dialog_parse_response(const QByteArray &xml)
 {
 	xmlNodePtr root;
 	xmlDocPtr doc = xmlParseMemory(xml.data(), xml.length());
@@ -458,8 +506,7 @@ end:
 // #
 // #
 
-struct DiveListResult
-{
+struct DiveListResult {
 	QString errorCondition;
 	QString errorDetails;
 	QByteArray idList; // comma-separated, suitable to be sent in the fetch request
@@ -477,7 +524,7 @@ static DiveListResult parseDiveLogsDeDiveList(const QByteArray &xmlData)
 	 * </DiveDateReader>
 	 */
 	QXmlStreamReader reader(xmlData);
-	const QString invalidXmlError = DivelogsDeWebServices::tr("Invalid response from server");
+	const QString invalidXmlError = QObject::tr("Invalid response from server");
 	bool seenDiveDates = false;
 	DiveListResult result;
 	result.idCount = 0;
@@ -485,7 +532,7 @@ static DiveListResult parseDiveLogsDeDiveList(const QByteArray &xmlData)
 	if (reader.readNextStartElement() && reader.name() != "DiveDateReader") {
 		result.errorCondition = invalidXmlError;
 		result.errorDetails =
-				DivelogsDeWebServices::tr("Expected XML tag 'DiveDateReader', got instead '%1")
+			QObject::tr("Expected XML tag 'DiveDateReader', got instead '%1")
 				.arg(reader.name().toString());
 		goto out;
 	}
@@ -531,22 +578,23 @@ static DiveListResult parseDiveLogsDeDiveList(const QByteArray &xmlData)
 
 	if (!seenDiveDates) {
 		result.errorCondition = invalidXmlError;
-		result.errorDetails = DivelogsDeWebServices::tr("Expected XML tag 'DiveDates' not found");
+		result.errorDetails = QObject::tr("Expected XML tag 'DiveDates' not found");
 	}
 
 out:
 	if (reader.hasError()) {
 		// if there was an XML error, overwrite the result or other error conditions
 		result.errorCondition = invalidXmlError;
-		result.errorDetails = DivelogsDeWebServices::tr("Malformed XML response. Line %1: %2")
-				.arg(reader.lineNumber()).arg(reader.errorString());
+		result.errorDetails = QObject::tr("Malformed XML response. Line %1: %2")
+						.arg(reader.lineNumber())
+						.arg(reader.errorString());
 	}
 	return result;
 }
 
-DivelogsDeWebServices* DivelogsDeWebServices::instance()
+DivelogsDeWebServices *DivelogsDeWebServices::instance()
 {
-	static DivelogsDeWebServices *self = new DivelogsDeWebServices(mainWindow());
+	static DivelogsDeWebServices *self = new DivelogsDeWebServices(MainWindow::instance());
 	self->setAttribute(Qt::WA_QuitOnClose, false);
 	return self;
 }
@@ -559,13 +607,11 @@ void DivelogsDeWebServices::downloadDives()
 	exec();
 }
 
-void DivelogsDeWebServices::prepareDivesForUpload()
+void DivelogsDeWebServices::prepareDivesForUpload(bool selected)
 {
-	QString errorText;
-
 	/* generate a random filename and create/open that file with zip_open */
 	QString filename = QDir::tempPath() + "/import-" + QString::number(qrand() % 99999999) + ".dld";
-	if (prepare_dives_for_divelogs(filename, true, &errorText)) {
+	if (prepare_dives_for_divelogs(filename, selected)) {
 		QFile f(filename);
 		if (f.open(QIODevice::ReadOnly)) {
 			uploadDives((QIODevice *)&f);
@@ -574,7 +620,7 @@ void DivelogsDeWebServices::prepareDivesForUpload()
 			return;
 		}
 	}
-	mainWindow()->showError(errorText);
+	MainWindow::instance()->showError(get_error_string());
 }
 
 void DivelogsDeWebServices::uploadDives(QIODevice *dldContent)
@@ -605,13 +651,17 @@ void DivelogsDeWebServices::uploadDives(QIODevice *dldContent)
 	}
 }
 
-DivelogsDeWebServices::DivelogsDeWebServices(QWidget* parent, Qt::WindowFlags f): WebServices(parent, f)
+DivelogsDeWebServices::DivelogsDeWebServices(QWidget *parent, Qt::WindowFlags f) : WebServices(parent, f), uploadMode(false)
 {
-	uploadMode = false;
 	QSettings s;
 	ui.userID->setText(s.value("divelogde_user").toString());
 	ui.password->setText(s.value("divelogde_pass").toString());
+	ui.saveUidLocal->hide();
 	hideUpload();
+	QShortcut *close = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_W), this);
+	connect(close, SIGNAL(activated()), this, SLOT(close()));
+	QShortcut *quit = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q), this);
+	connect(quit, SIGNAL(activated()), parent, SLOT(close()));
 }
 
 void DivelogsDeWebServices::startUpload()
@@ -622,7 +672,7 @@ void DivelogsDeWebServices::startUpload()
 	s.sync();
 
 	ui.status->setText(tr("Uploading dive list..."));
-	ui.progressBar->setRange(0,0); // this makes the progressbar do an 'infinite spin'
+	ui.progressBar->setRange(0, 0); // this makes the progressbar do an 'infinite spin'
 	ui.upload->setEnabled(false);
 	ui.userID->setEnabled(false);
 	ui.password->setEnabled(false);
@@ -630,6 +680,7 @@ void DivelogsDeWebServices::startUpload()
 	QNetworkRequest request;
 	request.setUrl(QUrl("https://divelogs.de/DivelogsDirectImport.php"));
 	request.setRawHeader("Accept", "text/xml, application/xml");
+	request.setRawHeader("User-Agent", userAgent.toUtf8());
 
 	QHttpPart part;
 	part.setRawHeader("Content-Disposition", "form-data; name=\"user\"");
@@ -644,8 +695,8 @@ void DivelogsDeWebServices::startUpload()
 	connect(reply, SIGNAL(finished()), this, SLOT(uploadFinished()));
 	connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this,
 		SLOT(uploadError(QNetworkReply::NetworkError)));
-	connect(reply, SIGNAL(uploadProgress(qint64,qint64)), this,
-		SLOT(updateProgress(qint64,qint64)));
+	connect(reply, SIGNAL(uploadProgress(qint64, qint64)), this,
+		SLOT(updateProgress(qint64, qint64)));
 
 	timeout.start(30000); // 30s
 }
@@ -653,7 +704,7 @@ void DivelogsDeWebServices::startUpload()
 void DivelogsDeWebServices::startDownload()
 {
 	ui.status->setText(tr("Downloading dive list..."));
-	ui.progressBar->setRange(0,0); // this makes the progressbar do an 'infinite spin'
+	ui.progressBar->setRange(0, 0); // this makes the progressbar do an 'infinite spin'
 	ui.download->setEnabled(false);
 	ui.userID->setEnabled(false);
 	ui.password->setEnabled(false);
@@ -661,9 +712,10 @@ void DivelogsDeWebServices::startDownload()
 	QNetworkRequest request;
 	request.setUrl(QUrl("https://divelogs.de/xml_available_dives.php"));
 	request.setRawHeader("Accept", "text/xml, application/xml");
+	request.setRawHeader("User-Agent", userAgent.toUtf8());
 	request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
 
-#if QT_VERSION < QT_VERSION_CHECK(5,0,0)
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 	QUrl body;
 	body.addQueryItem("user", ui.userID->text());
 	body.addQueryItem("pass", ui.password->text());
@@ -674,7 +726,7 @@ void DivelogsDeWebServices::startDownload()
 	body.addQueryItem("user", ui.userID->text());
 	body.addQueryItem("pass", ui.password->text());
 
-	reply = manager()->post(request, body.query(QUrl::FullyEncoded).toLatin1())
+	reply = manager()->post(request, body.query(QUrl::FullyEncoded).toLatin1());
 #endif
 	connect(reply, SIGNAL(finished()), this, SLOT(listDownloadFinished()));
 	connect(reply, SIGNAL(error(QNetworkReply::NetworkError)),
@@ -705,9 +757,10 @@ void DivelogsDeWebServices::listDownloadFinished()
 	QNetworkRequest request;
 	request.setUrl(QUrl("https://divelogs.de/DivelogsDirectExport.php"));
 	request.setRawHeader("Accept", "application/zip, */*");
+	request.setRawHeader("User-Agent", userAgent.toUtf8());
 	request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
 
-#if QT_VERSION < QT_VERSION_CHECK(5,0,0)
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 	QUrl body;
 	body.addQueryItem("user", ui.userID->text());
 	body.addQueryItem("pass", ui.password->text());
@@ -720,7 +773,7 @@ void DivelogsDeWebServices::listDownloadFinished()
 	body.addQueryItem("pass", ui.password->text());
 	body.addQueryItem("ids", diveList.idList);
 
-	reply = manager()->post(request, body.query(QUrl::FullyEncoded).toLatin1())
+	reply = manager()->post(request, body.query(QUrl::FullyEncoded).toLatin1());
 #endif
 
 	connect(reply, SIGNAL(readyRead()), this, SLOT(saveToZipFile()));
@@ -770,6 +823,9 @@ void DivelogsDeWebServices::downloadFinished()
 
 	zip_close(zip);
 	zipFile.close();
+#if defined(Q_OS_UNIX) && defined(LIBZIP_VERSION_MAJOR)
+	::close(duppedfd);
+#endif
 }
 
 void DivelogsDeWebServices::uploadFinished()
@@ -777,7 +833,7 @@ void DivelogsDeWebServices::uploadFinished()
 	if (!reply)
 		return;
 
-	ui.progressBar->setRange(0,1);
+	ui.progressBar->setRange(0, 1);
 	ui.upload->setEnabled(true);
 	ui.userID->setEnabled(true);
 	ui.password->setEnabled(true);
@@ -812,7 +868,6 @@ void DivelogsDeWebServices::uploadFinished()
 
 void DivelogsDeWebServices::setStatusText(int status)
 {
-
 }
 
 void DivelogsDeWebServices::downloadError(QNetworkReply::NetworkError)
@@ -828,11 +883,11 @@ void DivelogsDeWebServices::uploadError(QNetworkReply::NetworkError error)
 	downloadError(error);
 }
 
-void DivelogsDeWebServices::buttonClicked(QAbstractButton* button)
+void DivelogsDeWebServices::buttonClicked(QAbstractButton *button)
 {
 	ui.buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
-	switch(ui.buttonBox->buttonRole(button)){
-	case QDialogButtonBox::ApplyRole:{
+	switch (ui.buttonBox->buttonRole(button)) {
+	case QDialogButtonBox::ApplyRole: {
 		/* in 'uploadMode' button is called 'Done' and closes the dialog */
 		if (uploadMode) {
 			hide();
@@ -841,14 +896,9 @@ void DivelogsDeWebServices::buttonClicked(QAbstractButton* button)
 			break;
 		}
 		/* parse file and import dives */
-		char *error = NULL;
-		parse_file(QFile::encodeName(zipFile.fileName()), &error);
-		if (error != NULL) {
-			mainWindow()->showError(error);
-			free(error);
-		}
-		process_dives(TRUE, FALSE);
-		mainWindow()->refreshDisplay();
+		parse_file(QFile::encodeName(zipFile.fileName()));
+		process_dives(true, false);
+		MainWindow::instance()->refreshDisplay();
 
 		/* store last entered user/pass in config */
 		QSettings s;
@@ -858,8 +908,7 @@ void DivelogsDeWebServices::buttonClicked(QAbstractButton* button)
 		hide();
 		close();
 		resetState();
-	}
-		break;
+	} break;
 	case QDialogButtonBox::RejectRole:
 		// these two seem to be causing a crash:
 		// reply->deleteLater();
@@ -872,3 +921,18 @@ void DivelogsDeWebServices::buttonClicked(QAbstractButton* button)
 		break;
 	}
 }
+
+UserSurveyServices::UserSurveyServices(QWidget *parent, Qt::WindowFlags f) : WebServices(parent, f)
+{
+
+}
+
+QNetworkReply* UserSurveyServices::sendSurvey(QString values)
+{
+	QNetworkRequest request;
+	request.setUrl(QString("http://subsurface.hohndel.org/survey?%1").arg(values));
+	request.setRawHeader("Accept", "text/xml");
+	request.setRawHeader("User-Agent", userAgent.toUtf8());
+	reply = manager()->get(request);
+	return reply;
+}
