@@ -15,23 +15,24 @@ class QModelIndex;
 class QTableView;
 
 class TableView : public QWidget {
-Q_OBJECT
+	Q_OBJECT
 public:
 	TableView(QWidget *parent = 0);
 	virtual ~TableView();
-	void setTitle(const QString& title);
+	void setTitle(const QString &title);
 	/* The model is expected to have a 'remove' slot, that takes a QModelIndex as parameter.
 	 * It's also expected to have the column '1' as a trash icon. I most probably should create a
 	 * proxy model and add that column, will mark that as TODO. see? marked.
 	 */
-	void setModel(QAbstractItemModel* model);
-	void setBtnToolTip(const QString& tooltip);
+	void setModel(QAbstractItemModel *model);
+	void setBtnToolTip(const QString &tooltip);
 	void fixPlusPosition();
-	void edit(const QModelIndex& index);
+	void edit(const QModelIndex &index);
 	QTableView *view();
+
 protected:
-	virtual void showEvent(QShowEvent* );
-	virtual void resizeEvent(QResizeEvent* );
+	virtual void showEvent(QShowEvent *);
+	virtual void resizeEvent(QResizeEvent *);
 
 signals:
 	void addButtonClicked();
@@ -41,4 +42,4 @@ private:
 	QPushButton *plusBtn;
 };
 
-#endif
+#endif // TABLEVIEW_H
