@@ -66,11 +66,13 @@ static void process_dive(struct dive *dp, stats_t *stats)
 	if (!duration)
 		return;
 	stats->avg_depth.mm = (1.0 * old_tt * stats->avg_depth.mm +
-			duration * dp->meandepth.mm) / stats->total_time.seconds;
-	if (dp->sac > 2800) { /* less than .1 cuft/min (2800ml/min) is bogus */
+			       duration * dp->meandepth.mm) /
+			      stats->total_time.seconds;
+	if (dp->sac > 100) { /* less than .1 l/min is bogus, even with a pSCR */
 		sac_time = stats->total_sac_time + duration;
 		stats->avg_sac.mliter = (1.0 * stats->total_sac_time * stats->avg_sac.mliter +
-				duration * dp->sac) / sac_time ;
+					 duration * dp->sac) /
+					sac_time;
 		if (dp->sac > stats->max_sac.mliter)
 			stats->max_sac.mliter = dp->sac;
 		if (stats->min_sac.mliter == 0 || dp->sac < stats->min_sac.mliter)
@@ -112,11 +114,10 @@ void process_all_dives(struct dive *dive, struct dive **prev_dive)
 	 * case (one dive per year or all dives during
 	 * one month) for yearly and monthly statistics*/
 
-	if (stats_yearly != NULL) {
-		free(stats_yearly);
-		free(stats_monthly);
-		free(stats_by_trip);
-	}
+	free(stats_yearly);
+	free(stats_monthly);
+	free(stats_by_trip);
+
 	size = sizeof(stats_t) * (dive_table.nr + 1);
 	stats_yearly = malloc(size);
 	stats_monthly = malloc(size);
@@ -126,16 +127,15 @@ void process_all_dives(struct dive *dive, struct dive **prev_dive)
 	memset(stats_yearly, 0, size);
 	memset(stats_monthly, 0, size);
 	memset(stats_by_trip, 0, size);
-	stats_yearly[0].is_year = TRUE;
+	stats_yearly[0].is_year = true;
 
 	/* this relies on the fact that the dives in the dive_table
 	 * are in chronological order */
-	for (idx = 0; idx < dive_table.nr; idx++) {
-		dp = dive_table.dives[idx];
+	for_each_dive (idx, dp) {
 		if (dive && dp->when == dive->when) {
 			/* that's the one we are showing */
 			if (idx > 0)
-				*prev_dive = dive_table.dives[idx-1];
+				*prev_dive = dive_table.dives[idx - 1];
 		}
 		process_dive(dp, &stats);
 
@@ -147,7 +147,7 @@ void process_all_dives(struct dive *dive, struct dive **prev_dive)
 		if (current_year != tm.tm_year + 1900) {
 			current_year = tm.tm_year + 1900;
 			process_dive(dp, &(stats_yearly[++year_iter]));
-			stats_yearly[year_iter].is_year = TRUE;
+			stats_yearly[year_iter].is_year = true;
 		} else {
 			process_dive(dp, &(stats_yearly[year_iter]));
 		}
@@ -163,12 +163,12 @@ void process_all_dives(struct dive *dive, struct dive **prev_dive)
 			/* stats_by_trip[0] is all the dives combined */
 			stats_by_trip[0].selection_size++;
 			process_dive(dp, &(stats_by_trip[0]));
-			stats_by_trip[0].is_trip = TRUE;
+			stats_by_trip[0].is_trip = true;
 			stats_by_trip[0].location = strdup("All (by trip stats)");
 
 			process_dive(dp, &(stats_by_trip[trip_iter]));
 			stats_by_trip[trip_iter].selection_size++;
-			stats_by_trip[trip_iter].is_trip = TRUE;
+			stats_by_trip[trip_iter].is_trip = true;
 			stats_by_trip[trip_iter].location = dp->divetrip->location;
 		}
 
@@ -211,15 +211,15 @@ char *get_time_string(int seconds, int maxdays)
 {
 	static char buf[80];
 	if (maxdays && seconds > 3600 * 24 * maxdays) {
-		snprintf(buf, sizeof(buf), translate("gettextFromC","more than %d days"), maxdays);
+		snprintf(buf, sizeof(buf), translate("gettextFromC", "more than %d days"), maxdays);
 	} else {
 		int days = seconds / 3600 / 24;
 		int hours = (seconds - days * 3600 * 24) / 3600;
 		int minutes = (seconds - days * 3600 * 24 - hours * 3600) / 60;
 		if (days > 0)
-			snprintf(buf, sizeof(buf), translate("gettextFromC","%dd %dh %dmin"), days, hours, minutes);
+			snprintf(buf, sizeof(buf), translate("gettextFromC", "%dd %dh %dmin"), days, hours, minutes);
 		else
-			snprintf(buf, sizeof(buf), translate("gettextFromC","%dh %dmin"), hours, minutes);
+			snprintf(buf, sizeof(buf), translate("gettextFromC", "%dh %dmin"), hours, minutes);
 	}
 	return buf;
 }
@@ -229,15 +229,15 @@ static void get_ranges(char *buffer, int size)
 {
 	int i, len;
 	int first = -1, last = -1;
+	struct dive *dive;
 
-	snprintf(buffer, size, "%s", translate("gettextFromC","for dives #"));
-	for (i = 0; i < dive_table.nr; i++) {
-		struct dive *dive = get_dive(i);
-		if (! dive->selected)
+	snprintf(buffer, size, "%s", translate("gettextFromC", "for dives #"));
+	for_each_dive (i, dive) {
+		if (!dive->selected)
 			continue;
 		if (dive->number < 1) {
 			/* uhh - weird numbers - bail */
-			snprintf(buffer, size, "%s", translate("gettextFromC","for selected dives"));
+			snprintf(buffer, size, "%s", translate("gettextFromC", "for selected dives"));
 			return;
 		}
 		len = strlen(buffer);
@@ -272,16 +272,16 @@ void get_selected_dives_text(char *buffer, int size)
 {
 	if (amount_selected == 1) {
 		if (current_dive)
-			snprintf(buffer, size, translate("gettextFromC","for dive #%d"), current_dive->number);
+			snprintf(buffer, size, translate("gettextFromC", "for dive #%d"), current_dive->number);
 		else
-			snprintf(buffer, size, "%s", translate("gettextFromC","for selected dive"));
+			snprintf(buffer, size, "%s", translate("gettextFromC", "for selected dive"));
 	} else if (amount_selected == dive_table.nr) {
-		snprintf(buffer, size, "%s", translate("gettextFromC","for all dives"));
+		snprintf(buffer, size, "%s", translate("gettextFromC", "for all dives"));
 	} else if (amount_selected == 0) {
-		snprintf(buffer, size, "%s", translate("gettextFromC","(no dives)"));
+		snprintf(buffer, size, "%s", translate("gettextFromC", "(no dives)"));
 	} else {
 		get_ranges(buffer, size);
-		if (strlen(buffer) == size -1) {
+		if (strlen(buffer) == size - 1) {
 			/* add our own ellipse... the way Pango does this is ugly
 			 * as it will leave partial numbers there which I don't like */
 			int offset = 4;
@@ -292,27 +292,26 @@ void get_selected_dives_text(char *buffer, int size)
 	}
 }
 
-static bool is_gas_used(struct dive *dive, int idx)
+bool is_cylinder_used(struct dive *dive, int idx)
 {
-	struct divecomputer *dc = &dive->dc;
-	bool firstGasExplicit = FALSE;
+	struct divecomputer *dc;
+	bool firstGasExplicit = false;
 	if (cylinder_none(&dive->cylinder[idx]))
-		return FALSE;
+		return false;
 
-	while (dc) {
+	for_each_dc(dive, dc) {
 		struct event *event = get_next_event(dc->events, "gaschange");
 		while (event) {
 			if (event->time.seconds < 30)
-				firstGasExplicit = TRUE;
+				firstGasExplicit = true;
 			if (get_cylinder_index(dive, event) == idx)
-				return TRUE;
+				return true;
 			event = get_next_event(event->next, "gaschange");
 		}
-		dc = dc->next;
 	}
 	if (idx == 0 && !firstGasExplicit)
-		return TRUE;
-	return FALSE;
+		return true;
+	return false;
 }
 
 void get_gas_used(struct dive *dive, volume_t gases[MAX_CYLINDERS])
@@ -322,48 +321,48 @@ void get_gas_used(struct dive *dive, volume_t gases[MAX_CYLINDERS])
 		cylinder_t *cyl = &dive->cylinder[idx];
 		pressure_t start, end;
 
-		if (!is_gas_used(dive, idx))
+		if (!is_cylinder_used(dive, idx))
 			continue;
 
 		start = cyl->start.mbar ? cyl->start : cyl->sample_start;
 		end = cyl->end.mbar ? cyl->end : cyl->sample_end;
-		if (start.mbar && end.mbar)
+		if (end.mbar && start.mbar > end.mbar)
 			gases[idx].mliter = gas_volume(cyl, start) - gas_volume(cyl, end);
 	}
 }
 
-#define MAXBUF 80
-/* for the O2/He readings just create a list of them */
-char *get_gaslist(struct dive *dive)
+/* Quite crude reverse-blender-function, but it produces a approx result */
+static void get_gas_parts(struct gasmix mix, volume_t vol, int o2_in_topup, volume_t *o2, volume_t *he)
 {
-	int idx, offset = 0;
-	static char buf[MAXBUF];
-	int o2, he;
+	volume_t air = {};
 
-	buf[0] = '\0';
-	for (idx = 0; idx < MAX_CYLINDERS; idx++) {
-		cylinder_t *cyl;
-		if (!is_gas_used(dive, idx))
+	if (gasmix_is_air(&mix)) {
+		o2->mliter = 0;
+		he->mliter = 0;
+		return;
+	}
+
+	air.mliter = (vol.mliter * (1000 - get_he(&mix) - get_o2(&mix))) / (1000 - o2_in_topup);
+	he->mliter = (vol.mliter * get_he(&mix)) / 1000;
+	o2->mliter += vol.mliter - he->mliter - air.mliter;
+}
+
+void selected_dives_gas_parts(volume_t *o2_tot, volume_t *he_tot)
+{
+	int i, j;
+	struct dive *d;
+	for_each_dive (i, d) {
+		if (!d->selected)
 			continue;
-		cyl = &dive->cylinder[idx];
-		o2 = get_o2(&cyl->gasmix);
-		he = get_he(&cyl->gasmix);
-		if (offset > 0) {
-			strncpy(buf + offset, "\n", MAXBUF - offset);
-			offset = strlen(buf);
+		volume_t diveGases[MAX_CYLINDERS] = {};
+		get_gas_used(d, diveGases);
+		for (j = 0; j < MAX_CYLINDERS; j++) {
+			if (diveGases[j].mliter) {
+				volume_t o2 = {}, he = {};
+				get_gas_parts(d->cylinder[j].gasmix, diveGases[j], O2_IN_AIR, &o2, &he);
+				o2_tot->mliter += o2.mliter;
+				he_tot->mliter += he.mliter;
+			}
 		}
-		if (is_air(o2, he))
-			strncpy(buf + offset, translate("gettextFromC","air"), MAXBUF - offset);
-		else
-			if (he == 0)
-				snprintf(buf + offset, MAXBUF - offset,
-					translate("gettextFromC","EAN%d"), (o2 + 5) / 10);
-			else
-				snprintf(buf + offset, MAXBUF - offset,
-					"%d/%d", (o2 + 5) / 10, (he + 5) / 10);
-		offset = strlen(buf);
 	}
-	if (*buf == '\0')
-		strncpy(buf, translate("gettextFromC","air"), MAXBUF);
-	return buf;
 }
