@@ -12,7 +12,8 @@
 extern "C" {
 #endif
 
-typedef struct {
+typedef struct
+{
 	int period;
 	duration_t total_time;
 	/* avg_time is simply total_time / nr -- let's not keep this */
@@ -44,11 +45,11 @@ extern char *get_minutes(int seconds);
 extern void process_all_dives(struct dive *dive, struct dive **prev_dive);
 extern void get_selected_dives_text(char *buffer, int size);
 extern void get_gas_used(struct dive *dive, volume_t gases[MAX_CYLINDERS]);
-extern char *get_gaslist(struct dive *dive);
 extern void process_selected_dives(void);
+void selected_dives_gas_parts(volume_t *o2_tot, volume_t *he_tot);
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif
+#endif // STATISTICS_H
