@@ -21,20 +21,24 @@
 #include "uemis.h"
 #include "divelist.h"
 
-#define ERR_FS_ALMOST_FULL QT_TRANSLATE_NOOP("gettextFromC","Uemis Zurich: File System is almost full\nDisconnect/reconnect the dive computer\nand click \'Retry\'")
-#define ERR_FS_FULL QT_TRANSLATE_NOOP("gettextFromC","Uemis Zurich: File System is full\nDisconnect/reconnect the dive computer\nand try again")
-#define ERR_FS_SHORT_WRITE QT_TRANSLATE_NOOP("gettextFromC","Short write to req.txt file\nIs the Uemis Zurich plugged in correctly?")
+#define ERR_FS_ALMOST_FULL QT_TRANSLATE_NOOP("gettextFromC", "Uemis Zurich: File System is almost full\nDisconnect/reconnect the dive computer\nand click \'Retry\'")
+#define ERR_FS_FULL QT_TRANSLATE_NOOP("gettextFromC", "Uemis Zurich: File System is full\nDisconnect/reconnect the dive computer\nand try again")
+#define ERR_FS_SHORT_WRITE QT_TRANSLATE_NOOP("gettextFromC", "Short write to req.txt file\nIs the Uemis Zurich plugged in correctly?")
 #define BUFLEN 2048
 #define NUM_PARAM_BUFS 10
 
-#if UEMIS_DEBUG & 64  /* we are reading from a copy of the filesystem, not the device - no need to wait */
-#define UEMIS_TIMEOUT 50		/* 50ns */
-#define UEMIS_LONG_TIMEOUT 500		/* 500ns */
-#define UEMIS_MAX_TIMEOUT 2000		/* 2ms */
+#if UEMIS_DEBUG & 64	   /* we are reading from a copy of the filesystem, not the device - no need to wait */
+#define UEMIS_TIMEOUT 50       /* 50ns */
+#define UEMIS_LONG_TIMEOUT 500 /* 500ns */
+#define UEMIS_MAX_TIMEOUT 2000 /* 2ms */
 #else
-#define UEMIS_TIMEOUT 50000		/* 50ms */
-#define UEMIS_LONG_TIMEOUT 500000	/* 500ms */
-#define UEMIS_MAX_TIMEOUT 2000000	/* 2s */
+#define UEMIS_TIMEOUT 50000       /* 50ms */
+#define UEMIS_LONG_TIMEOUT 500000 /* 500ms */
+#define UEMIS_MAX_TIMEOUT 2000000 /* 2s */
+#endif
+
+#ifdef UEMIS_DEBUG
+#define debugfile stderr
 #endif
 
 static char *param_buff[NUM_PARAM_BUFS];
@@ -45,7 +49,7 @@ static int number_of_files;
 static char *mbuf = NULL;
 static int mbuf_size = 0;
 
-static int nr_divespots = 0;
+static int nr_divespots = -1;
 
 /* helper function to parse the Uemis data structures */
 static void uemis_ts(char *buffer, void *_when)
@@ -54,10 +58,10 @@ static void uemis_ts(char *buffer, void *_when)
 	timestamp_t *when = _when;
 
 	memset(&tm, 0, sizeof(tm));
-	sscanf(buffer,"%d-%d-%dT%d:%d:%d",
-		&tm.tm_year, &tm.tm_mon, &tm.tm_mday,
-		&tm.tm_hour, &tm.tm_min, &tm.tm_sec);
-	tm.tm_mon  -= 1;
+	sscanf(buffer, "%d-%d-%dT%d:%d:%d",
+	       &tm.tm_year, &tm.tm_mon, &tm.tm_mday,
+	       &tm.tm_hour, &tm.tm_min, &tm.tm_sec);
+	tm.tm_mon -= 1;
 	tm.tm_year -= 1900;
 	*when = utc_mktime(&tm);
 }
@@ -65,7 +69,7 @@ static void uemis_ts(char *buffer, void *_when)
 /* float minutes */
 static void uemis_duration(char *buffer, duration_t *duration)
 {
-	duration->seconds = ascii_strtod(buffer, NULL) * 60 + 0.5;
+	duration->seconds = rint(ascii_strtod(buffer, NULL) * 60);
 }
 
 /* int cm */
@@ -102,14 +106,15 @@ static void uemis_add_string(const char *buffer, char **text)
 static void uemis_get_weight(char *buffer, weightsystem_t *weight, int diveid)
 {
 	weight->weight.grams = uemis_get_weight_unit(diveid) ?
-		lbs_to_grams(ascii_strtod(buffer, NULL)) : ascii_strtod(buffer, NULL) * 1000;
-	weight->description = strdup(translate("gettextFromC","unknown"));
+				   lbs_to_grams(ascii_strtod(buffer, NULL)) :
+				   ascii_strtod(buffer, NULL) * 1000;
+	weight->description = strdup(translate("gettextFromC", "unknown"));
 }
 
 static struct dive *uemis_start_dive(uint32_t deviceid)
 {
 	struct dive *dive = alloc_dive();
-	dive->downloaded = TRUE;
+	dive->downloaded = true;
 	dive->dc.model = strdup("Uemis Zurich");
 	dive->dc.deviceid = deviceid;
 	return dive;
@@ -133,6 +138,8 @@ static long bytes_available(int file)
 	long now = lseek(file, 0, SEEK_CUR);
 	result = lseek(file, 0, SEEK_END);
 	lseek(file, now, SEEK_SET);
+	if (now == -1 || result == -1)
+		return 0;
 	return result;
 }
 
@@ -158,8 +165,7 @@ static int number_of_file(char *path)
 			break;
 		if (entry->d_type == DT_REG) /* If the entry is a regular file */
 #endif
-			count++;
-
+		count++;
 	}
 #ifdef WIN32
 	_wclosedir(dirp);
@@ -174,7 +180,7 @@ static char *build_filename(const char *path, const char *name)
 	int len = strlen(path) + strlen(name) + 2;
 	char *buf = malloc(len);
 #if WIN32
-	snprintf(buf, len, "%s\%s", path, name);
+	snprintf(buf, len, "%s\\%s", path, name);
 #else
 	snprintf(buf, len, "%s/%s", path, name);
 #endif
@@ -191,15 +197,15 @@ static bool uemis_init(const char *path)
 	int i;
 
 	if (!path)
-		return FALSE;
+		return false;
 	/* let's check if this is indeed a Uemis DC */
-	reqtxt_path = build_filename(path,"req.txt");
+	reqtxt_path = build_filename(path, "req.txt");
 	reqtxt_file = subsurface_open(reqtxt_path, O_RDONLY, 0666);
 	if (!reqtxt_file) {
 #if UEMIS_DEBUG & 1
 		fprintf(debugfile, ":EE req.txt can't be opened\n");
 #endif
-		return FALSE;
+		return false;
 	}
 	if (bytes_available(reqtxt_file) > 5) {
 		char tmp[6];
@@ -209,14 +215,14 @@ static bool uemis_init(const char *path)
 		fprintf(debugfile, "::r req.txt \"%s\"\n", tmp);
 #endif
 		if (sscanf(tmp + 1, "%d", &filenr) != 1)
-			return FALSE;
+			return false;
 	} else {
 		filenr = 0;
 #if UEMIS_DEBUG & 2
 		fprintf(debugfile, "::r req.txt skipped as there were fewer than 5 bytes\n");
 #endif
 	}
-	close (reqtxt_file);
+	close(reqtxt_file);
 
 	/* It would be nice if we could simply go back to the first set of
 	 * ANS files. But with a FAT filesystem that isn't possible */
@@ -226,7 +232,7 @@ static bool uemis_init(const char *path)
 	/* initialize the array in which we collect the answers */
 	for (i = 0; i < NUM_PARAM_BUFS; i++)
 		param_buff[i] = "";
-	return TRUE;
+	return true;
 }
 
 static void str_append_with_delim(char *s, char *t)
@@ -247,15 +253,20 @@ static void trigger_response(int file, char *command, int nr, long tailpos)
 
 	snprintf(fl, 8, "%s%04d", command, nr);
 #if UEMIS_DEBUG & 4
-	fprintf(debugfile,":tr %s (after seeks)\n", fl);
+	fprintf(debugfile, ":tr %s (after seeks)\n", fl);
 #endif
-	lseek(file, 0, SEEK_SET);
-	write(file, fl, strlen(fl));
-	lseek(file, tailpos, SEEK_SET);
-	write(file, fl + 1, strlen(fl + 1));
+	if (lseek(file, 0, SEEK_SET) == -1)
+		goto fs_error;
+	if (write(file, fl, strlen(fl)) == -1)
+		goto fs_error;
+	if (lseek(file, tailpos, SEEK_SET) == -1)
+		goto fs_error;
+	if (write(file, fl + 1, strlen(fl + 1)) == -1)
+		goto fs_error;
 #ifndef WIN32
 	fsync(file);
 #endif
+fs_error:
 	close(file);
 }
 
@@ -276,19 +287,19 @@ static char *next_segment(char *buf, int *offset, int size)
 {
 	int i = *offset;
 	int seg_size;
-	bool done = FALSE;
+	bool done = false;
 	char *segment;
 
 	while (!done) {
 		if (i < size) {
 			if (i < size - 1 && buf[i] == '\\' &&
-				(buf[i+1] == '\\' || buf[i+1] == '{'))
+			    (buf[i + 1] == '\\' || buf[i + 1] == '{'))
 				memcpy(buf + i, buf + i + 1, size - i - 1);
 			else if (buf[i] == '{')
-				done = TRUE;
+				done = true;
 			i++;
 		} else {
-			done = TRUE;
+			done = true;
 		}
 	}
 	seg_size = i - *offset - 1;
@@ -307,7 +318,7 @@ static void buffer_add(char **buffer, int *buffer_size, char *buf)
 {
 	if (!buf)
 		return;
-	if (! *buffer) {
+	if (!*buffer) {
 		*buffer = strdup(buf);
 		*buffer_size = strlen(*buffer) + 1;
 	} else {
@@ -316,7 +327,7 @@ static void buffer_add(char **buffer, int *buffer_size, char *buf)
 		strcat(*buffer, buf);
 	}
 #if UEMIS_DEBUG & 16
-	fprintf(debugfile,"added \"%s\" to buffer - new length %d\n", buf, *buffer_size);
+	fprintf(debugfile, "added \"%s\" to buffer - new length %d\n", buf, *buffer_size);
 #endif
 }
 
@@ -324,12 +335,12 @@ static void buffer_add(char **buffer, int *buffer_size, char *buf)
 static bool next_file(int max)
 {
 	if (filenr >= max)
-		return FALSE;
+		return false;
 	filenr++;
-	return TRUE;
+	return true;
 }
 
-static char *first_object_id_val(char* buf)
+static char *first_object_id_val(char *buf)
 {
 	char *object, *bufend;
 	if (!buf)
@@ -365,11 +376,11 @@ static void show_progress(char *buf, const char *what)
 {
 	char *val = first_object_id_val(buf);
 	if (val) {
-		/* let the user know what we are working on */
+/* let the user know what we are working on */
 #if UEMIS_DEBUG & 2
-		fprintf(debugfile,"reading %s %s\n", what, val);
+		fprintf(debugfile, "reading %s %s\n", what, val);
 #endif
-		uemis_info(translate("gettextFromC","Reading %s %s"), what, val);
+		uemis_info(translate("gettextFromC", "Reading %s %s"), what, val);
 		free(val);
 	}
 }
@@ -383,19 +394,19 @@ static void uemis_increased_timeout(int *timeout)
 
 /* send a request to the dive computer and collect the answer */
 static bool uemis_get_answer(const char *path, char *request, int n_param_in,
-			int n_param_out, const char **error_text)
+			     int n_param_out, const char **error_text)
 {
 	int i = 0, file_length;
 	char sb[BUFLEN];
 	char fl[13];
 	char tmp[101];
-	const char *what = translate("gettextFromC","data");
-	bool searching = TRUE;
-	bool assembling_mbuf = FALSE;
-	bool ismulti = FALSE;
-	bool found_answer = FALSE;
-	bool more_files = TRUE;
-	bool answer_in_mbuf = FALSE;
+	const char *what = translate("gettextFromC", "data");
+	bool searching = true;
+	bool assembling_mbuf = false;
+	bool ismulti = false;
+	bool found_answer = false;
+	bool more_files = true;
+	bool answer_in_mbuf = false;
 	char *ans_path;
 	int ans_file;
 	int timeout = UEMIS_LONG_TIMEOUT;
@@ -405,31 +416,31 @@ static bool uemis_get_answer(const char *path, char *request, int n_param_in,
 	str_append_with_delim(sb, request);
 	for (i = 0; i < n_param_in; i++)
 		str_append_with_delim(sb, param_buff[i]);
-	if (! strcmp(request, "getDivelogs") || ! strcmp(request, "getDeviceData") || ! strcmp(request, "getDirectory") ||
-		! strcmp(request, "getDivespot") || ! strcmp(request, "getDive")) {
-		answer_in_mbuf = TRUE;
+	if (!strcmp(request, "getDivelogs") || !strcmp(request, "getDeviceData") || !strcmp(request, "getDirectory") ||
+	    !strcmp(request, "getDivespot") || !strcmp(request, "getDive")) {
+		answer_in_mbuf = true;
 		str_append_with_delim(sb, "");
-		if (! strcmp(request, "getDivelogs"))
-			what = translate("gettextFromC","divelog entry id");
+		if (!strcmp(request, "getDivelogs"))
+			what = translate("gettextFromC", "divelog entry id");
 		else if (!strcmp(request, "getDivespot"))
-			what = translate("gettextFromC","divespot data id");
+			what = translate("gettextFromC", "divespot data id");
 		else if (!strcmp(request, "getDive"))
-			what = translate("gettextFromC","more data dive id");
+			what = translate("gettextFromC", "more data dive id");
 	}
 	str_append_with_delim(sb, "");
 	file_length = strlen(sb);
 	snprintf(fl, 10, "%08d", file_length - 13);
 	memcpy(sb + 5, fl, strlen(fl));
 #if UEMIS_DEBUG & 1
-	fprintf(debugfile,"::w req.txt \"%s\"\n", sb);
+	fprintf(debugfile, "::w req.txt \"%s\"\n", sb);
 #endif
 	if (write(reqtxt_file, sb, strlen(sb)) != strlen(sb)) {
-		*error_text = translate("gettextFromC",ERR_FS_SHORT_WRITE);
-		return FALSE;
+		*error_text = translate("gettextFromC", ERR_FS_SHORT_WRITE);
+		return false;
 	}
-	if (! next_file(number_of_files)) {
-		*error_text = translate("gettextFromC",ERR_FS_FULL);
-		more_files = FALSE;
+	if (!next_file(number_of_files)) {
+		*error_text = translate("gettextFromC", ERR_FS_FULL);
+		more_files = false;
 	}
 	trigger_response(reqtxt_file, "n", filenr, file_length);
 	usleep(timeout);
@@ -437,7 +448,7 @@ static bool uemis_get_answer(const char *path, char *request, int n_param_in,
 	mbuf_size = 0;
 	while (searching || assembling_mbuf) {
 		if (import_thread_cancelled)
-			return FALSE;
+			return false;
 		progress_bar_fraction = filenr / 4000.0;
 		snprintf(fl, 13, "ANS%d.TXT", filenr - 1);
 		ans_path = build_filename(build_filename(path, "ANS"), fl);
@@ -445,7 +456,7 @@ static bool uemis_get_answer(const char *path, char *request, int n_param_in,
 		read(ans_file, tmp, 100);
 		close(ans_file);
 #if UEMIS_DEBUG & 8
-		tmp[100]='\0';
+		tmp[100] = '\0';
 		fprintf(debugfile, "::t %s \"%s\"\n", ans_path, tmp);
 #elif UEMIS_DEBUG & 4
 		char pbuf[4];
@@ -457,28 +468,28 @@ static bool uemis_get_answer(const char *path, char *request, int n_param_in,
 #endif
 		free(ans_path);
 		if (tmp[0] == '1') {
-			searching = FALSE;
+			searching = false;
 			if (tmp[1] == 'm') {
-				assembling_mbuf = TRUE;
-				ismulti = TRUE;
+				assembling_mbuf = true;
+				ismulti = true;
 			}
 			if (tmp[2] == 'e')
-				assembling_mbuf = FALSE;
+				assembling_mbuf = false;
 			if (assembling_mbuf) {
-				if (! next_file(number_of_files)) {
-					*error_text = translate("gettextFromC",ERR_FS_FULL);
-					more_files = FALSE;
-					assembling_mbuf = FALSE;
+				if (!next_file(number_of_files)) {
+					*error_text = translate("gettextFromC", ERR_FS_FULL);
+					more_files = false;
+					assembling_mbuf = false;
 				}
 				reqtxt_file = subsurface_open(reqtxt_path, O_RDWR | O_CREAT, 0666);
 				trigger_response(reqtxt_file, "n", filenr, file_length);
 			}
 		} else {
-			if (! next_file(number_of_files - 1)) {
-				*error_text = translate("gettextFromC",ERR_FS_FULL);
-				more_files = FALSE;
-				assembling_mbuf = FALSE;
-				searching = FALSE;
+			if (!next_file(number_of_files - 1)) {
+				*error_text = translate("gettextFromC", ERR_FS_FULL);
+				more_files = false;
+				assembling_mbuf = false;
+				searching = false;
 			}
 			reqtxt_file = subsurface_open(reqtxt_path, O_RDWR | O_CREAT, 0666);
 			trigger_response(reqtxt_file, "r", filenr, file_length);
@@ -491,10 +502,16 @@ static bool uemis_get_answer(const char *path, char *request, int n_param_in,
 			ans_file = subsurface_open(ans_path, O_RDONLY, 0666);
 			size = bytes_available(ans_file);
 			if (size > 3) {
-				char *buf = malloc(size - 2);
-				lseek(ans_file, 3, SEEK_CUR);
-				read(ans_file, buf, size - 3);
-				buf[size - 3] = '\0';
+				char *buf;
+				int r;
+				if (lseek(ans_file, 3, SEEK_CUR) == -1)
+					goto fs_error;
+				buf = malloc(size - 2);
+				if ((r = read(ans_file, buf, size - 3)) != size - 3) {
+					free(buf);
+					goto fs_error;
+				}
+				buf[r] = '\0';
 				buffer_add(&mbuf, &mbuf_size, buf);
 				show_progress(buf, what);
 				free(buf);
@@ -515,10 +532,15 @@ static bool uemis_get_answer(const char *path, char *request, int n_param_in,
 			ans_file = subsurface_open(ans_path, O_RDONLY, 0666);
 			size = bytes_available(ans_file);
 			if (size > 3) {
+				int r;
+				if (lseek(ans_file, 3, SEEK_CUR) == -1)
+					goto fs_error;
 				buf = malloc(size - 2);
-				lseek(ans_file, 3, SEEK_CUR);
-				read(ans_file, buf, size - 3);
-				buf[size - 3] = '\0';
+				if ((r = read(ans_file, buf, size - 3)) != size - 3) {
+					free(buf);
+					goto fs_error;
+				}
+				buf[r] = '\0';
 				buffer_add(&mbuf, &mbuf_size, buf);
 				show_progress(buf, what);
 #if UEMIS_DEBUG & 8
@@ -529,22 +551,25 @@ static bool uemis_get_answer(const char *path, char *request, int n_param_in,
 			close(ans_file);
 			free(ans_path);
 		} else {
-			ismulti = FALSE;
+			ismulti = false;
 		}
 #if UEMIS_DEBUG & 8
-		fprintf(debugfile,":r: %s\n", buf);
+		fprintf(debugfile, ":r: %s\n", buf);
 #endif
 		if (!answer_in_mbuf)
 			for (i = 0; i < n_param_out && j < size; i++)
 				param_buff[i] = next_segment(buf, &j, size);
-		found_answer = TRUE;
+		found_answer = true;
 		free(buf);
 	}
 #if UEMIS_DEBUG & 1
 	for (i = 0; i < n_param_out; i++)
-		fprintf(debugfile,"::: %d: %s\n", i, param_buff[i]);
+		fprintf(debugfile, "::: %d: %s\n", i, param_buff[i]);
 #endif
 	return found_answer;
+fs_error:
+	close (ans_file);
+	return false;
 }
 
 static void parse_divespot(char *buf)
@@ -562,9 +587,9 @@ static void parse_divespot(char *buf)
 	do
 		tag = next_token(&bp);
 	while (*tag && strcmp(tag, "object_id"));
-	if (! *tag)
+	if (!*tag)
 		return;
-	type = next_token(&bp);
+	next_token(&bp);
 	val = next_token(&bp);
 	divespot = atoi(val);
 	do {
@@ -574,7 +599,7 @@ static void parse_divespot(char *buf)
 		if (!strcmp(type, "string") && *val && strcmp(val, " ")) {
 			len = strlen(locationstring);
 			snprintf(locationstring + len, sizeof(locationstring) - len,
-					"%s%s", len ? ", " : "", val);
+				 "%s%s", len ? ", " : "", val);
 		} else if (!strcmp(type, "float")) {
 			if (!strcmp(tag, "longitude"))
 				longitude = ascii_strtod(val, NULL);
@@ -594,15 +619,15 @@ static void track_divespot(char *val, int diveid, char **location, degrees_t *la
 	return;
 }
 
-static char *suit[] = { "", QT_TRANSLATE_NOOP("gettextFromC","wetsuit"), QT_TRANSLATE_NOOP("gettextFromC","semidry"), QT_TRANSLATE_NOOP("gettextFromC","drysuit") };
-static char *suit_type[] = { "", QT_TRANSLATE_NOOP("gettextFromC","shorty"), QT_TRANSLATE_NOOP("gettextFromC","vest"), QT_TRANSLATE_NOOP("gettextFromC","long john"), QT_TRANSLATE_NOOP("gettextFromC","jacket"), QT_TRANSLATE_NOOP("gettextFromC","full suit"), QT_TRANSLATE_NOOP("gettextFromC","2 pcs full suit") };
-static char *suit_thickness[] = { "", "0.5-2mm", "2-3mm", "3-5mm", "5-7mm", "8mm+", QT_TRANSLATE_NOOP("gettextFromC","membrane") };
+static char *suit[] = { "", QT_TRANSLATE_NOOP("gettextFromC", "wetsuit"), QT_TRANSLATE_NOOP("gettextFromC", "semidry"), QT_TRANSLATE_NOOP("gettextFromC", "drysuit") };
+static char *suit_type[] = { "", QT_TRANSLATE_NOOP("gettextFromC", "shorty"), QT_TRANSLATE_NOOP("gettextFromC", "vest"), QT_TRANSLATE_NOOP("gettextFromC", "long john"), QT_TRANSLATE_NOOP("gettextFromC", "jacket"), QT_TRANSLATE_NOOP("gettextFromC", "full suit"), QT_TRANSLATE_NOOP("gettextFromC", "2 pcs full suit") };
+static char *suit_thickness[] = { "", "0.5-2mm", "2-3mm", "3-5mm", "5-7mm", "8mm+", QT_TRANSLATE_NOOP("gettextFromC", "membrane") };
 
 static void parse_tag(struct dive *dive, char *tag, char *val)
 {
 	/* we can ignore computer_id, water and gas as those are redundant
 	 * with the binary data and would just get overwritten */
-	if (! strcmp(tag, "date")) {
+	if (!strcmp(tag, "date")) {
 		uemis_ts(val, &dive->when);
 	} else if (!strcmp(tag, "duration")) {
 		uemis_duration(val, &dive->dc.duration);
@@ -618,13 +643,13 @@ static void parse_tag(struct dive *dive, char *tag, char *val)
 		uemis_add_string(val, &dive->notes);
 	} else if (!strcmp(tag, "u8DiveSuit")) {
 		if (*suit[atoi(val)])
-			uemis_add_string(translate("gettextFromC",suit[atoi(val)]), &dive->suit);
+			uemis_add_string(translate("gettextFromC", suit[atoi(val)]), &dive->suit);
 	} else if (!strcmp(tag, "u8DiveSuitType")) {
 		if (*suit_type[atoi(val)])
-			uemis_add_string(translate("gettextFromC",suit_type[atoi(val)]), &dive->suit);
+			uemis_add_string(translate("gettextFromC", suit_type[atoi(val)]), &dive->suit);
 	} else if (!strcmp(tag, "u8SuitThickness")) {
 		if (*suit_thickness[atoi(val)])
-			uemis_add_string(translate("gettextFromC",suit_thickness[atoi(val)]), &dive->suit);
+			uemis_add_string(translate("gettextFromC", suit_thickness[atoi(val)]), &dive->suit);
 	}
 }
 
@@ -639,44 +664,59 @@ static void parse_tag(struct dive *dive, char *tag, char *val)
  * index into yet another data store that we read out later. In order to
  * correctly populate the location and gps data from that we need to remember
  * the adresses of those fields for every dive that references the divespot. */
-static void process_raw_buffer(uint32_t deviceid, char *inbuf, char **max_divenr, bool keep_number, int *for_dive)
+static bool process_raw_buffer(uint32_t deviceid, char *inbuf, char **max_divenr, bool keep_number, int *for_dive)
 {
 	char *buf = strdup(inbuf);
 	char *tp, *bp, *tag, *type, *val;
-	bool done = FALSE;
+	bool done = false;
 	int inbuflen = strlen(inbuf);
 	char *endptr = buf + inbuflen;
-	bool log = FALSE;
+	bool log = false;
 	char *sections[10];
 	int s, nr_sections = 0;
 	struct dive *dive = NULL;
 
+#if UEMIS_DEBUG & 4
+	fprintf(debugfile, "p_r_b %s\n", inbuf);
+#endif
 	if (for_dive)
 		*for_dive = -1;
 	bp = buf + 1;
 	tp = next_token(&bp);
 	if (strcmp(tp, "divelog") == 0) {
 		/* this is a divelog */
-		log = TRUE;
+		log = true;
 		tp = next_token(&bp);
-		if (strcmp(tp,"1.0") != 0) {
+		/* is it a valid entry or nothing ? */
+		if (strcmp(tp, "1.0") != 0 || strstr(inbuf, "divelog{1.0{{{{")) {
 			free(buf);
-			return;
+			return false;
 		}
 	} else if (strcmp(tp, "dive") == 0) {
 		/* this is dive detail */
 		tp = next_token(&bp);
-		if (strcmp(tp,"1.0") != 0) {
+		if (strcmp(tp, "1.0") != 0) {
 			free(buf);
-			return;
+			return false;
 		}
 	} else {
 		/* don't understand the buffer */
 		free(buf);
-		return;
+		return false;
 	}
-	if (log)
+	if (log) {
 		dive = uemis_start_dive(deviceid);
+	} else {
+		/* remember, we don't know if this is the right entry,
+		 * so first test if this is even a valid entry */
+		if (strstr(inbuf, "deleted{bool{true")) {
+#if UEMIS_DEBUG & 4
+			fprintf(debugfile, "p_r_b entry deleted\n");
+#endif
+			/* oops, this one isn't valid, suggest to try the previous one */
+			return false;
+		}
+	}
 	while (!done) {
 		/* the valid buffer ends with a series of delimiters */
 		if (bp >= endptr - 2 || !strcmp(bp, "{{"))
@@ -703,45 +743,47 @@ static void process_raw_buffer(uint32_t deviceid, char *inbuf, char **max_divenr
 			continue;
 		}
 		val = next_token(&bp);
-		if (log && ! strcmp(tag, "object_id")) {
+		if (log && !strcmp(tag, "object_id")) {
 			free(*max_divenr);
 			*max_divenr = strdup(val);
 			dive->dc.diveid = atoi(val);
 			if (keep_number)
 				dive->number = atoi(val);
-		} else if (!log && ! strcmp(tag, "logfilenr")) {
+		} else if (!log && !strcmp(tag, "logfilenr")) {
 			/* this one tells us which dive we are adding data to */
-			dive = get_dive_by_diveid(atoi(val), deviceid);
+			dive = get_dive_by_uemis_diveid(atoi(val), deviceid);
 			if (for_dive)
 				*for_dive = atoi(val);
-		} else if (!log && dive && ! strcmp(tag, "divespot_id")) {
+		} else if (!log && dive && !strcmp(tag, "divespot_id")) {
 			track_divespot(val, dive->dc.diveid, &dive->location, &dive->latitude, &dive->longitude);
 		} else if (dive) {
 			parse_tag(dive, tag, val);
 		}
-		if (log && ! strcmp(tag, "file_content"))
-			done = TRUE;
+		if (log && !strcmp(tag, "file_content"))
+			done = true;
 		/* done with one dive (got the file_content tag), but there could be more:
 		 * a '{' indicates the end of the record - but we need to see another "{{"
 		 * later in the buffer to know that the next record is complete (it could
 		 * be a short read because of some error */
 		if (done && ++bp < endptr && *bp != '{' && strstr(bp, "{{")) {
-			done = FALSE;
+			done = false;
 			record_dive(dive);
-			mark_divelist_changed(TRUE);
+			mark_divelist_changed(true);
 			dive = uemis_start_dive(deviceid);
 		}
 	}
 	if (log) {
 		if (dive->dc.diveid) {
 			record_dive(dive);
-			mark_divelist_changed(TRUE);
+			mark_divelist_changed(true);
 		} else { /* partial dive */
 			free(dive);
+			free(buf);
+			return false;
 		}
 	}
 	free(buf);
-	return;
+	return true;
 }
 
 static char *uemis_get_divenr(char *deviceidstr)
@@ -751,14 +793,14 @@ static char *uemis_get_divenr(char *deviceidstr)
 	char divenr[10];
 
 	deviceid = atoi(deviceidstr);
-	for (i = 0; i < dive_table.nr; i++) {
-		struct divecomputer *dc = &dive_table.dives[i]->dc;
-		while (dc) {
+	struct dive *d;
+	for_each_dive (i, d) {
+		struct divecomputer *dc;
+		for_each_dc(d, dc) {
 			if (dc->model && !strcmp(dc->model, "Uemis Zurich") &&
 			    (dc->deviceid == 0 || dc->deviceid == 0x7fffffff || dc->deviceid == deviceid) &&
 			    dc->diveid > maxdiveid)
 				maxdiveid = dc->diveid;
-			dc = dc->next;
 		}
 	}
 	snprintf(divenr, 10, "%d", maxdiveid);
@@ -768,31 +810,31 @@ static char *uemis_get_divenr(char *deviceidstr)
 const char *do_uemis_import(const char *mountpath, short force_download)
 {
 	char *newmax = NULL;
-	int start, end, i, offset;
+	int start, end = -2, i, offset;
 	uint32_t deviceidnr;
 	char objectid[10];
 	char *deviceid = NULL;
 	const char *result = NULL;
 	char *endptr;
-	bool success, keep_number = FALSE, once = TRUE;
+	bool success, keep_number = false, once = true;
 
 	if (dive_table.nr == 0)
-		keep_number = TRUE;
-	uemis_info(translate("gettextFromC","Init Communication"));
-	if (! uemis_init(mountpath))
-		return translate("gettextFromC","Uemis init failed");
-	if (! uemis_get_answer(mountpath, "getDeviceId", 0, 1, &result))
+		keep_number = true;
+	uemis_info(translate("gettextFromC", "Init Communication"));
+	if (!uemis_init(mountpath))
+		return translate("gettextFromC", "Uemis init failed");
+	if (!uemis_get_answer(mountpath, "getDeviceId", 0, 1, &result))
 		goto bail;
 	deviceid = strdup(param_buff[0]);
 	deviceidnr = atoi(deviceid);
 	/* the answer from the DeviceId call becomes the input parameter for getDeviceData */
-	if (! uemis_get_answer(mountpath, "getDeviceData", 1, 0, &result))
+	if (!uemis_get_answer(mountpath, "getDeviceData", 1, 0, &result))
 		goto bail;
 	/* param_buff[0] is still valid */
-	if (! uemis_get_answer(mountpath, "initSession", 1, 6, &result))
+	if (!uemis_get_answer(mountpath, "initSession", 1, 6, &result))
 		goto bail;
-	uemis_info(translate("gettextFromC","Start download"));
-	if (! uemis_get_answer(mountpath, "processSync", 0, 2, &result))
+	uemis_info(translate("gettextFromC", "Start download"));
+	if (!uemis_get_answer(mountpath, "processSync", 0, 2, &result))
 		goto bail;
 	/* before starting the long download, check if user pressed cancel */
 	if (import_thread_cancelled)
@@ -806,19 +848,30 @@ const char *do_uemis_import(const char *mountpath, short force_download)
 		newmax = strdup("0");
 	start = atoi(newmax);
 	for (;;) {
+#if UEMIS_DEBUG & 4
+		fprintf(debugfile, "d_u_i inner loop start %d end %d newmax %s\n", start, end, newmax);
+#endif
 		param_buff[2] = newmax;
 		param_buff[3] = 0;
 		success = uemis_get_answer(mountpath, "getDivelogs", 3, 0, &result);
 		/* process the buffer we have assembled */
 		if (mbuf)
-			process_raw_buffer(deviceidnr, mbuf, &newmax, keep_number, NULL);
+			if (!process_raw_buffer(deviceidnr, mbuf, &newmax, keep_number, NULL)) {
+				/* if no dives were downloaded, mark end appropriately */
+				if (end == -2)
+					end = start - 1;
+				success = false;
+			}
 		if (once) {
 			char *t = first_object_id_val(mbuf);
 			if (t && atoi(t) > start)
 				start = atoi(t);
 			free(t);
-			once = FALSE;
+			once = false;
 		}
+#if UEMIS_DEBUG & 4
+		fprintf(debugfile, "d_u_i after download and parse start %d end %d newmax %s\n", start, end, newmax);
+#endif
 		/* if the user clicked cancel, exit gracefully */
 		if (import_thread_cancelled)
 			goto bail;
@@ -827,15 +880,17 @@ const char *do_uemis_import(const char *mountpath, short force_download)
 			break;
 		/* finally, if the memory is getting too full, maybe we better stop, too */
 		if (progress_bar_fraction > 0.85) {
-			result = translate("gettextFromC",ERR_FS_ALMOST_FULL);
+			result = translate("gettextFromC", ERR_FS_ALMOST_FULL);
 			break;
 		}
 		/* clean up mbuf */
 		endptr = strstr(mbuf, "{{{");
 		if (endptr)
 			*(endptr + 2) = '\0';
+		/* last object_id we parsed */
+		sscanf(newmax, "%d", &end);
 	}
-	if (sscanf(newmax, "%d", &end) != 1)
+	if (end == -2 && sscanf(newmax, "%d", &end) != 1)
 		end = start;
 #if UEMIS_DEBUG & 2
 	fprintf(debugfile, "done: read from object_id %d to %d\n", start, end);
@@ -856,20 +911,29 @@ const char *do_uemis_import(const char *mountpath, short force_download)
 		success = uemis_get_answer(mountpath, "getDive", 3, 0, &result);
 		if (mbuf) {
 			int divenr;
-			process_raw_buffer(deviceidnr, mbuf, &newmax, FALSE, &divenr);
-			if (divenr > -1 && divenr != i) {
-				offset = i - divenr;
+			(void)process_raw_buffer(deviceidnr, mbuf, &newmax, false, &divenr);
 #if UEMIS_DEBUG & 2
-				fprintf(debugfile, "got dive %d -> trying again with offset %d\n", divenr, offset);
+			fprintf(debugfile, "got dive %d, looking for dive %d\n", divenr, i);
+#endif
+			if (divenr != i) {
+				if (divenr == -1) {
+					offset--;
+				} else {
+					offset += i - divenr;
+				}
+#if UEMIS_DEBUG & 2
+				fprintf(debugfile, " -> trying again with offset %d\n", offset);
 #endif
 				i = start - 1;
+				if (i + offset < 0)
+					break;
 				continue;
 			}
 		}
 		if (!success || import_thread_cancelled)
 			break;
 	}
-	success = TRUE;
+	success = true;
 	for (i = 0; i <= nr_divespots; i++) {
 		char divespotnr[10];
 		snprintf(divespotnr, sizeof(divespotnr), "%d", i);
@@ -882,14 +946,13 @@ const char *do_uemis_import(const char *mountpath, short force_download)
 			parse_divespot(mbuf);
 	}
 bail:
-	(void) uemis_get_answer(mountpath, "terminateSync", 0, 3, &result);
-	if (! strcmp(param_buff[0], "error")) {
-		if (! strcmp(param_buff[2],"Out of Memory"))
-			result = translate("gettextFromC",ERR_FS_FULL);
+	(void)uemis_get_answer(mountpath, "terminateSync", 0, 3, &result);
+	if (!strcmp(param_buff[0], "error")) {
+		if (!strcmp(param_buff[2], "Out of Memory"))
+			result = translate("gettextFromC", ERR_FS_FULL);
 		else
 			result = param_buff[2];
 	}
 	free(deviceid);
 	return result;
 }
-
